import java.util.Properties;
import java.util.Random;


import utility.Utils;
import solver.Algorithm;
import problem.Problem;
import solver.AdaptiveAlgorithm;

import utility.FileHandler;
import utility.PropertyReader;
import utility.reporting.LearningReporting;
import utility.reporting.Reporting;

public class Main{
    private static Properties props;
    private static Algorithm algorithm;
    private static Problem problem;
    private static Reporting reporting;
    private static int trials;
    
    /**
     * @param args
     * @throws Exception 
     */
    public static void main(String[] args) throws Exception{
	props = PropertyReader.loadProps();
        trials = Integer.parseInt(props.getProperty("runs"));
        String randseed = props.getProperty("rand_seed");
        if (randseed.equals("clock")) Utils.random = new Random(System.currentTimeMillis());
	else Utils.random = new Random(Integer.parseInt(randseed));
        initialiseProblem();
	batchRun();
    }

    /**
     * @throws Exception
     */
    public static void batchRun()throws Exception{
	if(props.getProperty("learning").equals("yes")){
            LearningReporting learningReporting = new LearningReporting(trials, props.getProperty("learning_type"));
            for (int i = 0; i < trials; i++){
                reporting.restart();
                AdaptiveAlgorithm algo = new AdaptiveAlgorithm(props, learningReporting);
                algo.initialise(problem);
                algo.run(reporting);
                reporting.reportRun();
            }
        }else{              
            for (int i = 0; i < trials; i++){
                reporting.restart();
		algorithm = (Algorithm) Class.forName(props.getProperty("algo_type")).newInstance();
                algorithm.initialise(props, problem);
                algorithm.run(reporting);
		reporting.reportRun();
	    }           
        }
        reporting.reportBatch();
    }

    private static void initialiseProblem(){      
        try{
         reporting = (Reporting) Class.forName(props.getProperty("reporting")).newInstance();
         reporting.initialise(trials, props.getProperty("algo_type"));
         problem = (Problem) Class.forName(props.getProperty("problem")).newInstance();
        }catch (Exception ex){
            System.out.println(ex.toString());
        }
        String problemType = props.getProperty("problem");
        if(props.getProperty("problem_file")!=null){
            String fileName = props.getProperty("problem_file");
            FileHandler fh = new FileHandler();
            if(problemType.equals("problem.QAP.ConstrainedQAPProblem"))
                problem = fh.getConstrainedProblemFromFile(fileName);
            else if(props.getProperty("problem").equals("problem.QAP.SOQAPProblem"))
                problem = fh.getProblemFromFile(fileName);
            else if(props.getProperty("problem").equals("problem.QAP.MOQAPProblem"))
                problem = fh.getMOProblemFromFile(fileName);
        }      
        problem.initialise(props);
    }
    
    public static void createNew(){
	String fileName = props.getProperty("problem_file");
	FileHandler fh = new FileHandler();
        int cardinality = Integer.parseInt(props.getProperty("problem_file"));
	fh.writeQAPProblem(fileName, cardinality);
    }   
}
