package comparator.constraints;

import java.util.List;
import java.util.Iterator;
import java.util.Properties;

import problem.Problem;
import solution.QAP.ConstrainedQAPSolution;

import solution.Solution;

/**
 * Self-adaptive Fitness Formulation for Constrained Optimisation
 * 2003
 * Farmani & Wright
 */
public class FarmaniWrightComparator extends ConstrainedGenericComparator
{
    Solution best = null;

    @Override
    public void prepare(List<Solution> population)
    {
	prepared = true;
	Solution bestIndividual = null;
	double bestCurrentFitness = 0;
	Solution bestIndividualNoFeasibles = null;
	int smallestViolation = Integer.MAX_VALUE;
	feasibleSolutionCount = 0;
	for (Iterator iterator = population.iterator(); iterator.hasNext();)
	{
	    Solution solution = (Solution) iterator.next();
	    if (solution.isValid())
	    {
		feasibleSolutionCount++;
                if(optimisation.equals("maximisation")){
                    if (solution.getCurrentFitness() > bestCurrentFitness)
                    {
                        bestCurrentFitness = solution.getCurrentFitness();
                        bestIndividual = solution;
                    }
                    else
                    {
                        if (((ConstrainedQAPSolution)solution).getConstraintViolation() < smallestViolation)
                        {
                            smallestViolation = ((ConstrainedQAPSolution)solution).getConstraintViolation();
                            bestIndividualNoFeasibles = solution;
                        }
                    }
                } else{
                    if (solution.getCurrentFitness() < bestCurrentFitness)
                    {
                        bestCurrentFitness = solution.getCurrentFitness();
                        bestIndividual = solution;
                    }
                    else
                    {
                        if (((ConstrainedQAPSolution)solution).getConstraintViolation() < smallestViolation)
                        {
                            smallestViolation = ((ConstrainedQAPSolution)solution).getConstraintViolation();
                            bestIndividualNoFeasibles = solution;
                        }
                    }



                }
            }
	}
	if (bestIndividual != null)
	    this.best = bestIndividual;
	else
	    this.best = bestIndividualNoFeasibles;
    }
    
    @Override
    public int compare(Solution sol1, Solution sol2)
    {
	if (!prepared)
	{
	    System.err.println("Comparator unprepared!");
	    System.exit(1);
	}

	sol1.setPenaltyFitness(sol1.getCurrentFitness());
	sol2.setPenaltyFitness(sol2.getCurrentFitness());
        if(optimisation.equals("maximisation")){
            if (sol1.getCurrentFitness() > sol2.getCurrentFitness())
                return -1;
            else if(sol1.getCurrentFitness() < sol2.getCurrentFitness())
                return 1;
        }else{
            if (sol1.getCurrentFitness() > sol2.getCurrentFitness())
                return 1;
            else if(sol1.getCurrentFitness() < sol2.getCurrentFitness())
                return -1;
        }
        return 0;
    }


}
