package comparator.constraints;

import java.util.List;
import java.util.Iterator;
import java.util.Properties;

import problem.Problem;
import solution.QAP.ConstrainedQAPSolution;
import problem.QAP.ConstrainedQAPProblem;

import solution.Solution;

public class TessemaYenComparator extends ConstrainedGenericComparator
{
    private double maxFitness;
    private double minFitness;
    private ConstrainedQAPProblem problem;
    private int popSize;

    public TessemaYenComparator(Problem p) {
        problem = (ConstrainedQAPProblem) p;
    }
   
    /*Tessema - Yen: A Self Adaptive Penalty Function Based Algorithm for Constrained Optimisation */

    @Override
    public int compare(Solution sol1, Solution sol2)
    {
	if (!prepared)
	{
	    System.err.println("Comparator unprepared!");
	    System.exit(1);
	}
	double sol1Fitness = (maxFitness - sol1.getCurrentFitness())/(maxFitness-minFitness);
	double sol2Fitness = (maxFitness - sol2.getCurrentFitness())/(maxFitness-minFitness);
	//System.out.println("Fitness 1 "+sol1Fitness);
	//System.out.println("Fitness 2 "+sol2Fitness);
	double violation1 = (double)((ConstrainedQAPSolution)sol1).getConstraintViolation()/problem.getMaxConstraintViolation();
	double violation2 = (double)((ConstrainedQAPSolution)sol2).getConstraintViolation()/problem.getMaxConstraintViolation();
	
	double distance1 ;
	double distance2 ;
	
	double feasibleRatio = (double) this.feasibleSolutionCount/popSize;
	//System.out.println("Feasible ratio "+feasibleRatio);
	
	double p1, p2;
	double y1 = 0, y2 = 0;
	
	if (!sol1.isValid())
	    y1 = sol1Fitness;
	if (!sol2.isValid())
	    y2 = sol2Fitness;
	
	if (this.feasibleSolutionCount == 0 )
	{
	    distance1 = 0;
	    distance2 = 0;
	    
	    p1 = feasibleRatio*y1;
	    p2 = feasibleRatio*y2;
	}
	else
	{
	    distance1 = Math.sqrt( Math.pow(sol1Fitness, 2)+ Math.pow(violation1, 2));
	    distance2 = Math.sqrt( Math.pow(sol2Fitness, 2)+ Math.pow(violation2, 2));
	    p1 = (1-feasibleRatio)*violation1 + feasibleRatio*y1;
	    p2 = (1-feasibleRatio)*violation2 + feasibleRatio*y2;
	    
	}
	sol1.setPenaltyFitness(distance1 + p1);
	sol2.setPenaltyFitness(distance2 + p2);
	if ( sol1.getPenaltyFitness() > sol2.getPenaltyFitness())
	    return -1;
	else if(sol1.getPenaltyFitness() < sol2.getPenaltyFitness())
	    return 1;
	return 0;
    }

    @Override
    public void prepare(List<Solution> population)
    {
        popSize = population.size();
	maxFitness = 0; 
	minFitness = Double.MAX_VALUE;
	feasibleSolutionCount = 0;
	for (Iterator<Solution> iterator = population.iterator(); iterator.hasNext();)
	{
	    Solution sol = iterator.next();
	    if (sol.getCurrentFitness() > maxFitness)
		maxFitness = sol.getCurrentFitness();
	    if (sol.getCurrentFitness() < minFitness)
		minFitness = sol.getCurrentFitness();
	    if (sol.isValid())
		feasibleSolutionCount++;
	    
	}
	prepared = true;
	
    }
    
   
    
   
}
