package comparator.multiObjective;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Properties;

import problem.Problem;
import solution.MOSolution;
import solution.Solution;

public class RankingComparator extends GenericMOComparator
{
    private int noObjectives;
    private ObjectiveComparator[] comparators;
    ArrayList<Solution> objectiveList;
    
    public int compare(MOSolution arg0, MOSolution arg1)
    {
	if (!prepared)
	{
	    System.err.println("Comparator unprepared!");
	    System.exit(1);
	}
	if ( arg0.getRanking().getAverageRanking() > arg1.getRanking().getAverageRanking())
	    return 1;
	else if(arg0.getRanking().getAverageRanking() < arg1.getRanking().getAverageRanking())
	    return -1;
	return 0;
    }

    @Override
    public void initialise(Properties props, Problem problem)
    {
	noObjectives = problem.getObjectives();
	objectiveList = new ArrayList<Solution>(noObjectives);
	comparators = new ObjectiveComparator[noObjectives];
	for (int i = 0; i < comparators.length; i++)
	{
	    comparators[i] = new ObjectiveComparator(i);
	}
    }

    @Override
    public void prepare(List<Solution> population)
    {
	objectiveList.clear();
	objectiveList.addAll(population);
	
	for (int i = 0; i < noObjectives; i++)
	{
	    Collections.sort(objectiveList, comparators[i]);
	    for (int j = 0; j < objectiveList.size(); j++)
	    {
		   
		((MOSolution)objectiveList.get(j)).getRanking().setRankingForObjective(i, j+1);
	//	System.out.println("Value "+objectiveList.get(j).getFitnessForObjective(i)+" gets rank "+(j+1));
	    }
	}
	prepared = true;
    }
    
    
    class ObjectiveComparator implements Comparator<Solution>
    {
        private int objectiveToCompare = 0;
        
        public ObjectiveComparator(int objective)
        {
    		objectiveToCompare = objective;
        }

        @Override
        public int compare(Solution arg0, Solution arg1)
        {
    	if ( ((MOSolution)arg0).getFitnessForObjective(objectiveToCompare) > ((MOSolution)arg1).getFitnessForObjective(objectiveToCompare))
    	    return 1;
    	else if(((MOSolution)arg0).getFitnessForObjective(objectiveToCompare) < ((MOSolution)arg1).getFitnessForObjective(objectiveToCompare))
    	    return -1;
    	return 0;
        }
  

    }
}
