
package learning;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import solution.Solution;
import utility.CommonFunctions;
import utility.reporting.LearningReporting;

/**
 *
 * @author Alda
 */
public class Experiment extends LearningAlgorithm{
    private double percentile = 0.5;
    private double threshold;
    
    @Override
    protected void preComputeStats(List<Solution> population, int iCompletedIteration, LearningReporting reporting){
        threshold = getPercentile(population, percentile);
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for(Bin bin : param.getListBins()){
                List<Double> binPerfHistory=bin.getBinPerformance().getPerformanceHistory(performanceMetric.toString()).get(iCompletedIteration);
                int inAboveAvg= 0;
                if(binPerfHistory!=null)
                    inAboveAvg = CommonFunctions.getOutperformerCount(binPerfHistory, threshold);
                int inTotalUsage = 0;
                try{
                   inTotalUsage= bin.getBinPerformance().getUsageHistory().get(iCompletedIteration).size();
                }catch(Exception ex){
                    System.out.println(ex.toString());
                }
                double dSuccessProbability = (inTotalUsage > 0)? ((double)inAboveAvg/(double)inTotalUsage) :0.01;
                bin.getBinPerformance().setSuccessRateHistory(iCompletedIteration,dSuccessProbability);
                reporting.report("Parameter,"+param.getName()+","+bin.getAlias()+",Performance,"+dSuccessProbability);
            }
        }
    }

    public double getPercentile(List<Solution> population, double dPercentile){
            ArrayList<Double> lstSorted = new ArrayList<Double>();
            for(Solution el : population){
                lstSorted.add((Double)el.getCurrentFitness());
            }
            Collections.sort(lstSorted);
            int iIndex = (int) ( (1- dPercentile) * lstSorted.size());
            return lstSorted.get(iIndex);
    }


     /**
     * Printing of results
     */

    @Override
    public void doFinalTasks(List<Solution> population, LearningReporting reporting) {
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for(Bin bin : param.getListBins()){
                List<Double> performanceHistory = bin.getBinPerformance().getSuccessRateHistory();
                for (int i=0; i < performanceHistory.size(); i++){
                    String printRes = "Parameter," + param.getName()+",Bin,"+bin.getAlias()+",Itr,"+i+",SucessRate,"+performanceHistory.get(i);
                    reporting.report(printRes);
                }
            }
        }
    }

}
