package learning;

import comparator.GenericComparator;
import java.util.*;
import java.lang.Object.*;
import solution.Solution;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import learning.parameters.ValueParameter;
import utility.reporting.LearningReporting;


public class LearningAlgorithm {
    protected Map<String, Parameter> lstParametersToControl;
    protected GenericComparator performanceMetric;
    protected Properties properties;
    protected int iMinIterations = 3;

    public void initialise(Properties props) {
        properties = props;
        try{
           performanceMetric = (GenericComparator) Class.forName(properties.getProperty("learning_metric")).newInstance();
        }catch(Exception ex){
            System.out.print(ex.toString());
        }
    }

    /**
     * 
     * @param properties
     */
    public void initialiseParameters(Properties properties) {
        lstParametersToControl = new HashMap<String, Parameter>();
        String valueParameters = (String)properties.get("value_parameters");
        String[] vParams = valueParameters.split(",");     
        for(int i=0; i<vParams.length;i++){
            String name = vParams[i++];
            Parameter newParameter = new ValueParameter(name,Integer.parseInt(vParams[i++]),Double.parseDouble(vParams[i++]),Double.parseDouble(vParams[i]));
            newParameter.initialiseBins();
            lstParametersToControl.put(name,newParameter);
        }
    }
   
    public void initialiseRecordingStructures() {}

    /**
     * This will be called on completion of each iteration.
     * @param population
     * @param iCurrentIteration
     */
    public void onIterationComplete(List<Solution> population, int iCurrentIteration, LearningReporting reporting){
        for(Solution sol : population){
            for (Parameter prm: sol.getParameterMap().values()){
                Bin curSolutionBin = prm.getCurrentBin();
                Parameter controlledParam=lstParametersToControl.get(prm.getName().toString());
                Bin controlledBin= controlledParam.getBin(curSolutionBin.getBinIndex());
                controlledBin.getBinPerformance().setUsageHistory(iCurrentIteration, curSolutionBin.getCurrentValue());
                controlledBin.getBinPerformance().setPerformanceHistory(performanceMetric.toString(), iCurrentIteration, sol.getCurrentFitness());
                controlledBin.getBinPerformance().setPerformanceChangeHistory(performanceMetric.toString(), iCurrentIteration, sol.getFitnessChange());
            }
        }
        preComputeStats(population, iCurrentIteration, reporting);
        prepareInstancesForNextIteration(population, iCurrentIteration); 
          
    }
  
    public void doFinalTasks(List<Solution> population, LearningReporting reporting) { }

	
    public void prepareInstancesForNextIteration(List<Solution> population, int iCurrentIteration){}

    protected void preComputeStats(List<Solution> population, int iCurrentIteration, LearningReporting reporting){}
    
    public void assignRandomParametersToSolutions(List<Solution> population){
        for (Solution sol : population) {
            HashMap<Object, Parameter> solutionParametersMap = new HashMap<Object, Parameter>();
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter prm = (Parameter)paramEntry.getValue();
                Parameter newParam = prm.clone();
                newParam.initialiseRandomly();
                solutionParametersMap.put(newParam.getName(), newParam);
            }
            sol.setParameterMap(solutionParametersMap);
        }
    }
	
}
