package learning;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import learning.clustering.EntropyKMeansClustering;
import learning.clustering.GenericClustering;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import learning.parameters.ValueBin;
import learning.parameters.ValueParameter;
import solution.Solution;
import utility.CommonFunctions;
import utility.reporting.LearningReporting;

/**
 * @author Alda
 */
public class ParameterRangeClusterer extends LearningAlgorithm{
    protected GenericClustering clustering;
    @Override
    protected void preComputeStats(List<Solution> population, int iCompletedIteration, LearningReporting reporting){
        super.preComputeStats(population, iCompletedIteration, reporting);

        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter)paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            List<Double> binsPerformance = new LinkedList<Double>();
            List<Double> binsUsage = new LinkedList<Double>();

            for( Bin xbin : lstBins ){
                List<Double> performance = xbin.getBinPerformance().getDiffPerformanceHistory(performanceMetric.toString()).get(iCompletedIteration);
                for(Object per : performance)
                    binsPerformance.add((Double)per);
                List<Object> usage = xbin.getBinPerformance().getUsageHistory().get(iCompletedIteration);
                if(usage!=null){
                    for(Object us: usage)
                        binsUsage.add((Double) us);
                }
            }
            try {
                clustering = (GenericClustering) Class.forName(properties.getProperty("clustering_type")).newInstance();
                clustering.initialise(2, 2, binsPerformance, binsUsage);
                clustering.classify();
                clustering.cluster();
                // System.out.println("cutPoint:"+((EntropyKMeansClustering)clustering).getCutPoint()[0]);
                //System.out.println(clustering.toString());
            } catch (Exception ex) {
                System.out.println(this.toString()+ ex.toString());
            }
            int[] clusters = clustering.getMembershipMatrix().getClusters();
            double[] cutPoints = ((EntropyKMeansClustering)clustering).getCutPoint();
            param.setCutPoints(cutPoints);
            int index =0;
            for(Bin xbin : lstBins ){
                List<Double> performance = xbin.getBinPerformance().getDiffPerformanceHistory(performanceMetric.toString()).get(iCompletedIteration);
                int i;
                for(i=0; i<performance.size();i++)
                    xbin.getBinPerformance().setClassifierHistory(iCompletedIteration, clusters[i+index]);
                index+=i;
            }
        }
        adaptParameterBins(iCompletedIteration, reporting);
    }

    public void adaptParameterBins(int iCompletedIteration, LearningReporting reporting){
         for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter)paramEntry.getValue();          
            List<Bin> newLstBins = new LinkedList<Bin>();
            double[] cutpoints = param.getCutPoints();
            double newMin = ((ValueParameter)param).getMinValue();
            List<Bin> lstBins = param.getListBins();
            int index = 0;
            for(double point: cutpoints){
                double newMax = point;
                Bin newBin = new ValueBin(newMin, newMax,index++);
                for(int i=0; i<lstBins.size();i++){
                    if(point>((ValueBin)lstBins.get(i)).getMin()&&point<((ValueBin)lstBins.get(i)).getMax()){
                        newBin.setBinPerformance(lstBins.get(i).getBinPerformance());
                        break;
                    }
                }
                newLstBins.add(newBin);
                newMin=point;
            }
            Bin lastBin = lstBins.get(lstBins.size()-1);
            double newMax = ((ValueBin)lastBin).getMax();
            Bin newBin = new ValueBin(newMin, newMax, index);
            newBin.setBinPerformance(lastBin.getBinPerformance());
            newLstBins.add(newBin);
            param.setListBins(newLstBins);
        }
    }

    /**
     * Populate New parameters for Executor Instances.
     * Use Previously calculated Success rates and select the bin from a discrete distribution
     * weighted from the success rates.
     * @param List<Solution> population
     * @param iCompletedIteration LastCompletedIteration
     */
    @Override
    public void prepareInstancesForNextIteration(List<Solution> population,int iCompletedIteration){
        for(Solution sol:population){
            HashMap<Object, Parameter> solParameterMap = new HashMap<Object, Parameter>();
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter) paramEntry.getValue();
                Parameter newParameter = param.clone();

                List<Bin> lstBins = newParameter.getListBins();
                double[] adProbValues = new double[lstBins.size()];
                for( int i=0; i<lstBins.size(); i++)
                    adProbValues[i] = CommonFunctions.getAverage(lstBins.get(i).getBinPerformance().getDiffPerformanceHistory(performanceMetric.toString()).get(iCompletedIteration))*(((ValueBin)lstBins.get(i)).getMax()-((ValueBin)lstBins.get(i)).getMin());

                int iBinIndex = CommonFunctions.doRoulette(adProbValues);
                Bin newBin = lstBins.get(iBinIndex);
                newBin.setCurrentValueRandomly();
                newParameter.setCurrentBin(newBin);
                solParameterMap.put(newParameter.getName(), newParameter);
            }
            sol.setParameterMap(solParameterMap);
        }
    }
    
}
