
package learning;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import learning.parameters.ValueBin;
import solution.Solution;
import utility.reporting.LearningReporting;

public class ParameterRangeLearner extends SuccessBasedLearner{

    @Override
    protected void preComputeStats(List<Solution> population, int iCompletedIteration, LearningReporting reporting){
        super.preComputeStats(population, iCompletedIteration, reporting);
        adaptParameterBins(population, iCompletedIteration, reporting);
    }

    public void adaptParameterBins(List<Solution> population, int iCompletedIteration, LearningReporting reporting){
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            double bestBinPerformance = (lstBins.get(1)).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
            double worstBinPerformance = bestBinPerformance;
            for( Bin bin : lstBins ){
                double binPerformance = (bin).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
                if (binPerformance>bestBinPerformance){
                    bestBinPerformance = binPerformance;
                }
                if (binPerformance<worstBinPerformance){
                    worstBinPerformance = binPerformance;
                }
                if(binPerformance==0.0){
                    bin.getBinPerformance().setSuccessRateHistory(iCompletedIteration, 0.01);
                }
            }
            List<Bin> newLstBins = new LinkedList<Bin>();
            for( Bin xbin : lstBins ){
                ValueBin bin = (ValueBin)xbin;
                double binPerformance = bin.getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
                double oldMax = bin.getMax();
                double oldMin = bin.getMin();
                if(binPerformance>=bestBinPerformance){
                    double intervalLength = ((Double)oldMax-(Double)oldMin)/2;
                    double newMax = (Double)oldMin+intervalLength / 2;
                    double newMin = (Double)oldMin+intervalLength/2+0.0001;

                    Bin newBin1 = bin.clone();
                    ((ValueBin)newBin1).setMax(newMax);
                    Bin newBin2 = bin.clone();
                    ((ValueBin)newBin2).setMin(newMin);
                    newLstBins.add(newBin1);
                    newLstBins.add(newBin2);
                }
                else if(binPerformance<=worstBinPerformance){
                    if(newLstBins.size()>0||bin.getMax()==((ValueBin)param.getListBins().get(lstBins.size()-1)).getMax()){
                        ((ValueBin)newLstBins.get(newLstBins.size()-1)).setMax(oldMax);
                    }
                    else{
                        Bin newBin = ((ValueBin)bin).clone();
                        newLstBins.add(newBin);
                    }
                }
                else{
                    Bin newBin = bin.clone();
                    newLstBins.add(newBin);
                }
            }
            param.setListBins(newLstBins);
        }
    }

}
