package learning;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import solution.Solution;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import utility.CommonFunctions;
import utility.reporting.LearningReporting;

public class SuccessBasedLearner extends LearningAlgorithm{
    private double percentile = 0.5;
    private double threshold;
    @Override
    protected void preComputeStats(List<Solution> population, int iCurrentIteration, LearningReporting reporting){
        threshold = getPercentile(population, percentile);
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for(Bin bin : param.getListBins()){
                List<Double> binPerfHistory=bin.getBinPerformance().getPerformanceHistory(performanceMetric.toString()).get(iCurrentIteration);
                int inAboveAvg= 0;
                if(binPerfHistory!=null)
                    inAboveAvg = CommonFunctions.getOutperformerCount(binPerfHistory, threshold);
                int inTotalUsage = bin.getBinPerformance().getUsageHistory().get(iCurrentIteration).size();
                double dSuccessProbability = (inTotalUsage > 0)? ((double)inAboveAvg/(double)inTotalUsage) :0.01;
                (bin).getBinPerformance().setSuccessRateHistory(iCurrentIteration,dSuccessProbability);
            }
        }
    }

    public double getPercentile(List<Solution> population, double dPercentile){
            ArrayList<Double> lstSorted = new ArrayList<Double>();
            for(Solution el : population){
                lstSorted.add((Double)el.getCurrentFitness());
            }
            Collections.sort(lstSorted);
            int iIndex = (int) ( (1- dPercentile) * lstSorted.size());
            return lstSorted.get(iIndex);
    }


     /**
     * Printing of results
     */
    
    @Override
    public void doFinalTasks(List<Solution> population, LearningReporting reporting) {
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for(Bin bin : param.getListBins()){
                List<Double> performanceHistory = bin.getBinPerformance().getSuccessRateHistory();
                for (int i=0; i < performanceHistory.size(); i++){
                    String printRes = "Parameter," + param.getName()+",Bin,"+bin.getAlias()+",Itr,"+i+",SucessRate,"+performanceHistory.get(i);
                    reporting.report(printRes);
                }
            }
        }
    }

    /**
     * Populate New parameters for Executor Instances.
     * Use Previously calculated Success rates and select the bin from a discrete distribution
     * weighted from the success rates.
     * @param List<Solution> population 
     * @param iCompletedIteration LastCompletedIteration
     */
    @Override
    public void prepareInstancesForNextIteration(List<Solution> population, int iCompletedIteration){
        for(Solution sol:population){
            HashMap<Object, Parameter> solParameterMap = new HashMap<Object, Parameter>();
            for(Map.Entry paramEntry : sol.getParameterMap().entrySet()){
                Parameter param = (Parameter) paramEntry.getValue();
                Parameter newParameter = param.clone();
                List<Bin> lstBins = param.getListBins();
                double[] adProbValues = new double[lstBins.size()];
                for( int i=0; i<lstBins.size(); i++)
                    adProbValues[i] = (lstBins.get(i)).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);//(lstBins.get(i)).getSuccessProbHistory(iCompletedIteration-1);
                int iBinIndex = CommonFunctions.doRoulette(adProbValues);
                Bin newCurrentBin = newParameter.getBin(iBinIndex);
                newCurrentBin.setCurrentValueRandomly();
                newParameter.setCurrentBin(newCurrentBin);
                solParameterMap.put(newParameter.getName(), newParameter);
            }
            sol.setParameterMap(solParameterMap);
        }
    }
}
