package learning.benchmarks;


import learning.LearningAlgorithm;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import solution.Solution;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import utility.reporting.LearningReporting;

public class DMABLearner extends LearningAlgorithm {

    private Map<Object,BanditRestart> opSelector;

    protected double scalingC = 0.5, gamma = 100, decay = 0.5;
    protected	int W = 50;
    protected boolean user = false;
    protected int typeCAS = 0;
    protected boolean normalized = false;
    protected int timeTotal = 4;


    @Override
    public void initialiseRecordingStructures() {
            super.initialiseRecordingStructures();
            initializeLearningStructures();
    }

    protected void initializeLearningStructures() {
        CreditAssignment crA;
        opSelector = new HashMap<Object,BanditRestart>();
         for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter)paramEntry.getValue();
                BanditRestart opS= new BanditRestart(param.getLevels(),  user, scalingC, 0, W);
                opS.credit = new LinkedList<CreditAssignment>();
                List<Bin> lstBins = param.getListBins();
                int inBins = lstBins.size();
                for(int i=0; i<inBins;i++){

                        crA = new CreditAssignment(W, true, typeCAS, normalized);
                        opS.credit.add(crA);
                }
                opSelector.put(param.getName(),opS);
        }

    }
    @Override
    protected void preComputeStats(List<Solution> population, int iCompletedIteration, LearningReporting reporting){
        int iMinimumIterations =2;
        if( iCompletedIteration < iMinimumIterations){
            return;
        }

         for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter)paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            int inBins = lstBins.size();
            for( int i=0; i<inBins; i++){
                double reward = lstBins.get(i).getBinPerformance().getAveragePerformanceHistory(performanceMetric.toString(),iCompletedIteration)- lstBins.get(i).getBinPerformance().getAveragePerformanceHistory(performanceMetric.toString(),iCompletedIteration-1);
                opSelector.get(param.getName()).credit.get(i).addReward(reward);
            }
        }

    }
    @Override
    public void prepareInstancesForNextIteration(List<Solution> population,int iCompletedIteration){
        for(Solution sol: population){
            for( Parameter param : sol.getParameterMap().values()){
                List<Bin> lstBins = param.getListBins();
                int iBinIndex = opSelector.get(param.getName()).selectOperator(timeTotal);
                lstBins.get(iBinIndex).setCurrentValueRandomly();
                param.setCurrentBin(lstBins.get(iBinIndex));
            }
        }

    }
}