package learning.clustering;

/**
 *
 * @author Alda
 */

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import utility.Utils;

public class EntropyKMeansClustering extends GenericClustering {

    private LinkedList<Double> cutPoints;

    @Override
    public void initialise(int iClasses, int iClusters, List<Double> values, List<Double> names){
        super.initialise(iClasses, iClusters, values, names);
        cutPoints = new LinkedList<Double>();
    }
     public double[] getCutPoint(){
        double[] cPoints = new double[cutPoints.size()];
        for(int i=0; i<cutPoints.size();i++){
            cPoints[i]= cutPoints.get(i);
        }
        return cPoints;
    }
    /**
     * returns an array of integers, the ith elements represents the cluster
     * index of the ith pattern
     */
    @Override
    public void cluster(){
         double totalEntropy=0.0;
        double minEntropy = 100.0;
        cutPoints.add(Double.parseDouble(patterns[0].getName()));
        int pLength = patterns.length;
        for(int i=2; i<pLength;i++){
            double ce1 = ((double)i/(double)pLength)*classEntropy(0, i);
            double ce2 = ((double)(pLength-i)/(double)pLength)*classEntropy(i, pLength-1);
            totalEntropy= ce1+ce2;
            if(totalEntropy<minEntropy){
                minEntropy=totalEntropy;
                cutPoints.set(0,Double.parseDouble(patterns[i].getName()));
            }
        }
    }

    public void dynamicCluster(){
        double totalEntropy=0.0;
        double minEntropy = 100.0;
        int cIndex = 0;
        cutPoints.add(Double.parseDouble(patterns[cIndex].getName()));
        int pLength = patterns.length;
        while(true){
            for(int i=2; i<pLength;i++){
                double ce1 = ((double)i/(double)pLength)*classEntropy(0, i);
                double ce2 = ((double)(pLength-i)/(double)pLength)*classEntropy(i, pLength-1);
                totalEntropy= ce1+ce2;
                if(totalEntropy<minEntropy){
                    minEntropy=totalEntropy;
                    cutPoints.set(cIndex,Double.parseDouble(patterns[i].getName()));
                }
            }
            cIndex++;
        }
    }

    

    @Override
    public void classify(){
         Vector[] newZ = new Vector[nClasses];
        for(int i=0; i<nClasses; i++)
            newZ[i] = patterns[Utils.random.nextInt(patterns.length)];
        Vector[] oldZ = new Vector[nClasses];

        for (int m = 0; ; m++) {
            System.arraycopy(newZ, 0, oldZ, 0, nClasses);
            //oldZ = newZ;

            // for each pattern, find the cluster center that is closest
            // to that pattern, and then put that pattern into that cluster
            int[] clusterIndices = new int[patterns.length];
            for (int i = 0; i < patterns.length; i++){
                try{
                    clusterIndices[i] = patterns[i].getNearestPointIndex(newZ);
                }catch(Exception ex){
                    System.out.println(ex.toString()+clusterIndices[i]+patterns[i].toString()+newZ);
                }
            }
            mm.moveAllPatterns(clusterIndices);
            for (int j = 0; j < nClasses; j++) {
                Vector[] vec= getPatternsWithIndexes(mm.getPatternIndicesForCluster(j));
                if(vec!=null && vec.length>0 )
                    newZ[j]= calculateCenter(vec);
            }
            if (Arrays.equals(newZ, oldZ)) break;
        }
    }

    public Vector calculateCenter(Vector... vectors) {
        Vector center = vectors[0].clone();
        for (int i = 1; i < vectors.length; i++) {
            if (vectors[i].getValues().length != center.getValues().length)
                throw new IllegalArgumentException("vector " + i + "'s dimension is not compatible with first vector!");

            for (int j = 0; j < center.getValues().length; j++) 
                center.getValues()[j] += vectors[i].getValues()[j];
            
        }
        for (int i = 0; i < center.getValues().length; i++) 
            center.getValues()[i] = center.getValues()[i] / vectors.length;  
        return center;
    }

}