
package learning.clustering;

/**
 *
 * @author Alda
 */

import java.util.ArrayList;
import java.util.List;
import utility.Utils;

public class Membership {

        private int[] patternClusters;
        
        /**
         * creates a Membership and randomly initialize memberships
         *
         * @param n number of patterns/observations
         * @param k number of clusters/groups
         */
        public Membership(int n, int clusters) {
            patternClusters = new int[n];
            
            // for the first k patterns, assign them to k cluster
            // this way, we guarantee that every cluster has at least one pattern
           
            for (int i = 0; i < clusters; i++) {
                patternClusters[i] = i;
            }
            // for each pattern, randomly choose a cluster for it
            for (int i = clusters; i < n; i++) {
                int clusterIndex = Utils.random.nextInt(clusters);
                patternClusters[i] = clusterIndex;
            }  
        }

        public Membership(int n){
            patternClusters = new int[n];
        }
        /**
         * creates a Membership using the specified patternClusters
         */
        public Membership(int[] newMatrix) {patternClusters = newMatrix;}

        public void movePattern(int patternIndex, int clusterIndex) {
            patternClusters[patternIndex] = clusterIndex;
        }

        public void moveAllPatterns(int[] clusterIndexes) {
            System.arraycopy(clusterIndexes, 0, patternClusters, 0, patternClusters.length);
        }

        /**
         * returns an arrays of length n, where the ith element represents
         * the clusters index of the ith pattern.
         *
         */
        public int[] getClusters() {
           
            return patternClusters;
        }

        public List<Integer> getPatternIndicesForCluster(int cluster) {
            List<Integer> patternIndices = new ArrayList<Integer>();
            for (int i = 0; i < patternClusters.length; i++) {
                if (patternClusters[i] == cluster) {
                    patternIndices.add(i);
                }
            }
            return patternIndices;
        }
}