package learning.forecast;
import java.util.List;
import java.util.Map;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import utility.CommonFunctions;
import utility.forecasting.AutoARIMA;


public class ARIMAForecast extends ForecastBasedLearner{
    
    @Override
    public void forecast(int iCompletedIteration){
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter)paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            for(int i=0; i<lstBins.size(); i++){
                List<Double> adHistory = lstBins.get(i).getBinPerformance().getSuccessRateHistory();
                double [] diffSeries = new double[adHistory.size()];
                diffSeries[0]=0;
                for(int j=1; j<adHistory.get(i); j++){
                    diffSeries[j]=adHistory.get(j)-adHistory.get(j-1);
                }
                int[] adIntIndexSet = CommonFunctions.getIntIndexSet(diffSeries);
                AutoARIMA autoArima = new AutoARIMA(adIntIndexSet,diffSeries);
                try{
                    autoArima.compute(5);
                }catch(Exception ex){
                    System.out.println(ex.toString());
                    return;
                }
                autoArima.forecast(1);
                double[] outlierForecast = autoArima.getForecast();
                double average = CommonFunctions.getAverage(adHistory);
                double predictedPerformance=outlierForecast[0]+average;
                lstBins.get(i).getBinPerformance().setPredictedSuccessRateHistory(iCompletedIteration, predictedPerformance);
            }
        }
    }
}

