
package learning.forecast;

import java.util.List;
import java.util.Map;
import learning.SuccessBasedLearner;
import solution.Solution;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import utility.CommonFunctions;
import utility.reporting.LearningReporting;


public class ForecastBasedLearner extends SuccessBasedLearner{

    /**
     * Populate New parameters for Executor Instances.
     * Create ARIMA model for each parameter and use to forecast next parameter
     * usages
     * @param List<Solution>
     * @param iCompletedIteration LastCompletedIteration
     */
    @Override
    public void prepareInstancesForNextIteration(List<Solution> population,int iCompletedIteration){
        if  ( ( iCompletedIteration ) < iMinIterations){
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter)paramEntry.getValue();
                for( Bin bin : param.getListBins()){
                    bin.getBinPerformance().setPredictedSuccessRateHistory(iCompletedIteration,bin.getBinPerformance().getSuccessRateHistory().get(iCompletedIteration));
                }
            }
        }
        else{
            forecast(iCompletedIteration);
            assignParameterValues(population,iCompletedIteration);

        }
    }

     public void forecast(int iCompletedIteration){}

    public void assignParameterValues(List<Solution> population, int iCompletedIteration){
        for(Solution sol:population){
            for(Map.Entry paramEntry : sol.getParameterMap().entrySet()){
                Parameter param = (Parameter)paramEntry.getValue();
                List<Bin> lstBins = param.getListBins();
                int inBins = lstBins.size();
                double[] adProbValues = new double[inBins];
                Parameter newParameter = param.clone();
                for( int i=0; i<inBins; i++){
                    adProbValues[i]= lstBins.get(i).getBinPerformance().getPredictedSuccessRateHistory().get(iCompletedIteration);
                }
                int iBinIndex = CommonFunctions.doRoulette(adProbValues);
                Bin newCurrentBin = newParameter.getBin(iBinIndex);
                newCurrentBin.setCurrentValueRandomly();
                newParameter.setCurrentBin(newCurrentBin);
                sol.getParameterMap().put(newParameter.getName(), newParameter);
            }
        }
    }
     
    @Override
    public void doFinalTasks(List<Solution> population, LearningReporting reporting) {
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for( Bin bin : param.getListBins()){
                String s ="Parameter," + param.getName() +",Bin,"+bin.getAlias()+",MeanSuccessRate,"+bin.getBinPerformance().getAverageSuccess();
                reporting.report(s);
                for (int i=0; i < bin.getBinPerformance().getSuccessRateHistory().size(); i++){
                    s ="Parameter," + param.getName()+",Bin,"+bin.getAlias()+",Itr,"+i+",SucessRate,"+bin.getBinPerformance().getSuccessRateHistory().get(i)+",PredictedSuccessRate,"+bin.getBinPerformance().getPredictedSuccessRateHistory().get(i);
                    reporting.report(s);
                }
            }
        }

    }
}

