
package learning.forecast;
import learning.parameters.ValueBin;
import learning.ParameterRangeLearner;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import jsc.datastructures.PairedData;
import jsc.regression.PearsonCorrelation;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import solution.Solution;
import utility.CommonFunctions;
import utility.reporting.LearningReporting;


public class InvasiveForecast extends ParameterRangeLearner{

    /**
	 * Populate New parameters for Executor Instances.
	 * @param mapExInstance
	 * @param iCompletedIteration LastCompletedIteration
	 */

    @Override
    public void adaptParameterBins(List<Solution> population, int iCompletedIteration, LearningReporting reporting){
        super.preComputeStats(population,iCompletedIteration, reporting);
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            double bestBinPerformance = 0.0;
            double worstBinPerformance = 1.0;
            for( Bin bin : lstBins ){
                    double binPerformance = bin.getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
                    List<Double> adHist = bin.getBinPerformance().getSuccessRateHistory();
                    double[] adHistory = new double[adHist.size()];
                    for(int i=0; i<adHist.size();i++)
                        adHistory[i]= adHist.get(i);
                    double[] adIndexSet = CommonFunctions.getIndexSet(adHistory);
                    PairedData o_PairedData = new PairedData(adIndexSet, adHistory);
                    PearsonCorrelation o_PearsonCorrelation = null;
                    try {
                        o_PearsonCorrelation = new PearsonCorrelation(o_PairedData);
                    } catch (Exception ex) {
                        System.out.println(ex.toString());
                        return;
                    }

                    double A = o_PearsonCorrelation.getA();
                    double B = o_PearsonCorrelation.getB();
                    double r = o_PearsonCorrelation.getR();

                    double next = A + B * (iCompletedIteration+1);

                    bin.getBinPerformance().setPredictedSuccessRateHistory(iCompletedIteration, next);

                    if (next>bestBinPerformance){
                        bestBinPerformance = next;
                    }
                    if (next<worstBinPerformance){
                        worstBinPerformance = next;
                    }
                    if(binPerformance==0.0){
                        bin.getBinPerformance().setPredictedSuccessRateHistory(iCompletedIteration, 0.01);
                    }

            }

            List<Bin> newLstBins = new LinkedList<Bin>();
            for( Bin bin : lstBins ){
                double binPerformance = (bin).getBinPerformance().getPredictedSuccessRateHistory().get(iCompletedIteration);
                Object oldMax = ((ValueBin)bin).getMax();
                Object oldMin = ((ValueBin)bin).getMin();
                if(binPerformance==bestBinPerformance || binPerformance>bestBinPerformance){
                    if(oldMax instanceof Double){
                        double intervalLength = ((Double)oldMax-(Double)oldMin)/2;
                        double newMax = (Double)oldMin+intervalLength / 2;
                        double newMin = (Double)oldMin+intervalLength/2+0.0001;

                        Bin newBin1 = bin.clone();
                        ((ValueBin)newBin1).setMax(newMax);
                        Bin newBin2 = bin.clone();
                        ((ValueBin)newBin2).setMin(newMin);

                        newLstBins.add(newBin1);
                        newLstBins.add(newBin2);
                    }
                    else if(oldMax instanceof Integer){
                        int intervalLength = ((Integer)oldMax-(Integer)oldMin)/2;
                        int newMax = (Integer)oldMin + intervalLength/2;
                        int newMin = (Integer)oldMin + intervalLength/2 +1;

                        Bin newBin1 = bin.clone();
                        ((ValueBin)newBin1).setMax(newMax);
                        Bin newBin2 = bin.clone();
                        ((ValueBin)newBin2).setMin(newMin);

                        newLstBins.add(newBin1);
                        newLstBins.add(newBin2);

                    }
                }
                else if(binPerformance<0.05){
                    if(newLstBins.size()>0){
                        if(oldMax instanceof Double){
                                ((ValueBin)newLstBins.get(newLstBins.size()-1)).setMax((Double)oldMax);
                        }else{
                                ((ValueBin)newLstBins.get(newLstBins.size()-1)).setMax((Integer)oldMax);
                        }
                    }
                    else{
                        Bin newBin = ((ValueBin)bin).clone();
                        newBin.getBinPerformance().setPredictedSuccessRateHistory(iCompletedIteration, 0.05);
                        newLstBins.add(newBin);
                    }
                }
                else{
                    Bin newBin = ((ValueBin)bin).clone();
                    newLstBins.add(newBin);
                }
            }
            param.setListBins(newLstBins);
        }
    }
}
