package learning.forecast;

import java.util.List;
import java.util.Properties;
import learning.parameters.Bin;
import learning.parameters.Parameter;
import org.apache.commons.math3.filter.DefaultMeasurementModel;
import org.apache.commons.math3.filter.DefaultProcessModel;
import org.apache.commons.math3.filter.KalmanFilter;
import org.apache.commons.math3.filter.MeasurementModel;
import org.apache.commons.math3.filter.ProcessModel;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.random.JDKRandomGenerator;
import org.apache.commons.math3.random.RandomGenerator;


/**
 *
 * @author Alda
 */
public class KalmanFilterLearner extends ForecastBasedLearner {
    private KalmanFilter filter;
    private RealVector pNoise;
    private RealVector mNoise;
    private RealMatrix A;
    private RealVector x;
    private RealMatrix H;
    private double processNoise = 1e-5d;
    
    @Override
     public void initialise(Properties props) {
        super.initialise(props);
        // A = [ 1 ]
        A = new Array2DRowRealMatrix(new double[] { 1d });
        // no control input
        RealMatrix B = null;
        x = new ArrayRealVector(new double[] {1d });
        // H = [ 1 ]
        H = new Array2DRowRealMatrix(new double[] { 1d });
        // Q = [ 0 ]
        RealMatrix Q = new Array2DRowRealMatrix(new double[] { processNoise });
        // R = [ 0 ]
        RealMatrix R = new Array2DRowRealMatrix(new double[] { 0 });
        RealMatrix P0 = new Array2DRowRealMatrix(new double[] { 1d });
        
        ProcessModel pm = new DefaultProcessModel(A, B, Q, x, P0);
        MeasurementModel mm = new DefaultMeasurementModel(H, R);
        filter = new KalmanFilter(pm, mm);
        pNoise = new ArrayRealVector(1);
        mNoise = new ArrayRealVector(1);
                        
    }
   @Override
    public void forecast(int iCompletedIteration){
        for( Parameter param : lstParametersToControl.values()){
            List<Bin> lstBins = param.getListBins();
            RandomGenerator rand = new JDKRandomGenerator();
            for( int i=0; i<lstBins.size(); i++){
                double performance = (Double)lstBins.get(i).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
               
                filter.predict();
                 // simulate the process
                //pNoise.setEntry(0, processNoise * rand.nextGaussian());

                // x = A * x + p_noise
                //x = A.operate(x).add(pNoise);
                mNoise.setEntry(0,performance);
                RealVector z = H.operate(x).add(mNoise);
                filter.correct(z);
                double next = filter.getStateEstimation()[0];
                double predictedPerformance=next;
                (lstBins.get(i)).getBinPerformance().setPredictedSuccessRateHistory(iCompletedIteration, predictedPerformance);
            }         
        }
    }
}
