package learning.parameters;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import utility.CommonFunctions;

/**
 *
 * @author Alda
 */
public class BinPerformance {
    private Map<String, LinkedList<LinkedList<Double>>> performanceHistory;
    private List<List<Object>> usageHistory;
    private List<List<Object>> classifierHistory;
    private List<Double> predictedPerformanceHistory;
    private Map<String, LinkedList<LinkedList<Double>>> diffPerformanceHistory;
    private List<Double> successRateHistory;
    private List<Double> predictedSuccessRateHistory;
    
    public BinPerformance(){     
        predictedPerformanceHistory= new LinkedList<Double>();
        performanceHistory = new HashMap<String, LinkedList<LinkedList<Double>>>();
        diffPerformanceHistory = new HashMap<String, LinkedList<LinkedList<Double>>>();
        successRateHistory = new LinkedList<Double>();
        predictedSuccessRateHistory= new LinkedList<Double>();
        usageHistory = new LinkedList<List<Object>>();
        classifierHistory = new LinkedList<List<Object>>();
    }
    
    @Override
    public BinPerformance clone() {
        BinPerformance newPerformaceStructure = new BinPerformance();  
        newPerformaceStructure.setPredictedPerformanceHistory(predictedPerformanceHistory);
        newPerformaceStructure.setPredictedSuccessRateHistory(predictedSuccessRateHistory);
        newPerformaceStructure.setSuccessRateHistory(successRateHistory);
        newPerformaceStructure.setUsageHistory(usageHistory);
        newPerformaceStructure.setClassifierHistory(classifierHistory);
        for(Map.Entry<String, LinkedList<LinkedList<Double>>> entry : performanceHistory.entrySet()) {
            String key = entry.getKey();
            List<LinkedList<Double>> value = entry.getValue();
            newPerformaceStructure.setPerformanceHistory(key, value);
        }
        for(Map.Entry<String, LinkedList<LinkedList<Double>>> entry : diffPerformanceHistory.entrySet()) {
            String key = entry.getKey();
            List<LinkedList<Double>> value = entry.getValue();
            newPerformaceStructure.setPerformanceHistory(key, value);
        }
        return newPerformaceStructure;
    }

    public void setUsageHistory(int iHistoryIndex, Object value){
        if(usageHistory==null)
            usageHistory = new LinkedList<List<Object>>();
        if(usageHistory.size()<=iHistoryIndex || usageHistory.get(iHistoryIndex)==null)
            usageHistory.add(iHistoryIndex, new LinkedList<Object>());
        usageHistory.get(iHistoryIndex).add(value);
    }
    public List<List<Object>> getUsageHistory(){
        return usageHistory;
    }

    public void setUsageHistory(List<List<Object>> newUsage){   
        if(newUsage!=null){
            usageHistory = new LinkedList<List<Object>>();
            for(List<Object> eList: newUsage){
                List<Object> newList = new LinkedList<Object>();
                for(Object el: eList)
                    newList.add(el);
                usageHistory.add(newList);
            }
        }
    }
    public void setClassifierHistory(List<List<Object>> newHistory){
        if (newHistory != null){
            if(classifierHistory==null)
                classifierHistory = new LinkedList<List<Object>>();
            for(List<Object> elist: newHistory){
                List<Object> ls = new LinkedList<Object>();
                for(Object el: elist){
                    ls.add(el);
                }
                classifierHistory.add(ls);
            }
        }
    }
    public void setClassifierHistory(int iHistoryIndex, Integer value){
        if(classifierHistory==null)
            classifierHistory = new LinkedList<List<Object>>();
        if(classifierHistory.size()<=iHistoryIndex||classifierHistory.get(iHistoryIndex)==null)
            classifierHistory.add(iHistoryIndex,new LinkedList<Object>());
        classifierHistory.get(iHistoryIndex).add(value);
    }

   public List<List<Object>> getClassifierHistory(){
       return classifierHistory;
   }
   
   private void setPerformanceHistory(String pMetric, List<LinkedList<Double>>newHistory){
        LinkedList<LinkedList<Double>> lstArrPerfHistory = performanceHistory.get(pMetric);
        if(lstArrPerfHistory==null) 
            lstArrPerfHistory = new LinkedList<LinkedList<Double>>();    
        
        for(List<Double> eList: newHistory){
            LinkedList<Double> newL = new LinkedList<Double>();
            for(Double el: eList){
                newL.add(el);
            }
            lstArrPerfHistory.add(newL);
        }
        performanceHistory.put(pMetric,lstArrPerfHistory);
    }

    public List<Double> getPredictedPerformance(){
        return predictedPerformanceHistory;
    }

    public void setPredictedPerformance(double dValue){
        predictedPerformanceHistory.add(dValue);
    }
    public void setPredictedPerformanceHistory(List<Double> predictedHistory){
        predictedPerformanceHistory=new LinkedList<Double>();
        for(Double el:predictedHistory)
            predictedPerformanceHistory.add(el);
    }
    public void clearHistory(){
        if(performanceHistory!=null) performanceHistory.clear();
        if(diffPerformanceHistory!=null) diffPerformanceHistory.clear();
    }

    public void setPerformanceChangeHistory(String pMetric, int iHistoryIndex, Double dVal){
        LinkedList<LinkedList<Double>> lstArrDiffPerfHistory = diffPerformanceHistory.get(pMetric);
        if (lstArrDiffPerfHistory == null){
            lstArrDiffPerfHistory = new LinkedList<LinkedList<Double>>();
        }
        while(lstArrDiffPerfHistory.size()<=iHistoryIndex)
            lstArrDiffPerfHistory.add(iHistoryIndex, new LinkedList<Double>());
        lstArrDiffPerfHistory.get(iHistoryIndex).add(dVal);
        diffPerformanceHistory.put(pMetric,lstArrDiffPerfHistory);
    }

    public double getAveragePerformanceHistory(String pMetric, int window){
        double avPer = 0.0;
        try{
            avPer = CommonFunctions.getAverage(performanceHistory.get(pMetric).get(window));
        } catch(Exception ex){
            System.out.println(ex.toString());
        }
        return avPer;
    }

    public LinkedList<LinkedList<Double>> getPerformanceHistory(String pMetric){
        if(performanceHistory.get(pMetric)==null)
            performanceHistory.put(pMetric, new LinkedList<LinkedList<Double>>());
        return performanceHistory.get(pMetric);
    }

    public LinkedList<LinkedList<Double>> getDiffPerformanceHistory(String pMetric){
        if(diffPerformanceHistory.get(pMetric)==null)
            return getPerformanceHistory(pMetric);
        return diffPerformanceHistory.get(pMetric);
    }
    public void setSuccessRateHistory(int iHistoryIndex, double dValue){
        while(successRateHistory.size()<iHistoryIndex)
            successRateHistory.add(0.01);
        successRateHistory.add(dValue);
    }

    private void setSuccessRateHistory(List<Double> history){
        successRateHistory=new LinkedList<Double>();
        for(Double el:history)
            successRateHistory.add(el);
    }

    public List<Double> getSuccessRateHistory(){return successRateHistory;}

    public List<Double> getPredictedSuccessRateHistory(){
        return predictedSuccessRateHistory;
    }
    
    public void setPredictedSuccessRateHistory(int iHistoryIndex, double dValue){
        if(predictedSuccessRateHistory==null)
            predictedSuccessRateHistory = new LinkedList<Double>();
        while(predictedSuccessRateHistory.size()<=iHistoryIndex)
            predictedSuccessRateHistory.add(0.01);
        predictedSuccessRateHistory.set(iHistoryIndex,dValue);
    }
    private void setPredictedSuccessRateHistory(List<Double> predictedHistory){
        predictedSuccessRateHistory=new LinkedList<Double>();
        for(Double el: predictedHistory)
            predictedSuccessRateHistory.add(el);
    }


    public void setPerformanceHistory(String pMetric, int iHistoryIndex, Double dVal){
        LinkedList<LinkedList<Double>> lstArrPerfHistory = performanceHistory.get(pMetric);
        if (lstArrPerfHistory == null)
            lstArrPerfHistory = new LinkedList<LinkedList<Double>>();
        while(lstArrPerfHistory.size()<=iHistoryIndex)
            lstArrPerfHistory.add(new LinkedList<Double>());
        lstArrPerfHistory.get(iHistoryIndex).add(dVal);
        performanceHistory.put(pMetric, lstArrPerfHistory);
    }
    public void setDiffPerformanceHistory(String pMetric, List<List<Double>>newHistory){
        LinkedList<LinkedList<Double>> lstArrPerfHistory = diffPerformanceHistory.get(pMetric);
        if(lstArrPerfHistory==null)
                lstArrPerfHistory = new LinkedList<LinkedList<Double>>(); 
        for(List<Double> eList:newHistory){
            LinkedList<Double> newList = new LinkedList<Double>();        
            for(Double el: eList){
                newList.add(el);
            lstArrPerfHistory.add(newList);
            }
        }
        diffPerformanceHistory.put(pMetric, lstArrPerfHistory);
    }
    public double getAverageSuccessInWindow(int window){
        double avg = 0.0;
        int iH= successRateHistory.size();
        for (int i=1;i<window;i++){
            avg=avg+successRateHistory.get(iH-i);
        }
        avg = avg/(double)window;
        return avg;
    }
    public double getAverageSuccess(){
        double avg = 0.0;
        int iH= successRateHistory.size();
        for (int i=1;i<iH;i++){
            avg=avg+successRateHistory.get(iH-i);
        }
        avg = avg/(double)iH;
        return avg;
    }
    public double getExtremeSuccessInWindow(int window){
        double max = 0.0;
        int iHistoryLength = successRateHistory.size();
        for (int i=1;i<window;i++){
            if(max>successRateHistory.get(iHistoryLength-i))
            max=successRateHistory.get(iHistoryLength-i);
        }
        return max;
    }
}
