/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package solution;

import comparator.multiObjective.MOSolutionRanking;
import java.util.BitSet;
import problem.Problem;
import utility.Utils;

/**
 *
 * @author Alda
 */
public class MOSolution extends Solution{

    private double[] fitnesses;
    private int nonDomRank;
    protected double hypervolume;
    private MOSolutionRanking ranking;
    private int[] MSOPSRanks;

    @Override
    public void initialiseRandomly(Problem p){
	fitnesses = new double[p.getObjectives()];
	ranking = new MOSolutionRanking(p.getObjectives());
	super.initialise(p.getCardinality());
	BitSet used = new BitSet();
	for (int i = 0; i < structure.length; i++){
	    do {
		structure[i] = Utils.random.nextInt(p.getOptions());
	    }while (used.get((Integer)structure[i]));
	    used.set((Integer)structure[i]);
	}
    }


    @Override
    public String getSolutionInSequence(){
	String s = "";
	int index = 0;
	do
	{
	    s+= getAssignmentForPosition(index)+", ";
	    index++;
	}while (index <structure.length);
	return s;
    }
    @Override
    public MOSolution clone(){
	MOSolution solution = new MOSolution();
        solution.structure = new Object[structure.length];
        System.arraycopy(this.structure, 0, solution.structure, 0, structure.length);
        solution.setParameterMap(getParameterMap());
	return solution;
    }
    public void setFitnessForObjective(int fitness, int obj){fitnesses[obj] = fitness;}

    public double getFitnessForObjective(int obj){return fitnesses[obj];}

    public double[] getAllFitnesses(){return fitnesses;}
    public int getMSOPSRank(int index){return MSOPSRanks[index];}
    public void setMSOPSRank(int fitness, int index, int targets){
        if(MSOPSRanks==null)
            MSOPSRanks = new int[targets];
        MSOPSRanks[index]= fitness;
    }

    public MOSolution distancePreservingCrossover(MOSolution other){
	MOSolution sol =new MOSolution();
	Solution [] originals = {this, other};
	BitSet bsLocations = new BitSet(structure.length);
	BitSet bsFacilities = new BitSet(structure.length);
		//the first loop just for finding common assignments
	for (int i = 0; i < structure.length; i++){
	    if (structure[i].equals(other.structure[i]))
	    {
		bsFacilities.set((Integer)structure[i]);
		bsLocations.set(i);
		sol.structure[i]= this.structure[i];
	    }
	}
	int temp1 = 0;
	int temp2 = 0;
	Object facilityToFind = -1;
	//assigning the non-common ones
	for (int i = 0; i < structure.length; i++){
	    if (!bsLocations.get(i)){
		if (bsFacilities.get((Integer)originals[0].structure[i])){   //both unassigned, choose randomly
		    if (bsFacilities.get((Integer)originals[1].structure[i])){
			temp1 = Utils.random.nextInt(2);
			temp2 = Math.abs(temp1-1);
		    }
		    else{
			temp1 = 0;
			temp2 = 1;
		    }
		}
		else if (bsFacilities.get((Integer)originals[1].structure[i])){
		    temp1 = 1;
		    temp2 = 0;
		}
		sol.structure[i] = originals[temp1].structure[i];
		bsLocations.set(i);
		bsFacilities.set((Integer)sol.structure[i]);
		//if it is already assigned, we don't have to look for it.
		if (bsFacilities.get((Integer)originals[temp2].structure[i]))
		    continue;
		facilityToFind = originals[temp2].structure[i];
		//finding the corresponding assignment of the other solution in the same solution
		for (int j = 0; j < structure.length; j++){
		    if ((Integer)originals[temp1].structure[j] == facilityToFind){
			sol.structure[j] = facilityToFind;
			bsFacilities.set((Integer)facilityToFind);
			bsLocations.set(j);
			if (bsFacilities.get((Integer)originals[temp2].structure[j]))
			    break; //the loop ends when the facility in the other parent has been assigned already
			facilityToFind = originals[temp2].structure[j];
			j = 0;
			continue;
		    }
		}
	    }
	}
	return sol;
    }

    public void setNonDomRank(int newNonDomRank){nonDomRank = newNonDomRank;}

    public int getNonDomRank(){return nonDomRank;}

    public boolean isDominated( Solution other){
	for ( int i=0; i< fitnesses.length; i++)
	    if (fitnesses[i] < ((MOSolution)other).fitnesses[i]) return false;	
	return true;
    }

    public double getHypervolume(){ return hypervolume; }

    public void setHypervolume(double hyp){ hypervolume = hyp;}

    @Override
    public String toString()
    {
/*	String s = super.toString();
	s += System.getProperty("line.separator");
	s += "Rank: "+this.nonDomRank;
	s += System.getProperty("line.separator");*/
	String s = "";
	for ( int i=0; i< fitnesses.length; i++)
	    s+= fitnesses[i] +" ";
	return s;
    }

    public MOSolutionRanking getRanking(){return ranking;}

    public boolean isDuplicate(Solution otherSol){
	for (int i = 0; i < fitnesses.length; i++){
	    if(this.fitnesses[i] != ((MOSolution)otherSol).fitnesses[i])
		return false;
	}
	return true;
    }

    public MOSolution distancePreservingMutation(int distance){
	//System.out.println("Parent 1 = \t"+this);
	MOSolution sol = clone();
	BitSet bsLocations = new BitSet(this.structure.length);
	int temp1;
	int temp2 =-1;
	Object initial;

	do{
	    temp1 = Utils.random.nextInt(this.structure.length);
	}while(bsLocations.get(temp1));
	bsLocations.set(temp1);

	initial = structure[temp1];
	for (int i = 0; i < distance-1; i++){
	    do{
		temp2 = Utils.random.nextInt(this.structure.length);
	    }while(bsLocations.get(temp2));
	    bsLocations.set(temp2);
	    sol.setAssignmentForPosition(temp1, this.structure[temp2]);
	    temp1 = temp2;
	}
	sol.setAssignmentForPosition(temp2, initial);
	//System.out.println("Child = \t"+sol);
	return sol;
    }
}
