package solution.QAP;

import problem.QAP.ConstrainedQAPProblem;
import java.util.BitSet;
import solution.Solution;
import problem.Problem;

import utility.Utils;

public class ConstrainedQAPSolution extends Solution
{

    private int[] path;
    private int constraintViolation;
 
    public ConstrainedQAPSolution()
    {
	super();

    }
    public int getConstraintViolation()
    {
        return constraintViolation;
    }

    public void setConstraintViolation(int constraintViolation)
    {
        this.constraintViolation = constraintViolation;
    }
    @Override
    public void initialiseRandomly(Problem p)
    {
	initialise(p.getCardinality());
	for (int i = 0; i < structure.length; i++)
	{
	    structure[i] = Utils.random.nextInt(p.getOptions());
	}
    }
            @Override
    public Solution clone()
    {
	Solution solution = new ConstrainedQAPSolution();
	solution.setValid(valid);
	solution.setCurrentFitness(currentFitness);
        System.arraycopy(this.structure, 0, solution.structure, 0, structure.length);
	return solution;
    }
    
    public Solution[] singlePointCrossover(Solution other)
    {
	Solution [] children = new Solution[2];
	children [0] = this.clone();
	children [1] = other;
	int point = Utils.random.nextInt(structure.length);
	
	for (int i = 0; i < point; i++)
	{
	    children[0].setAssignmentForPosition(i, this.structure[i]);
	    children[1].setAssignmentForPosition(i, other.structure[i]);
	}
	
	for (int i = point; i < structure.length; i++)
	{
	    children[0].setAssignmentForPosition(i, other.structure[i]);
	    children[1].setAssignmentForPosition(i, this.structure[i]);
	}
	
	return children;
    }
    
    
    public Solution mutateMultipoint(int options, double probability)
    {
	Solution sol = this.clone();
	Object newValue;
	for (int i = 0; i < structure.length; i++)
	{
	    if (Utils.random.nextDouble() < probability)
	    {
		do{
		    newValue = Utils.random.nextInt(options);
		}while(sol.structure[i].equals(newValue));
		sol.structure[i] = newValue;
	    }
	}
	return sol;
    }

     public double getAverageSequenceLength(Problem p)
    {
	double result = 0;
	double sequences[] = new double[p.getCardinality()];
	int sequencesCounter = 0;

	int pathindex = findFirst();
	Object currentValue = getAssignmentForPosition(pathindex);
	Object y;
	//System.out.println("Start at "+currentValue);
	int consecutiveNumbersCounter = 0;
	int i = pathindex;
	do
	{
	    y = getAssignmentForPosition(path[i]);
	    i = path[i];
	    if ( y == currentValue)
	    {
		if (consecutiveNumbersCounter == 0)
		    consecutiveNumbersCounter = 2;
		else
		    consecutiveNumbersCounter++;
		if (consecutiveNumbersCounter >((ConstrainedQAPProblem)p).getK())
		{
		    consecutiveNumbersCounter = ((ConstrainedQAPProblem)p).getK();
		}
	    }
	    else
	    {

		if (consecutiveNumbersCounter != 0)
		{
		    sequences[sequencesCounter++] = consecutiveNumbersCounter;
		    //System.out.println("Have "+consecutiveNumbersCounter+" times "+currentValue+" equals "+contribution);
		    consecutiveNumbersCounter =0;
		}
		currentValue = y;
	    }
	}while (path[i]!= pathindex);

	for (int j = 0; j < sequencesCounter; j++)
	{
	    //  System.out.println("Sequence no "+j+" is "+sequences[j]+" counter "+sequencesCounter);
	    result += sequences[j];
	}
	result /= (sequencesCounter*((ConstrainedQAPProblem)p).getK());
	//System.out.println("result "+result);
	return result;
    }
 
    @Override
    public void evaluateSolution(Problem p)
    {
	int violations = 0;
	double result = 0;
	valid = true;
        int k = ((ConstrainedQAPProblem)p).getK();
	int consecutiveNumbersCounter = 0;
	Object currentValue;
	Object y;
	int pathindex = findFirst();
	currentValue = getAssignmentForPosition(pathindex);
	/*if (FEcounter > 9999000)
	{
	    System.out.println(getSolutionInSequence(sol));
	    System.out.println("Start at "+currentValue);
	}*/
	double contribution = 0;
	int i = pathindex;
	do
	{
	    contribution = 0;
	    y = getAssignmentForPosition(path[i]);
	    i = path[i];
	    if ( y == currentValue)
	    {
		if (consecutiveNumbersCounter == 0)
		    consecutiveNumbersCounter = 2;
		else
		    consecutiveNumbersCounter++;
		if (consecutiveNumbersCounter >k)
		{
		    consecutiveNumbersCounter = k;
		}
	    }
	    else
	    {
		if (consecutiveNumbersCounter != 0)
		{
		    contribution = (double)Math.pow((Double)currentValue, (consecutiveNumbersCounter/(double)k))*consecutiveNumbersCounter;

		    int viol = calculateViolations(consecutiveNumbersCounter, p);
		    //  System.out.println("Have "+consecutiveNumbersCounter+" gives "+viol+" violations ");
		    violations += viol;
		    consecutiveNumbersCounter =0;
		}
		currentValue = y;
	    }
	    if (consecutiveNumbersCounter > ((ConstrainedQAPProblem)p).getOffset() && consecutiveNumbersCounter <= p.getCardinality())
		valid = false;


	    result += contribution;

	}while (path[i]!= pathindex);
	setValid(valid);
	setCurrentFitness(result);
	setConstraintViolation(violations);


    }
        private int calculateViolations(int successiveItems, Problem p)
    {
	successiveItems -= ((ConstrainedQAPProblem)p).getOffset();
        int c = ((ConstrainedQAPProblem)p).getC();
	if (successiveItems < 0 || successiveItems > c)
	    return 0;


	if (successiveItems == c/2)
	    return successiveItems + c%2;

	if (successiveItems > c/2)
	    return c+1-successiveItems;

	return successiveItems;

    }

    private int findFirst()
    {
	for (int i = 0; ; )
	{
	    if (getAssignmentForPosition(i) != getAssignmentForPosition(path[i]))
		return path[i];
	    i = path[i];

	}
    }

    @Override
    public String getSolutionInSequence()
    {
	String s = "";
	int index = 0;
	do
	{
	    s+= getAssignmentForPosition(index)+", ";
	    index = path[index];
	}while (index != 0);
	return s;
    }

    public void initialisePathFromMatrix(Problem p) throws Exception
    {
	BitSet bs = new BitSet(p.getCardinality());
        int[][] matrixA= ((ConstrainedQAPProblem)p).getDistanceMatrix();
	boolean found = false;

	for (int i = 0; i < matrixA.length; i++)
	{
	    found = false;
	    for (int j = 0; j < matrixA[i].length; j++)
	    {
		if (matrixA[i][j] != 0)
		{
		    path[j] = i;
		    bs.set(j);
		    found = true;
		    break;
		}
	    }
	}
	if (!found || bs.cardinality() < p.getCardinality())
	    throw new Exception("Path incomplete \n+"+found);
    }



   
   
}
