
package solution.QAP;

import java.util.BitSet;
import solution.Solution;
import problem.Problem;
import problem.QAP.SOQAPProblem;
import utility.OutOfFunctionEvaluationsException;
import utility.Utils;

/**
 *
 * @author Alda
 */
public class SOQAPSolution extends Solution{

    @Override
    public void initialiseRandomly(Problem p){
        BitSet bs = new BitSet(p.getCardinality());
	initialise(p.getCardinality());
	for (int i = 0; i < structure.length; i++){
            int value;
            do{value = Utils.random.nextInt(p.getCardinality());}while(bs.get(value));
            bs.set(value);
	    setAssignmentForPosition(i, value);
	}
    }
    
    @Override
    public Solution clone(){
	Solution solution = new SOQAPSolution();
	solution.setValid(valid);
	solution.setCurrentFitness(currentFitness);
        solution.setFitnessChange(fitnessChange);
        solution.setParameterMap(this.getParameterMap());
        solution.setValid(valid);
        solution.structure = new Object[this.structure.length];
        System.arraycopy(this.getStructure(), 0, solution.getStructure(), 0, structure.length);
        solution.setParameterMap(getParameterMap());
	return solution;
    }

    @Override
     public void evaluateSolution(Problem p) throws OutOfFunctionEvaluationsException{
	int result = 0;
        p.incrementFE();
        int[][] distance = ((SOQAPProblem)p).getDistanceMatrix();
        int[][] flow = ((SOQAPProblem)p).getFlowMatrix();
	for (int i = 0; i < structure.length; i++){
	    for (int j = 0; j < structure.length; j++)
		result += (Integer)distance[i][j] * (Integer)flow[(Integer)getAssignmentForPosition(i)][(Integer)getAssignmentForPosition(j)];
	}
	setCurrentFitness(result);
        if (!p.availableFunctionEvaluations())
	    throw new OutOfFunctionEvaluationsException();
    }

    @Override
    public Solution[] crossover(Solution sol2){
        return singlePointCrossover(sol2);
    }

    @Override
   public String getSolutionInSequence(){
	String s = "";
	int index = 0;
	do{
	    s+= getAssignmentForPosition(index)+", ";
	    index++;
	}while (index < structure.length);
	return s;
    }

    @Override
    public Solution[] singlePointCrossover(Solution sol2){

        Solution [] children = new Solution[2];
        children [0] = clone();
        children [1] = sol2.clone();
        BitSet bs0 = new BitSet(structure.length);
        double crossoverRate = 0.0;
        if(getParameterMap()!=null){
            crossoverRate = (Double)getParameterMap().get("crossoverRate").getCurrentBin().getCurrentValue();
            double tempCross = (Double)sol2.getParameterMap().get("crossoverRate").getCurrentBin().getCurrentValue();
            if(tempCross > crossoverRate) crossoverRate = tempCross;
        }
 
        if (Utils.random.nextDouble() < crossoverRate){
            int cutPoint = Utils.random.nextInt(structure.length);
            for (int i = 0; i < cutPoint; i++){
                Object tempValue = children[1].structure[i];
                children[1].structure[i] = children[0].structure[i];
                children[0].structure[i] = tempValue;
                bs0.set((Integer)tempValue);
            }
            for (int i = cutPoint; i < structure.length; i++){
                if(bs0.get((Integer)children[0].structure[i])){
                    int j=cutPoint-1;
                    do{
                        j++;
                    }while(j < structure.length && bs0.get((Integer)children[1].structure[j]));
                    Object tempValue = children[1].structure[j];
                    children[1].structure[j] = children[0].structure[i];
                    children[0].structure[i] = tempValue;
                    bs0.set((Integer)tempValue);
                }
                bs0.set((Integer)children[0].structure[i]);
            }
        }
        return children;
    }

}
