package solution.RoyalRoad;

import java.util.BitSet;
import problem.Problem;
import problem.RoyalRoad.RoyalRoadProblem;
import solution.Solution;
import utility.OutOfFunctionEvaluationsException;
import utility.Utils;


public class RoyalRoadSolution extends Solution
{
    
    @Override
    public void initialiseRandomly(Problem p){
        BitSet bs = new BitSet(p.getCardinality());
	initialise(p.getCardinality());
	for (int i = 0; i < structure.length; i++){
            int value = Utils.random.nextInt(((RoyalRoadProblem)p).getOptions());     
	    setAssignmentForPosition(i, value);
	}
    }
    @Override
    public Solution clone() {
        Solution solution = new RoyalRoadSolution();
        solution.structure = new Object[this.structure.length];
        System.arraycopy(getStructure(), 0, solution.getStructure(), 0, structure.length);
        solution.setParameterMap(getParameterMap());
        solution.setCurrentFitness(currentFitness);
        return solution;
    }

     @Override
     public void evaluateSolution(Problem p) throws OutOfFunctionEvaluationsException{
        p.incrementFE();
        RoyalRoadProblem rP = (RoyalRoadProblem)p;
        int blockCount = rP.getBlockCount();
	int blockSize = rP.getBlockSize();
	double partValue = rP.getPartValue();
	double partFitness = rP.getPartFitness();
	int levels = rP.getLevels();
	int regionSize = rP.getRegionSize();
	double bonusStartValue = rP.getBonusStartValue();
	double bonusValue = rP.getBonusValue();

	int[] blockComplete = new int[blockCount];

	// PART calculation
	double part = 0;
	for(int i = 0; i < blockCount;i++){
            int currBlockCount = getBitCount(i*regionSize, i*regionSize+blockSize, structure);
            if(currBlockCount==blockSize) blockComplete[i]=1;
            else if(currBlockCount<=partFitness) part = part + partValue*currBlockCount;
            else part = part - partValue*(currBlockCount-partFitness);
	}

        // BONUS calculation
        double bonus = 0;
        for(int i=0;i<=levels;i++)
        {
            int levelCompleted = 0;
            int levelSize = (int) Math.pow(2,i);
            for(int j=0;j<blockCount;j += levelSize){
                    boolean completed = true;
                    for(int k=j;k<j+levelSize;k++)
                        if(blockComplete[k]==0){
                            completed = false;
                            break;
                        }
                    if(completed) levelCompleted++;
            }
            if(levelCompleted>0) bonus += bonusStartValue + (levelCompleted-1) * bonusValue;
        }

        currentFitness = part + bonus;
    }
    public int getBitCount(int start, int end, Object[] struc){
        int count = 0;
        for(int i=start ; i < end; i++) if((Integer)struc[i]==1) count++;
        return count;
    }
    @Override
    public Solution[] singlePointCrossover(Solution sol2){
        Solution [] children = new Solution[2];
        children [0] = clone();
        children [1] = sol2.clone();
        double crossoverRate = 0.0;
        if(getParameterMap()!=null){
            crossoverRate = (Double)getParameterMap().get("crossoverRate").getCurrentBin().getCurrentValue();
            double tempCross = (Double)sol2.getParameterMap().get("crossoverRate").getCurrentBin().getCurrentValue();
            if(tempCross > crossoverRate) crossoverRate = tempCross;
        }

        if (Utils.random.nextDouble() < crossoverRate){
            int cutPoint = Utils.random.nextInt(structure.length);
            for (int i = 0; i < cutPoint; i++){
                Object tempValue = children[1].structure[i];
                children[1].structure[i] = children[0].structure[i];
                children[0].structure[i] = tempValue;
            }
            for (int i = cutPoint; i < structure.length; i++){              
                Object tempValue = children[1].structure[i];
                children[1].structure[i] = children[0].structure[i];
                children[0].structure[i] = tempValue;
            }
        }
        return children;
   }
   @Override
   public String getSolutionInSequence(){
	String s = "";
	int index = 0;
	do{
	    s+= getAssignmentForPosition(index)+", ";
	    index++;
	}while (index < structure.length);
	return s;
    }
}
