package solution.SAW;

import java.util.Random;
import problem.Problem;
import solution.Solution;
import utility.OutOfFunctionEvaluationsException;
import utility.StdDraw;
import utility.Utils;

/**
 *
 * @author Alda
 */
public class SAWSolution extends Solution{
        private boolean[][] visited;
        private int posX;
        private int posY;
        private boolean feasible = true;

    @Override
    public void initialise(int n){
        structure = new String[n*n];
        visited = new boolean[n][n];
        for(int i=0; i<n;i++){
            for(int j=0; j<n;j++)
                visited[i][j]=false;
        }
       // StdDraw.setXscale(0, n);
        //StdDraw.setYscale(0, n);
        //StdDraw.clear();
    }
    @Override
    public String toString(){
        String output=" ";
        for(Object w : structure){
            output+= w;
        }
        output+= " Fitness="+ currentFitness;
        return output;
    }

    public boolean isFeasible(){
        int i = 0, x = 0, y = 0;
        int size = structure.length;
        boolean[][] v = new boolean[size][size];
        while(i<structure.length){
            String w = (String) structure[i];
            if(w.equals("L")){x--;}
            else if(w.equals("R")){x++;}
            else if(w.equals("D")){y--;}
            else if(w.equals("U")){y++;}

            if(x<0 || x>(size-1) || y<0 || y>(size-1) || v[x][y]){
                feasible = false;
                return false;
            }
            v[x][y]=true;
            i++;
        }
        return true;
    }

    @Override
    public void initialiseRandomly(Problem problem) {
       int cardinality = problem.getCardinality();
       initialise(cardinality);
       posX = 0;
       posY = 0;
       int position =0;
       visited[posX][posY] = true;
       // make a random move as long as particle is not boxed in
       while ((posX>0 && !visited[posX-1][posY]) || (posX<(cardinality-1) && !visited[posX+1][posY]) || (posY>0 && !visited[posX][posY-1]) || (posY<(cardinality-1) && !visited[posX][posY+1])) {
           // try until you find an available move
           while (true) {
               double r = Math.random();
               if (r < 0.25 && (posX > 0 && !visited[posX-1][posY])) {
                   posX--;
                   structure[position++]="L";
                   break;
               }
               else if (r < 0.50 && (posX<(cardinality-1) && !visited[posX+1][posY])) {
                   posX++;
                   structure[position++]="R";
                   break;
               }
               else if (r < 0.75 && (posY>0 && !visited[posX][posY-1])) {
                   posY--;
                   structure[position++]= "D";
                   break;
               }
               else if (r < 1.00 && (posY<(cardinality-1) && !visited[posX][posY+1])) { 
                   posY++;
                   structure[position++]= "U";
                   break;
               }
            }
            visited[posX][posY] = true;
           // StdDraw.filledSquare(posX + 0.5, posY + 0.5, .45);
           // StdDraw.show(25);
        }
       //StdDraw.show(1000);
    }

    @Override
    public void evaluateSolution(Problem p) throws OutOfFunctionEvaluationsException{
        int i = 0;
        p.incrementFE();
        while(i<structure.length && structure[i]!=null) i++;
        currentFitness = i;
    }

    @Override
    public String getSolutionInSequence() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    @Override
    public Solution singlepointMutation(){
        Random rnd = Utils.random;
        Solution newSol = clone();
        int position = rnd.nextInt((int)currentFitness);
        int letter = rnd.nextInt(4);  
        if(letter==0){newSol.structure[position]="L";}
        else if(letter==1){newSol.structure[position]= "R";}
        else if(letter==2){newSol.structure[position]= "D";}
        else if(letter==3){newSol.structure[position]="U";}
        return newSol;
    }

    @Override
    public Solution[] singlePointCrossover(Solution parentSolution){
        Solution[] children = new SAWSolution[2];
        children[0]= this.clone();
        children[1]= parentSolution.clone();

        int minLength = parentSolution.structure.length;
        if(minLength>structure.length) minLength = structure.length;

        int position = Utils.random.nextInt(minLength);
        int i = 0;
        while(i<position){
            children[0].structure[i] = parentSolution.structure[i];
            children[1].structure[i] = structure[i];
            i++;
        }
        int j=i;
        while(j<structure.length){
            children[1].structure[j] = structure[j];
            j++;
        }
        while(i<parentSolution.structure.length){
            children[1].structure[i] = parentSolution.structure[i];
            i++;
        }
        return children;
    }
}
