package solver;

import java.util.Properties;
import problem.Problem;
import learning.LearningAlgorithm;
import utility.reporting.LearningReporting;
import utility.reporting.Reporting;

/**
 *
 * @author Alda
 */
public class AdaptiveAlgorithm {

    private Properties properties;
    private Algorithm optimisationAlgorithm;
    private LearningAlgorithm learningAlgorithm;
    private LearningReporting learningReporting;
    private Problem optimisationProblem;
    private int iCurrentIteration;


    public AdaptiveAlgorithm(Properties props, LearningReporting reporting){
        properties = props;
        learningReporting = reporting;
    }

    public void initialise(Problem problem){
        iCurrentIteration=0;
        optimisationProblem = problem;
        initialiseOptimisation();
        initialiseLearning();
    }
    
    public void run(Reporting optimisationReporting){
        optimisationProblem.resetCouter();
        while(optimisationProblem.availableFunctionEvaluations()){
            optimisationAlgorithm.oneIteration(optimisationReporting);
            learningAlgorithm.onIterationComplete(optimisationAlgorithm.getPopulation(), iCurrentIteration, learningReporting);
            iCurrentIteration++;
        }
    }

    public void initialiseOptimisation(){
        try{
            optimisationAlgorithm = (Algorithm) Class.forName(properties.getProperty("algo_type")).newInstance();
            optimisationAlgorithm.initialise(properties, optimisationProblem);
            System.gc();
        }catch(Exception ex){
            System.out.print(ex.toString());
        }
    }

    public void initialiseLearning(){
        try{
            learningAlgorithm = (LearningAlgorithm) Class.forName(properties.getProperty("learning_type")).newInstance();
            learningAlgorithm.initialise(properties);
            learningAlgorithm.initialiseParameters(properties);
            learningAlgorithm.initialiseRecordingStructures();
            learningAlgorithm.assignRandomParametersToSolutions(optimisationAlgorithm.getPopulation());     
        }
        catch(Exception ex){
            System.out.println(ex.toString());
        }
    }
}
