package solver;
import comparator.GenericComparator;
import java.util.List;
import java.util.Properties;
import problem.Problem;

import solution.Solution;
import utility.reporting.Reporting;

public abstract class Algorithm{
    protected int matingPoolSize;
    protected Problem optimisationProblem;
    protected GenericComparator currentComparator;
    protected double bestFitness;
    protected int noImprovementCounter;

    public void initialise(Properties props, Problem problem){
        try{
            currentComparator = (GenericComparator) Class.forName(props.getProperty("comparator")).newInstance();
        }catch(Exception ex){
            System.out.println(ex.toString());
        }
        optimisationProblem = problem;
        optimisationProblem.resetCouter();
        noImprovementCounter = 0;
        if(props.getProperty("optimisation_type").equals("maximisation"))
            bestFitness = Double.MAX_VALUE;
        else
            bestFitness = 0;
    }
    
    public void changeParameters(Properties props){
	matingPoolSize = Integer.parseInt(props.getProperty("mating_pool"));
    }
    
    public abstract void oneIteration(Reporting reporting);
   
    public void run(Reporting reporting){
        optimisationProblem.resetCouter();
        while(optimisationProblem.availableFunctionEvaluations()){       
            oneIteration(reporting);
        }
    }
    public abstract List<Solution> getPopulation();

}
