package solver.multiobjective;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import comparator.multiObjective.GenericMOComparator;
import problem.Problem;
import solution.MOSolution;
import solver.Algorithm;
import solution.Solution;

public abstract class MOAlgorithm extends Algorithm
{
    protected int nonDomFrontCount;
    protected LinkedList<LinkedList<MOSolution>> nondominatedFronts = null;
    protected LinkedList<MOSolution> matingPool;
    protected GenericMOComparator currentMOComparator;
    protected LinkedList<MOSolution> MOPopulation;
        protected List<Solution> population;
    protected int populationSize;

    @Override
    public void initialise(Properties props, Problem problem)
    {
	super.initialise(props, problem);
        ((GenericMOComparator)currentComparator).initialise(props, problem);
        currentMOComparator = (GenericMOComparator)currentComparator;
	nondominatedFronts = new LinkedList<LinkedList<MOSolution>>();
        populationSize = Integer.parseInt(props.getProperty("population"));
        population = new LinkedList<Solution>();
    }
   
    

    private void extractNondominatedFront(List<Solution> sourcePop,
	    LinkedList<MOSolution> currentFront, Set<MOSolution> otherFronts)
    {

	if ( sourcePop.isEmpty() )
	    return;

	//Everything here could be done with iterators, but this results in many Iterator objects being created, so making careful use of indices instead

	int first = 0;
	MOSolution firstInclusion;
	do {
	    firstInclusion = (MOSolution)sourcePop.get(first);
	    first++;
	} while ( otherFronts != null && otherFronts.contains( firstInclusion ) );
	currentFront.add( firstInclusion );

	final int popSize = sourcePop.size(); //saves method call overhead
	for (int i = first; i < popSize; i++) {
	    MOSolution sol = (MOSolution)sourcePop.get(i);
	    //Deb's pseudocode includes a condition on the loop that should never occur, but just in case...
	    assert ! currentFront.contains(sol);

	    if ( otherFronts == null || ! otherFronts.contains(sol) ) { //consider this solution as per usual dominance check
		currentFront.add( sol );

		boolean isNondominated = true;
		int j = 0;
		while ( isNondominated && j < currentFront.size() - 1 ) { //temporarily-added solution is guaranteed to be at the end
		    MOSolution otherSol = currentFront.get(j);
		    if ( otherSol.isDominated( sol) ) {
			currentFront.remove(j);
		    } else if ( sol.isDominated( otherSol) ) {
			isNondominated = false;
			currentFront.removeLast();
		    } else {
			j++;
		    }
		}
	    }
	}
    }

     protected void fastExtractNonDomFronts(){
	nondominatedFronts.clear();
	LinkedList<MOSolution> currentFront = null;

	Set<MOSolution> allFronts = new HashSet<MOSolution>();

	//Zero current front sizes
	nonDomFrontCount = 0;


	int rank = 0;
	int combinedSize = 0;
	do  {
	    rank++; //this will be 1 on first read

	    currentFront = new LinkedList<MOSolution>();
	    extractNondominatedFront(population, currentFront, allFronts);
	    combinedSize += currentFront.size();
	    nondominatedFronts.add(currentFront);
	    allFronts.addAll(currentFront);

	    for (MOSolution sol : currentFront)
		sol.setNonDomRank( rank );
	}while (combinedSize < populationSize);

    }

          @Override
    public List<Solution> getPopulation(){
        return population;
    }

}
