/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package solver.performance;

import java.util.List;
import solution.QAP.MOQAPSolution;

/**
 * This is adopted from C implemenation by Zitzler
 * ftp://ftp.tik.ee.ethz.ch/pub/people/zitzler/hypervol.c
 * @author indika
 */
public class HypervolumeCalculator {

    public HypervolumeCalculator(){

    }
    
    public static double calculateHypervolume(List<MOQAPSolution> front, int noObjectives) {
        int n;
        double volume, distance;

        volume = 0;
        distance = 0;
        n = front.size();
        while (n > 0) {
            
            double tempVolume, tempDistance;

            int noNondominatedPoints = filterNondominatedSet(front, noObjectives - 1);
            tempVolume = 0;
            if (noObjectives < 3) {
                if (noNondominatedPoints < 1) {
                    ERROR("run-time error");
                }
                tempVolume = front.get(0).getFitnessForObjective(0);
            } else {
                tempVolume = calculateHypervolume(front, noObjectives - 1);
            }
            tempDistance = surfaceUnchangedTo(front, noObjectives - 1);
            volume += tempVolume * (tempDistance - distance);
            distance = tempDistance;
            n = reduceNondominatedSet(front, n, noObjectives - 1, distance);
        }
        return volume;
    } /* CalculateHypervolume */

     /* all nondominated points regarding the first 'noObjectives' dimensions
    are collected; the points referenced by 'front[0..noPoints-1]' are
    considered; 'front' is resorted, such that 'front[0..n-1]' contains
    the nondominated points; n is returned */
    private static int filterNondominatedSet(List<MOQAPSolution> front, int  noObjectives){
        int i, j;
        int n;

        n = front.size();
        i = 0;
        while (i < n) {
            j = i + 1;
            while (j < n) {
                if (dominates(front.get(i), front.get(j), noObjectives)) {
                    /* remove point 'j' */
                    n--;
                    swap(front, j, n);
                } else if (dominates(front.get(j), front.get(i), noObjectives)) {
                    /* remove point 'i'; ensure that the point copied to index 'i'
                    is considered in the next outer loop (thus, decrement i) */
                    n--;
                    swap(front, i, n);
                    i--;
                    break;
                } else {
                    j++;
                }
            }
            i++;
        }
        return n;
    } /* FilterNondominatedSet */

    /* calculate next value regarding dimension 'objective'; consider
	points referenced in 'front[0..noPoints-1]' */
    private static double surfaceUnchangedTo(List<MOQAPSolution> front, int objective) {
        int i;
        double minValue, value;

        if (front.size() < 1) {
            ERROR("run-time error");
        }
        minValue = front.get(0).getFitnessForObjective(objective);
        for (i = 1; i < front.size(); i++) {
            value = front.get(i).getFitnessForObjective(objective);
            if (value < minValue) {
                minValue = value;
            }
        }
        return minValue;
    } /* SurfaceUnchangedTo */


    /* returns true if 'point1' dominates 'points2' with respect to the
	to the first 'noObjectives' objectives */
    static boolean dominates(MOQAPSolution point1, MOQAPSolution point2, int noObjectives) {
        int i;
        boolean betterInAnyObjective = false;

        for (i = 0; i < noObjectives && point1.getFitnessForObjective(i) >= point2.getFitnessForObjective(i); i++) {
            if (point1.getFitnessForObjective(i) > point2.getFitnessForObjective(i)) {
                betterInAnyObjective = true;
            }
        }
        return ((i >= noObjectives) && betterInAnyObjective);
    } /* Dominates */

    /* remove all points which have a value <= 'threshold' regarding the
    dimension 'objective'; the points referenced by
    'front[0..noPoints-1]' are considered; 'front' is resorted, such that
    'front[0..n-1]' contains the remaining points; 'n' is returned */
    private static int reduceNondominatedSet(List<MOQAPSolution> front, int  noPoints, int  objective, double  threshold){
        int n;
        int i;

        n = noPoints;
        for (i = 0; i < n; i++) {
            if (front.get(i).getFitnessForObjective(objective) <= threshold) {
                n--;
                swap(front, i, n);
            }
        }
        return n;
    } /* ReduceNondominatedSet */

    private static void swap(List<MOQAPSolution> front, int i, int j) {
        MOQAPSolution temp;
        temp = front.get(i);
        front.set(i, front.get(j));
        front.set(j, temp);
    } /* Swap */
    
    private static void ERROR(String string) {
        System.out.println("HV ERROR :" +string);
    }


}

