package utility;

import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;



import problem.QAP.MOQAPProblem;
import problem.Problem;
import solution.MOSolution;


public class MOResultsEvaluation
{
    private int objectives;
    private MOQAPProblem problem;
    private ObjectiveComparator comparator;

    
  
    public MOResultsEvaluation()
    {
	comparator = new ObjectiveComparator();
    }

    public double calculateHypervolumebySlicingObjectives(List<MOSolution> paretoFront)
    {
	double result = 0;
	
	double [][] pl = new double[paretoFront.size()][] ;
	
	
	objectives = problem.getObjectives();
	
	//assuming paretoFront is sorted in descending quality by first objective

	for (int i = 0; i < paretoFront.size(); i++)
	{
	    pl[i] =  paretoFront.get(i).getAllFitnesses();
	}
	
	
	LinkedList<Tuple> s = new LinkedList<Tuple>();
	LinkedList<Tuple> sprime; 
	s.add(new Tuple(1, pl));
	
	
	for (int k = 0; k < objectives-1; k++)
	{
	    sprime = new LinkedList<Tuple>();
	    for (Tuple tuple : s)
	    {
		comparator.prepare(k);
		Arrays.sort(tuple.points, comparator);
		List<Tuple> temp = slice( tuple.points, k);
		for (Tuple tuple2 : temp)
		{
		    tuple2.level *= tuple.level;
		}
		sprime.addAll(temp);
	    }

	    s = sprime;
	   

	}
	
	for (Tuple tuple : s)
	{	    
	    comparator.prepare(objectives-1);
	    Arrays.sort(tuple.points, comparator);
	    result += tuple.level* Math.abs( tuple.points[0][objectives-1] - problem.getMaxValueForObjective(objectives-1));		    
	}
	
	System.out.println("Numbers: "+paretoFront.size());
	System.out.println("HV, new version: "+result);
	
	return result;
    }
    
    private List<Tuple> slice( double[][] pl, int k)
    {	
	double [][] ql;
	List<Tuple> s = new LinkedList <Tuple>();
	
	for (int i = 0; i < pl.length-1; i++)
	{    
	    
	    ql = insert (i, k+1, pl);
	    s.add(new Tuple(Math.abs((int)pl[i][k]-(int)pl[i+1][k]),ql));
	    
	}
	ql = insert (pl.length-1, k+1, pl);
	s.add(new Tuple(Math.abs((int)pl[pl.length-1][k] -  (int)problem.getMaxValueForObjective(k)), ql));
	return s;
	
    }
  
    
    private double[][] insert (int currentPosition, int k, double [][] pl)
    {
	int j = 0;
	double[][] ql = new double[currentPosition+1][];
	ql[j++] = pl[currentPosition];
	int i = (currentPosition-1) ;
	
	
	//while (i < pl.length)  // won't happen if only fed the nondominated set
	while(i >=0)
	{
	    //checking whether the current point will be dominated in any of the following dimensions
	    if (!dominates (pl[currentPosition], pl[i], k))
		ql[j++] = pl[i];
	   
	    i--;
	    
	}
	ql = Arrays.copyOf(ql, j);
	return ql;	
    }
    
    //checks only the rest of the dimensions
    private boolean dominates(double[] point1, double[] point2, int k)
    {
	while (k < point2.length)
	{
	    if (point2[k] < point1[k])
		return false;
	    k++;
	}
	return true;
	
    }
    
    
     
 
    
    class Tuple{
	public int level;
	public double[][] points;
	
	public Tuple(int lvl, double[][] array )
	{
	    level = lvl;
	    points = array;
	}
    }
    
    /* originals from paper (including correction)
    public double calculateHypervolumebySlicingObjectives()
    {
	pl = sort ps worsening in objective 1
	s = {(1,pl)}
	for (int k = 1; k < objectives-1; k++)
	{
	    sprime = new Set()
	    for each (x, ql) in s
	     for each (x*, ql*) in slice (ql, k)
	    	add(x*x*, ql') into s'
	    s = s'
	    vol = 0
	    for each (x, ql) in s
	    	if q1 = []
	    	         vol += x* Math.abs( z(n) - refPoint[n])
	    	         vol += x* Math.abs(z(n) - head(q1[n]))
	    	                                  
	    return vol;
	}
    }
    private set slice(z, pl, k)
    {
	ql = new double[];
	s = set
	v = z[k]
	dominated = false
	while pl not empty and not dominated
	{
	    p = head (pl)
	    pl = tail (pl)
	    if v beats p[k]
	        add (Math.abs(v - p[k]), ql) into s
	        	v =p[k]
	    ql = insert (p, k)
	    dominated = dominates (p, z, k+1)
	    if not dominated
	    	add (Math.abs(v-refPoint[k]), ql) into s
	    return s
	}
    }
    
    private double[] insert (double [] p, int k, double [] pl)
    {
	double[] ql = new double[pl.length];
	while (pl not empty and head (pl)[k] beats p[k])
	{
	    append head (pl) to ql
	    pl = tail (pl)
	}
	append p to ql
	while (pl is not empty)
	    if not (dominates (p, head (pl), k))
	    	append head(pl) to ql
	    pl = tail (pl)
	return ql;
	    
	    
	                                            }
	
    }*/
    class ObjectiveComparator implements Comparator<double[]>
    {
        private int objectiveToCompare = 0;

        @Override
        public int compare(double[] arg0, double[] arg1)
        {
    	if ( arg0[objectiveToCompare] > arg1[objectiveToCompare])
    	    return 1;
    	else if(arg0[objectiveToCompare] < arg1[objectiveToCompare])
    	    return -1;
    	return 0;
        }

        
        

        
        public void prepare(int objective)
        {
    	objectiveToCompare = objective;

        }
        //this comparator is always ready
        public boolean isPrepared()
        {
    	return true;
        }

    }

   /* class TempComparator implements Comparator<MOGASolution>
    {
	@Override
	    public int compare(MOGASolution arg0, MOGASolution arg1)
	    {
		if ( arg0.getFitnessForObjective(0) > arg1.getFitnessForObjective(0))
		    return 1;
		else if(arg0.getFitnessForObjective(0) < arg1.getFitnessForObjective(0))
		    return -1;
		return 0;
	    }
    }*/
    
}
