
package utility.forecasting;

import java.io.Serializable;

// Referenced classes of package
//            Cdf

public class ANOVA
    implements Serializable, Cloneable
{


    private static final long serialVersionUID = 0x47b10556872e6b05L;
    private double y[][];
    private double anova[];
    private double groupStdev[];
    private double stat[][];
    private int nmissing;
    private int ngroups;
    public static final int TUKEY = 0;
    public static final int TUKEY_KRAMER = 1;
    public static final int DUNN_SIDAK = 2;
    public static final int BONFERRONI = 3;
    public static final int SCHEFFE = 4;
    public static final int ONE_AT_A_TIME = 5;
    public ANOVA(double ad[][])
    {
        y = ad;
        int i = ad.length;
        int j = i;
        int k = 0;
        double d = 0.0D;
        boolean flag = true;
        boolean flag1 = true;
        double d1 = 0.0D;
        int l = 0;
        boolean flag2 = false;
        int j1 = 0;
        int ai[] = new int[i];
        double ad1[] = new double[i];
        ngroups = i;
        groupStdev = new double[i];
        stat = new double[i][4];
        for(int k1 = 0; k1 < i; k1++)
        {
            double d3 = 0.0D;
            int l1 = 0;
            for(int i2 = 0; i2 < ad[k1].length; i2++)
            {
                int i1 = l + i2;
                if(!Double.isNaN(ad[k1][i2]))
                {
                    d3 += ad[k1][i2];
                    if(flag)
                    {
                        d1 = ad[k1][i2];
                        flag = false;
                        continue;
                    }
                    if(ad[k1][i2] != d1)
                        flag1 = false;
                } else
                {
                    k++;
                    l1++;
                }
            }

            nmissing = k;
            if(l1 == ad[k1].length)
                j--;
            l += ad[k1].length;
            stat[k1][0] = k1;
            stat[k1][1] = ad[k1].length - l1;
            ai[k1] = ad[k1].length - l1;
            j1 += ad[k1].length - l1;
            ad1[k1] = d3 / (double)ai[k1];
            stat[k1][2] = ad1[k1];
            d += d3;
        }

        double d2 = d / (double)j1;
        double d4 = 0.0D;
        double d5 = 0.0D;
        if(!flag1)
        {
            for(int j2 = 0; j2 < i; j2++)
            {
                if((double)ai[j2] > 0.0D)
                {
                    double d6 = ad1[j2] - d2;
                    d4 += (double)ai[j2] * d6 * d6;
                    for(int l3 = 0; l3 < ad[j2].length; l3++)
                        if(!Double.isNaN(ad[j2][l3]))
                        {
                            double d7 = ad[j2][l3] - ad1[j2];
                            groupStdev[j2] += d7 * d7;
                        }

                }
                stat[j2][3] = groupStdev[j2];
            }

            for(int k2 = 0; k2 < groupStdev.length; k2++)
                d5 += groupStdev[k2];

        }
        for(int l2 = 0; l2 < i; l2++)
        {
            int j3 = Math.max(0, ai[l2] - 1);
            if(flag1)
            {
                if(j3 == 0)
                    groupStdev[l2] = (0.0D / 0.0D);
                else
                    groupStdev[l2] = 0.0D;
            } else
            {
                groupStdev[l2] = Math.sqrt(groupStdev[l2] / (double)j3);
            }
            stat[l2][3] = groupStdev[l2];
        }

        int i3 = 0;
        int k3 = 0;
        if(j > 0)
        {
            i3 = j - 1;
            k3 = j1 - i3 - 1;
        }
        compute(i3, d4, k3, d5, d2);
    }

    public ANOVA(double d, double d1, double d2, double d3, double d4)
    {
        compute(d, d1, d2, d3, d4);
    }

    private void compute(double d, double d1, double d2, double d3, double d4)
    {
        anova = new double[15];
        anova[0] = d;
        anova[1] = d2;
        anova[2] = d + d2;
        anova[3] = d1;
        anova[4] = d3;
        anova[5] = d1 + d3;
        anova[6] = d1 / d;
        anova[7] = d3 / d2;
        anova[8] = anova[6] / anova[7];
        if(Double.isNaN(anova[8]))
            anova[9] = (0.0D / 0.0D);
        else
        if(anova[8] == (1.0D / 0.0D))
            anova[9] = 0.0D;
        else
            anova[9] = Cdf.complementaryF(anova[8], d, d2);
        double d5 = (100D * d1) / anova[5];
        anova[10] = d5;
        double d6 = anova[5] / anova[2];
        double d7 = anova[7] / d6;
        if(!Double.isNaN(d7))
            if(d7 > 1.0D)
                d7 = 0.0D;
            else
                d7 = 100D * (1.0D - d7);
        anova[11] = d7;
        if(Double.isNaN(anova[7]))
        {
            anova[12] = (0.0D / 0.0D);
            anova[14] = (0.0D / 0.0D);
        } else
        {
            double d8 = Math.sqrt(anova[7]);
            anova[12] = d8;
            double d9 = (100D * d8) / d4;
            anova[14] = d9;
        }
        anova[13] = d4;
    }

    public double[] getArray()
    {
        double ad[] = new double[anova.length];
        System.arraycopy(anova, 0, ad, 0, anova.length);
        return ad;
    }

    public double getDegreesOfFreedomForModel()
    {
        return anova[0];
    }

    public double getDegreesOfFreedomForError()
    {
        return anova[1];
    }

    public double getTotalDegreesOfFreedom()
    {
        return anova[2];
    }

    public double getSumOfSquaresForModel()
    {
        return anova[3];
    }

    public double getSumOfSquaresForError()
    {
        return anova[4];
    }

    public double getTotalSumOfSquares()
    {
        return anova[5];
    }

    public double getModelMeanSquare()
    {
        return anova[6];
    }

    public double getErrorMeanSquare()
    {
        return anova[7];
    }

    public double getF()
    {
        return anova[8];
    }

    public double getP()
    {
        return anova[9];
    }

    public double getRSquared()
    {
        return anova[10];
    }

    public double getAdjustedRSquared()
    {
        return anova[11];
    }

    public double getModelErrorStdev()
    {
        return anova[12];
    }

    public double getMeanOfY()
    {
        return anova[13];
    }

    public int getTotalMissing()
    {
        return nmissing;
    }

    public double getCoefficientOfVariation()
    {
        return anova[14];
    }

    public double[][] getGroupInformation()
    {
        double ad[][] = new double[stat.length][4];
        for(int i = 0; i < stat.length; i++)
            System.arraycopy(stat[i], 0, ad[i], 0, 4);

        return ad;
    }

    /**
     * @deprecated Method getDunnSidak is deprecated
     */

    public double getDunnSidak(int i, int j)
    {
        if((i >= y.length) | (i < 0))
        {
            Object aobj[] = {
                "i", new Integer(i), new Integer(0), new Integer(y.length - 1)
            };
            System.out.println("OutOfRange"+ aobj);
        }
        if((j >= y.length) | (j < 0))
        {
            Object aobj1[] = {
                "j", new Integer(j), new Integer(0), new Integer(y.length - 1)
            };
            System.out.println("OutOfRange"+ aobj1);
        }
        double d = anova[4] / anova[1];
        double d1 = d / (double)y[i].length;
        double d2 = d / (double)y[j].length;
        double d3 = 0.5D + 0.5D * Math.pow(0.94999999999999996D, 1.0D / ((double)(ngroups * ngroups - 1) / 2D));
        return Cdf.inverseStudentsT(d3, getDegreesOfFreedomForError()) * Math.sqrt(d1 + d2);
    }

    public double[] getConfidenceInterval(double d, int i, int j, int k)
    {
        if((i >= y.length) | (i < 0))
        {
            Object aobj[] = {
                "i", new Integer(i), new Integer(0), new Integer(y.length - 1)
            };
            System.out.println("OutOfRange"+ aobj);
        }
        if((j >= y.length) | (j < 0))
        {
            Object aobj1[] = {
                "j", new Integer(j), new Integer(0), new Integer(y.length - 1)
            };
            System.out.println("OutOfRange"+ aobj1);
        }
        double ad[] = new double[5];
        double d1 = 0.0D;
        if(k == 0 && y[i].length != y[j].length || k == 1)
        {
            if(d < 0.0D || d > 100D)
            {
                Object aobj2[] = {
                    "conLevel", new Double(d), new Double(0.0D), new Double(100D)
                };
                System.out.println("OutOfRange"+ aobj2);
            }
            double d21 = anova[4] / anova[1];
            double d22 = d21 / (double)y[i].length;
            double d23 = d21 / (double)y[j].length;
            double d24 = d / 100D;
            double d25 = ngroups;
            d1 = l_srin(d24, getDegreesOfFreedomForError(), d25) * Math.sqrt((d22 + d23) * 0.5D);
        } else
        if(k == 0)
        {
            if(d < 90D || d > 99D)
            {
                Object aobj3[] = {
                    "conLevel", new Double(d), new Double(90D), new Double(99D)
                };
                System.out.println("OutOfRange"+ aobj3);
            }
            if(y[i].length != y[j].length)
            {
                Object aobj4[] = {
                    "NotBalanced", new Integer(y[i].length), new Integer(y[j].length)
                };
                System.out.println("NotBalanced"+ aobj4);
            }
            double d2 = anova[4] / anova[1];
            double d7 = d / 100D;
            double d12 = ngroups;
            d1 = l_srin(d7, getDegreesOfFreedomForError(), d12) * Math.sqrt(d2 / (double)y[i].length);
        } else
        if(k == 2)
        {
            if(d < 0.0D || d >= 100D)
            {
                Object aobj5[] = {
                    "conLevel", new Double(d), new Double(0.0D), new Double(100D)
                };
                System.out.println("OutOfRange"+ aobj5);
            }
            double d3 = anova[4] / anova[1];
            double d8 = d3 / (double)y[i].length;
            double d13 = d3 / (double)y[j].length;
            double d17 = 0.5D + 0.5D * Math.pow(d / 100D, 1.0D / ((double)(ngroups * (ngroups - 1)) / 2D));
            d1 = Cdf.inverseStudentsT(d17, getDegreesOfFreedomForError()) * Math.sqrt(d8 + d13);
        } else
        if(k == 3)
        {
            if(d < 0.0D || d >= 100D)
            {
                Object aobj6[] = {
                    "conLevel", new Double(d), new Double(0.0D), new Double(100D)
                };
                System.out.println("OutOfRange"+ aobj6);
            }
            double d4 = anova[4] / anova[1];
            double d9 = d4 / (double)y[i].length;
            double d14 = d4 / (double)y[j].length;
            double d18 = 1.0D - (1.0D - d / 100D) / (2D * ((double)(ngroups * (ngroups - 1)) / 2D));
            d1 = Cdf.inverseStudentsT(d18, getDegreesOfFreedomForError()) * Math.sqrt(d9 + d14);
        } else
        if(k == 4)
        {
            if(d < 0.0D || d >= 100D)
            {
                Object aobj7[] = {
                    "conLevel", new Double(d), new Double(0.0D), new Double(100D)
                };
                System.out.println("OutOfRange"+ aobj7);
            }
            double d5 = anova[4] / anova[1];
            double d10 = d5 / (double)y[i].length;
            double d15 = d5 / (double)y[j].length;
            double d19 = d / 100D;
            d1 = Cdf.inverseF(d19, ngroups - 1, getDegreesOfFreedomForError()) * (d10 + d15);
            d1 = Math.sqrt((double)(ngroups - 1) * d1);
        } else
        if(k == 5)
        {
            if(d < 0.0D || d >= 100D)
            {
                Object aobj8[] = {
                    "conLevel", new Double(d), new Double(0.0D), new Double(100D)
                };
                System.out.println("OutOfRange"+ aobj8);
            }
            double d6 = anova[4] / anova[1];
            double d11 = d6 / (double)y[i].length;
            double d16 = d6 / (double)y[j].length;
            double d20 = 1.0D - (1.0D - d / 100D) / 2D;
            d1 = Cdf.inverseStudentsT(d20, getDegreesOfFreedomForError()) * Math.sqrt(d11 + d16);
        } else
        {
            System.out.println("ANOVA.InvalidCompMethod");
        }
        ad[0] = i;
        ad[1] = j;
        ad[2] = stat[i][2] - stat[j][2];
        ad[3] = ad[2] - d1;
        ad[4] = ad[2] + d1;
        return ad;
    }

    private double l_srin(double d, double d1, double d2)
    {
        double d6 = 0.0D;
        double d10 = 0.0D;
        double d8 = l_s3in(d, d1, d2);
        double d5 = l_srdf(d8, d1, d2);
        d10 = d8;
        if(Math.abs(d5 - d) < 0.001D)
            return d10;
        if(d5 > d)
            d5 = 1.75D * d - 0.75D * d5;
        else
        if(d5 < d)
            d6 = d + (((d - d5) * (1.0D - d)) / (1.0D - d5)) * 0.75D;
        if(d6 < 0.80000000000000004D)
            d6 = 0.80000000000000004D;
        else
        if(d6 > 0.995D)
            d6 = 0.995D;
        double d9 = l_s3in(d6, d1, d2);
        for(int i = 2; i <= 8; i++)
        {
            double d7 = l_srdf(d9, d1, d2);
            double d3 = d5 - d;
            double d4 = d7 - d;
            d10 = (d4 * d8 - d3 * d9) / (d4 - d3);
            if(Math.abs(d3) >= Math.abs(d4))
            {
                d8 = d9;
                d5 = d7;
            }
            if(Math.abs(d5 - d) < 0.0050000000000000001D)
                return d10;
            d9 = d10;
        }

        return d10;
    }

    private double l_s3in(double d, double d1, double d2)
    {
        double d4 = 0.0D;
        double d5 = Cdf.inverseNormal(d);
        if(d1 < 120D)
            d5 += (d5 * d5 * d5 + d5) / d1 / 4D;
        double d3 = 0.88429999999999997D - 0.23680000000000001D * d5;
        if(d1 < 120D)
            d3 += -1.214D / d1 + (1.208D * d5) / d1;
        d4 = d5 * (d3 * Math.log(d2 - 1.0D) + 1.4141999999999999D);
        return d4;
    }

    private double l_srdf(double d, double d1, double d2)
    {
        double ad[] = new double[60];
        double ad1[] = new double[60];
        double ad2[] = new double[4];
        boolean flag = true;
        if(flag)
        {
            ad2[0] = 0.31830988599999999D;
            ad2[1] = -0.0026813271600000001D;
            ad2[2] = 0.0034722222200000001D;
            ad2[3] = 0.083333333300000006D;
            boolean flag1 = false;
        }
        double d18 = 0.0D;
        double d10 = 0.0D;
        double d6 = 0.45000000000000001D * Math.pow(d2, -0.20000000000000001D);
        double d8 = 0.5D * Math.log(d2);
        double d17 = d2 - 1.0D;
        double d3 = Math.log(d2 * d6 * 0.39894227999999998D);
        if(d1 <= 120D)
        {
            d10 = 0.45000000000000001D / Math.sqrt(d1);
            double d4 = d1 * 0.5D;
            if(d1 == 1.0D)
                d3 = 0.193064705D;
            else
            if(d1 == 2D)
                d3 = 0.29352532599999998D;
            else
                d3 = (Math.sqrt(d4) * ad2[0]) / (1.0D + ((ad2[1] / d4 + ad2[2]) / d4 + ad2[3]) / d4);
            d3 = Math.log(d3 * d2 * d6 * d10);
        }
        double d9 = d6;
        ad1[0] = -1D;
        ad1[15] = -1D;
        double d14 = 1.0D;
        double d15 = 1.0D;
        for(int l = 1; l <= 15; l++)
        {
            d9 -= d6;
            do
            {
                d9 = -d9;
                double d7 = d8 + d9;
                double d13 = 0.0D;
                if(d15 > 0.0001D || l <= 7)
                {
                    double d19 = d3 - d7 * d7 * 0.5D;
                    double d16 = Cdf.normal(-d7);
                    double d20 = Cdf.normal(-d7 + d) - d16;
                    if(d20 > 0.0D)
                        d13 = Math.exp(d19 + d17 * Math.log(d20));
                    if(d1 <= 120D)
                    {
                        int k = -15;
                        do
                        {
                            k += 15;
                            int i = 1;
                            do
                            {
                                if(i > 15)
                                    break;
                                int j = i + k;
                                if(ad1[j - 1] <= 0.0D)
                                {
                                    double d11 = d10 * (double)i;
                                    if(i < 15)
                                        ad1[j] = -1D;
                                    double d5 = Math.exp(d11);
                                    ad1[j - 1] = d * d5;
                                    ad[j - 1] = d1 * ((d11 + 0.5D) - d5 * d5 * 0.5D);
                                }
                                double d12 = 0.0D;
                                double d21 = Cdf.normal(-d7 + ad1[j - 1]) - d16;
                                if(d21 > 0.0D)
                                    d12 = Math.exp(d19 + ad[j - 1] + d17 * Math.log(d21));
                                d13 += d12;
                                if(d12 <= 3.0000000000000001E-005D && (j > 3 || l > 7))
                                    break;
                                i++;
                            } while(true);
                            d10 = -d10;
                        } while(d10 < 0.0D);
                    }
                }
                d18 += d13;
                if(l > 7 && d13 <= 0.0001D && d14 <= 0.0001D)
                    return d18;
                d15 = d14;
                d14 = d13;
            } while(d9 > 0.0D);
        }

        return d18;
    }


}