package utility.forecasting;

import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

// Referenced classes of package 
//            ARMA, ARMAMaxLikelihood

public class ARAutoUnivariate
    implements Serializable
{
    /**
     * @deprecated Class Formatter is deprecated
     */

    private static final double EPSILON_LARGE = 2.2204460492503E-016D;
    private double aic;
    private double aicc;
    private double bic;
    private int l_infoCriterion;
    private double confidence;
    private double constant;
    private int iprint;
    private int maxit;
    private int maxlag;
    private int method;
    private double variance;
    private double armaVar;
    private boolean center;
    private double z_mean;
    private double p_constant[];
    private double armaConstant;
    private double par[];
    private double armaAR[];
    private boolean success;
    private double z[];
    private int nobs;
    private int npar;
    private int backwardOrigin;
    private boolean compute_flag;
    private double l_convergence_tolerance;
    private double forecast_matrix[][];
    private double l_forecast[];
    private double l_dev[];
    private double l_psiWeights[];
    private double residuals[];
    private double maxarma_likelihood;
    private static final String NEWLINE = System.getProperty("line.separator");
    private Logger logger;
    public String nameClass;
    public String nameMethod;
    public String keyMessage;
    public static final int METHOD_OF_MOMENTS = 0;
    public static final int LEAST_SQUARES = 1;
    public static final int MAX_LIKELIHOOD = 2;
    public static final int AIC = 0;
    public static final int AICC = 1;
    public static final int BIC = 2;
    private static final double EMPTY_DOUBLE_VECTOR[] = new double[0];
	
    public static class Formatter extends java.util.logging.Formatter
    {

        public String format(LogRecord logrecord)
        {
            String s1 = logrecord.getMessage();
            Object aobj[] = logrecord.getParameters();
            return (new StringBuilder()).append(s1).append(aobj).append(ARAutoUnivariate.NEWLINE).toString();
        }

        public Formatter()
        {
        }
    }

    public static class TriangularMatrixSingularException extends Exception
    {

        private static final long serialVersionUID = 0xf990ba3be02b995bL;

        public TriangularMatrixSingularException(String s)
        {
            super(s);
        }

        public TriangularMatrixSingularException(String s, Object aobj[])
        {
            super(s+ aobj);
        }
    }


    public ARAutoUnivariate(int i, double ad[])
    {
        aic = (1.0D / 0.0D);
        aicc = (1.0D / 0.0D);
        bic = (1.0D / 0.0D);
        l_infoCriterion = 0;
        confidence = 0.94999999999999996D;
        constant = 0.0D;
        iprint = 0;
        maxit = 300;
        maxlag = 10;
        method = 1;
        variance = (0.0D / 0.0D);
        armaVar = (0.0D / 0.0D);
        center = true;
        z_mean = 0.0D;
        success = true;
        backwardOrigin = 0;
        compute_flag = false;
        maxarma_likelihood = 0.0D;
        if(ad.length <= 1)
        {
            Object aobj[] = {
                "z.length", new Integer(ad.length), new Integer(0)
            };
            System.out.println("NotLargeEnough"+ aobj);
        }
        if(i < 1 || i > ad.length / 2)
        {
            Object aobj1[] = {
                "maxlag", new Integer(i), new Integer(0), new Integer(ad.length / 2)
            };
            System.out.println("GreaterLess"+ aobj1);
        }

        p_constant = new double[1];
        par = new double[i];
        z = new double[ad.length];
        System.arraycopy(ad, 0, z, 0, ad.length);
        nobs = ad.length;
        maxlag = i;
        backwardOrigin = 0;
        npar = i;
        l_convergence_tolerance = Math.max(1.0000000000000001E-009D, Math.pow(2.2204460492503131E-016D, 0.66666666666666663D));
    }

    public void compute()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(logger != null && logger.isLoggable(Level.CONFIG))
        {
            nameClass = "ARAutoUnivariate";
            nameMethod = "compute";
            iprint = 1;
        }
        if(logger != null && logger.isLoggable(Level.FINE))
        {
            nameClass = "ARAutoUnivariate";
            nameMethod = "compute";
            iprint = 1;
        }
        if(logger != null && logger.isLoggable(Level.FINER))
        {
            nameClass = "ARAutoUnivariate";
            nameMethod = "compute";
            iprint = 2;
        }
        if(logger != null && logger.isLoggable(Level.FINEST))
        {
            nameClass = "ARAutoUnivariate";
            nameMethod = "compute";
            iprint = 2;
        }
        l_a2to_uni_ar();
        compute_flag = true;
    }

    public double getAIC()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        double d = aic - (double)(z.length - maxlag) * Math.log(6.2831853071795862D) - 1.0D;
        return d;
    }

    public double getAICC()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return aicc;
    }

    public double getBIC()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return bic;
    }

    public double getConfidence()
    {
        return confidence;
    }

    public int getBackwardOrigin()
    {
        return backwardOrigin;
    }

    public double getTimsacConstant()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return p_constant[0];
    }

    public double getConstant()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return armaConstant;
    }

    public double[] getTimeSeries()
    {
        return z;
    }

    public void setBackwardOrigin(int i)
    {
        boolean flag = false;
        if(i < 0)
        {
            Object aobj[] = {
                "backwardOrigin", new Integer(i)
            };
            System.out.println("NotPositive"+ aobj);
        }
        if(i > z.length - npar)
        {
            Object aobj1[] = {
                "backwardOrigin", new Integer(i), new Integer(npar)
            };
            System.out.println("LessThan"+ aobj1);
        }
        backwardOrigin = i;
    }

    public double[][] forecast(int i)
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(i <= 0)
            return (double[][])null;
        if(!compute_flag)
            compute();
        ARMA arma = new ARMA(npar, 0, z);
        arma.setArmaInfo(getConstant(), getAR(), new double[0], getInnovationVariance());
        arma.setBackwardOrigin(backwardOrigin);
        arma.setConfidence(confidence);
        arma.setConvergenceTolerance(l_convergence_tolerance);
        forecast_matrix = arma.forecast(i);
        double ad[] = arma.getPsiWeights();
        if(backwardOrigin > 0)
        {
            l_dev = new double[backwardOrigin + i];
            double ad1[] = arma.getDeviations();
            residuals = new double[backwardOrigin];
            for(int j = 0; j < backwardOrigin; j++)
            {
                int l = (z.length - backwardOrigin) + j;
                residuals[j] = z[l] - forecast_matrix[0][j];
                l_dev[j] = ad1[0];
            }

            for(int k = backwardOrigin; k < backwardOrigin + i; k++)
                l_dev[k] = ad1[k - backwardOrigin];

        } else
        {
            l_dev = arma.getDeviations();
            residuals = null;
        }
        return forecast_matrix;
    }

    public double[] getForecast(int i)
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        boolean flag = false;
        if(i <= 0)
            return null;
        if(!compute_flag)
            compute();
        double ad[][] = forecast(i);
        l_forecast = new double[i + backwardOrigin];
        for(int j = 0; j < backwardOrigin; j++)
            l_forecast[j] = ad[0][j];

        for(int k = 0; k < i; k++)
            l_forecast[k + backwardOrigin] = ad[k][backwardOrigin];

        return (double[])(double[])l_forecast.clone();
    }

    public double[] getDeviations()
    {
        if(l_dev == null)
            return null;
        else
            return (double[])(double[])l_dev.clone();
    }

    public double getLikelihood()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        if(method == 2)
            return maxarma_likelihood;
        else
            return Math.exp((aic - 2D * (double)npar) * -0.5D);
    }

    public double getTimsacVariance()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return variance;
    }

    public double getInnovationVariance()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return armaVar;
    }

    public double getMean()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return z_mean;
    }

    public void setMean(double d)
    {
        center = false;
        z_mean = d;
    }

    public void setConvergenceTolerance(double d)
    {
        if(d < 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "convergenceTolerance", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("OutOfRange"+ aobj);
        }
        l_convergence_tolerance = d;
    }

    public double getConvergenceTolerance()
    {
        return l_convergence_tolerance;
    }

    public void setMaxIterations(int i)
    {
        if(i <= 0)
        {
            Object aobj[] = {
                "MaxIterations", new Integer(i)
            };
            System.out.println("NotPositive"+ aobj);
        } else
        {
            maxit = i;
        }
        compute_flag = false;
    }

    public void setConfidence(double d)
    {
        if(d <= 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "confidence", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("GreaterLess"+ aobj);
        }
        confidence = d;
    }

    public int getMaxIterations()
    {
        return maxit;
    }

    public void setEstimationMethod(int i)
    {
        if(i < 0 || i > 2)
            System.out.println("ARAutoUnivariate.InvalidEstimationMethod");
        else
            method = i;
        compute_flag = false;
    }

    public int getEstimationMethod()
    {
        return method;
    }

    public double[] getTimsacAR()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        double ad[] = new double[npar];
        System.arraycopy(par, 0, ad, 0, npar);
        return ad;
    }

    public double[] getAR()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        double ad[] = new double[npar];
        System.arraycopy(armaAR, 0, ad, 0, npar);
        return ad;
    }

    public double[] getResiduals()
    {
        if(residuals == null)
            return null;
        else
            return (double[])(double[])residuals.clone();
    }

    public int getOrder()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        if(!compute_flag)
            compute();
        return npar;
    }

    public int getMaxlag()
    {
        return maxlag;
    }

    public void setModelSelectionCriterion(int i)
    {
        if(i < 0 || i > 2)
        {
            Object aobj[] = {
                "infoCriterion", new Integer(i), "0", "2"
            };
            System.out.println("GreaterEqual"+ aobj);
        }
        l_infoCriterion = i;
    }

    public Logger getLogger()
    {
        return logger;
    }

    private void l_a2to_uni_ar()
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, TriangularMatrixSingularException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.InitialMAException
    {
        boolean flag = false;
        double d = 0.0D;
        double ad5[] = {
            0.0D, 0.0D, 0.0D
        };
        int l1 = maxlag;
        double ad4[] = new double[nobs];
        double ad3[] = new double[(nobs + 1) * (nobs + 1)];
        double ad[] = new double[maxlag];
        double ad2[] = new double[nobs];
        double ad1[] = new double[maxlag + 1];
        BLAS.set(maxlag + 1, 0.0D, ad1, 0);
        aic = 0.0D;
        aicc = 0.0D;
        bic = 0.0D;
        for(int i = 0; i < maxlag; i++)
            par[i] = (0.0D / 0.0D);

        npar = 0;
        int i1 = nobs + 1;
        byte byte0 = 2;
        BLAS.set((nobs + 1) * (nobs + 1), 0.0D, ad3, 0);
        int j1 = nobs;
        System.arraycopy(z, 0, ad4, 0, nobs);
        if(center)
            z_mean = BLAS.sum(j1, ad4, 0, 1) / (double)j1;
        BLAS.add(j1, -z_mean, ad4, 0, 1);
        d = BLAS.dot(j1, ad4, 0, ad4, 0) / (double)j1;
        if(iprint == 2)
        {
            keyMessage = "ARAutoUnivariate.FinerOutput1";
            Object aobj[] = {
                Double.toString(z_mean), Double.toString(d)
            };
            logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
            keyMessage = "ARAutoUnivariate.BlankLine";
            logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
        }
        int k1 = j1 - l1;
        int l = l1;
        hhReduction(ad4, ad2, k1, 0, l, i1, l1, ad3);
        l_a3to_uni_ar(ad3, l, l1, k1, byte0, i1, iprint, ad, variance, npar, aic, aicc, bic);
        for(int j = 0; j < npar; j++)
            par[j] = ad[j];

        if(npar > 0)
            p_constant[0] = z_mean * (1.0D - BLAS.sum(npar, ad, 0, 1));
        else
            p_constant[0] = z_mean;
        if(iprint > 0)
        {
            keyMessage = "ARAutoUnivariate.FinerOutput2";
            Object aobj1[] = {
                Integer.toString(npar)
            };
            logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj1);
            if(l_infoCriterion == 0)
            {
                keyMessage = "ARAutoUnivariate.FinerOutput3";
                Object aobj2[] = {
                    Double.toString(aic)
                };
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj2);
                keyMessage = "ARAutoUnivariate.BlankLine";
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj2);
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj2);
            } else
            if(l_infoCriterion == 1)
            {
                keyMessage = "ARAutoUnivariate.FinerOutput31";
                Object aobj3[] = {
                    Double.toString(aicc)
                };
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj3);
                keyMessage = "ARAutoUnivariate.BlankLine";
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj3);
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj3);
            } else
            {
                keyMessage = "ARAutoUnivariate.FinerOutput32";
                Object aobj4[] = {
                    Double.toString(bic)
                };
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj4);
                keyMessage = "ARAutoUnivariate.BlankLine";
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj4);
                logger.logp(Level.FINE, nameClass, nameMethod, keyMessage, aobj4);
            }
        }
        if(method == 0)
        {
            if(npar != 0)
            {
                armaAR = new double[npar];
                ARMA arma = new ARMA(npar, 0, z);
                arma.setMethod(0);
                arma.setMaxIterations(maxit);
                arma.compute();
                armaConstant = arma.getConstant();
                armaAR = arma.getAR();
                armaVar = arma.getInnovationVariance();
            } else
            {
                armaAR = EMPTY_DOUBLE_VECTOR;
                armaVar = BLAS.dot(j1, ad4, 0, ad4, 0) / (double)j1;
                armaConstant = z_mean;
            }
        } else
        if(method == 2)
        {
            ARMAMaxLikelihood armamaxlikelihood = new ARMAMaxLikelihood(npar, 0, z);
            armaAR = new double[npar];
            for(int k = 0; k < npar; k++)
                armaAR[k] = ad[k];

            try
            {
                armamaxlikelihood.setAR(armaAR);
                armamaxlikelihood.setMaxIterations(maxit);
                armamaxlikelihood.setMean(z_mean);
                armamaxlikelihood.setTolerance(l_convergence_tolerance);
                armamaxlikelihood.compute();
                armaVar = armamaxlikelihood.getInnovationVariance();
                armaConstant = armamaxlikelihood.getConstant();
                armaAR = armamaxlikelihood.getAR();
                maxarma_likelihood = armamaxlikelihood.getLikelihood();
            }
            catch(Exception ex)
            {
                armaVar = (0.0D / 0.0D);
                armaAR = EMPTY_DOUBLE_VECTOR;
                armaConstant = (0.0D / 0.0D);
				System.out.println(ex.toString());
            }
        } else
        if(npar > 0)
        {
            armaAR = new double[npar];
            ARMA arma1 = new ARMA(npar, 0, z);
            arma1.setMethod(1);
            arma1.setBackcasting(0, 9.9999999999999995E-007D);
            arma1.setMaxIterations(maxit);
            arma1.setConvergenceTolerance(l_convergence_tolerance);
            arma1.compute();
            armaConstant = arma1.getConstant();
            armaAR = arma1.getAR();
            armaVar = arma1.getInnovationVariance();
        } else
        {
            armaVar = BLAS.dot(j1, ad4, 0, ad4, 0) / (double)j1;
            armaConstant = z_mean;
            armaAR = EMPTY_DOUBLE_VECTOR;
        }
    }

    private void l_a3to_uni_ar(double ad[], int i, int j, int k, int l, int i1, int j1, 
            double ad1[], double d, int k1, double d1, double d2, double d3)
    {
        if(j1 == 2)
        {
            keyMessage = "ARAutoUnivariate.BlankLine";
            Object aobj[] = {
                new String("")
            };
            logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
            logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
            keyMessage = "ARAutoUnivariate.FinerOutput4";
            logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
            keyMessage = "ARAutoUnivariate.BlankLine";
            logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
        }
        double ad2[] = new double[j + 1];
        double ad3[] = new double[j + 1];
        double ad4[] = new double[j + 1];
        double ad5[] = new double[j + 1];
        l_a4to_uni_ar(ad, k, i, i1, ad5, ad2, ad3, ad4);
        aicDisplay(ad2, ad3, ad4, ad5, i, l);
        if(l >= 2)
        {
            int k2 = 0;
            if(j1 == 2)
            {
                keyMessage = "ARAutoUnivariate.BlankLine";
                Object aobj1[] = {
                    new String("")
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj1);
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj1);
                keyMessage = "ARAutoUnivariate.FinerOutput5";
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj1);
                keyMessage = "ARAutoUnivariate.BlankLine";
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj1);
                keyMessage = "ARAutoUnivariate.FinerOutput6";
                Object aobj3[] = {
                    Integer.toString(k2), Integer.toString(k2), Double.toString(ad5[0])
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj3);
            }
            for(int l2 = 1; l2 <= i; l2++)
            {
                int l1 = l2 - 1;
                int i3 = l2 + 1;
                l_a8to_uni_ar(ad, l2, i, i1, ad1);
                if(j1 != 2)
                    continue;
                keyMessage = "ARAutoUnivariate.FinerOutput6";
                Object aobj2[] = {
                    Integer.toString(l2), Integer.toString(i3 - 1), Double.toString(ad5[i3 - 1])
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj2);
                for(int i2 = 1; i2 <= l2; i2++)
                {
                    keyMessage = "ARAutoUnivariate.FinerOutput7";
                    Object aobj4[] = {
                        new String("a"), Integer.toString(i2 - 1), Double.toString(ad1[i2 - 1])
                    };
                    logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj4);
                }

                keyMessage = "ARAutoUnivariate.BlankLine";
                Object aobj5[] = {
                    new String("")
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj5);
            }

        }
        if(npar >= 1)
            l_a8to_uni_ar(ad, npar, i, i1, ad1);
        int j2 = j + 1;
        int j3 = k + j;
    }

    private void l_a4to_uni_ar(double ad[], int i, int j, int k, double ad1[], double ad2[], double ad3[], 
            double ad4[])
    {
        int i1 = 0;
        boolean flag = false;
        double d1 = 0.0D;
        double d3 = 0.0D;
        double d = i;
        i1 = j + 1;
        d1 = 0.0D;
        for(int j1 = 1; j1 <= i1; j1++)
        {
            int l = j1 - 1;
            int k1 = (i1 - j1) + 1;
            double d4 = ad[(k1 - 1) * k + (i1 - 1)];
            d1 += d4 * d4;
            ad1[k1 - 1] = d1 / d;
            double d2 = 3.1415926535897931D;
            d2 *= 2D;
            if(ad1[k1 - 1] == 0.0D)
            {
                ad2[k1 - 1] = -1E+030D;
                ad3[k1 - 1] = -1E+030D;
                ad4[k1 - 1] = -1E+030D;
            } else
            {
                ad2[k1 - 1] = d * Math.log(ad1[k1 - 1]) + 2D * (double)k1 + (d * Math.log(d2) + 1.0D);
                ad3[k1 - 1] = d * Math.log(ad1[k1 - 1]) + (2D * (double)k1 * d) / (d - (double)k1 - 1.0D) + d * (Math.log(d2) + 1.0D);
                ad4[k1 - 1] = d * Math.log(ad1[k1 - 1]) + (double)k1 * Math.log(d) + d * (Math.log(d2) + 1.0D);
            }
        }

    }

    private void l_a5to_uni_ar(double ad[], double ad1[], int i, int j, int k)
        throws TriangularMatrixSingularException
    {
        double d4 = 2.2204460492503001E-015D;
        for(int l3 = 1; l3 <= k; l3++)
        {
            int i2 = l3 - 1;
            double d2 = 0.0D;
            for(int k2 = l3; k2 <= j; k2++)
            {
                int l = k2 - 1;
                ad1[l] = ad[l * i + i2];
                d2 += ad1[l] * ad1[l];
            }

            double d1;
            if(d2 <= d4)
            {
                d1 = 0.0D;
            } else
            {
                d1 = Math.sqrt(d2);
                double d = ad[i2 * i + i2];
                if(d >= 0.0D)
                    d1 = -d1;
                ad1[i2] = d - d1;
                d2 += -d * d1;
                int l1 = l3 + 1;
                for(int l2 = l1; l2 <= j; l2++)
                {
                    int i1 = l2 - 1;
                    ad[i1 * i + i2] = 0.0D;
                }

                if(l3 != k)
                {
                    for(int k3 = l1; k3 <= k; k3++)
                    {
                        int j2 = k3 - 1;
                        double d3 = 0.0D;
                        for(int i3 = l3; i3 <= j; i3++)
                        {
                            int j1 = i3 - 1;
                            d3 += ad1[j1] * ad[j1 * i + j2];
                        }

                        d3 /= d2;
                        for(int j3 = l3; j3 <= j; j3++)
                        {
                            int k1 = j3 - 1;
                            ad[k1 * i + j2] += -ad1[k1] * d3;
                        }

                    }

                }
            }
            ad[i2 * i + i2] = d1;
            if(l3 != k && ad[i2 * i + i2] == 0.0D)
            {
                Object aobj[] = {
                    new Integer(i2)
                };
                throw new TriangularMatrixSingularException("ARAutoUnivariate.SingularTriMatrix"+ aobj);
            }
        }

    }

    private void aicDisplay(double ad[], double ad1[], double ad2[], double ad3[], int i, int j)
    {
        int i2 = i + 1;
        npar = 0;
        variance = ad3[0];
        aic = ad[0];
        aicc = ad1[0];
        bic = ad2[0];
        if(l_infoCriterion == 0)
        {
            for(int j1 = 1; j1 <= i; j1++)
            {
                int k = j1 - 1;
                if(ad[k + 1] < aic)
                {
                    npar = j1;
                    variance = ad3[k + 1];
                    aic = ad[k + 1];
                    aicc = ad1[j1];
                    bic = ad2[j1];
                }
            }

        }
        if(l_infoCriterion == 1)
        {
            for(int k1 = 1; k1 <= i; k1++)
            {
                int l = k1 - 1;
                if(ad1[l + 1] < aicc)
                {
                    npar = k1;
                    variance = ad3[l + 1];
                    aicc = ad1[l + 1];
                    aic = ad[k1];
                    bic = ad2[k1];
                }
            }

        }
        if(l_infoCriterion == 2)
        {
            for(int l1 = 1; l1 <= i; l1++)
            {
                int i1 = l1 - 1;
                if(ad2[i1 + 1] < bic)
                {
                    npar = l1;
                    variance = ad3[i1 + 1];
                    bic = ad2[i1 + 1];
                    aic = ad[l1];
                    aicc = ad1[l1];
                }
            }

        }
        if(j <= 0)
            return;
        if(iprint == 2)
            if(l_infoCriterion == 0)
            {
                keyMessage = "ARAutoUnivariate.FinerOutput8";
                Object aobj[] = {
                    new String("")
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj);
                keyMessage = "ARAutoUnivariate.FinerOutput9";
                Object aobj3[] = {
                    Integer.toString(npar), Double.toString(variance), Double.toString(aic)
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj3);
            } else
            if(l_infoCriterion == 1)
            {
                keyMessage = "ARAutoUnivariate.FinerOutput81";
                Object aobj1[] = {
                    new String("")
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj1);
                keyMessage = "ARAutoUnivariate.FinerOutput9";
                Object aobj4[] = {
                    Integer.toString(npar), Double.toString(variance), Double.toString(aicc)
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj4);
            } else
            {
                keyMessage = "ARAutoUnivariate.FinerOutput82";
                Object aobj2[] = {
                    new String("")
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj2);
                keyMessage = "ARAutoUnivariate.FinerOutput9";
                Object aobj5[] = {
                    Integer.toString(npar), Double.toString(variance), Double.toString(bic)
                };
                logger.logp(Level.FINER, nameClass, nameMethod, keyMessage, aobj5);
            }
    }

    private void l_a8to_uni_ar(double ad[], int i, int j, int k, double ad1[])
    {
        int j2 = j + 1;
        ad1[i - 1] = ad[(i - 1) * k + (j2 - 1)] / ad[(i - 1) * k + (i - 1)];
        if(i == 1)
            return;
        int k2 = i - 1;
        for(int l1 = 1; l1 <= k2; l1++)
        {
            int i1 = l1 - 1;
            int k1 = i - l1;
            double d = ad[(k1 - 1) * k + (j2 - 1)];
            int l = k1 + 1;
            for(int i2 = l; i2 <= i; i2++)
            {
                int j1 = i2 - 1;
                d += -ad1[j1] * ad[(k1 - 1) * k + j1];
            }

            ad1[k1 - 1] = d / ad[(k1 - 1) * k + (k1 - 1)];
        }

    }

    private void hhReduction(double ad[], double ad1[], int i, int j, int k, int l, int i1, 
            double ad2[])
        throws TriangularMatrixSingularException
    {
        int l2 = 0;
        int i3 = 1;
        int j1 = Math.min(i, l);
        int i2 = k + 1;
        int j2 = j1;
        l_a10o_uni_ar(ad, j, j1, k, l, l2, i1, ad2);
        l_a5to_uni_ar(ad2, ad1, l, j1, i2);
        if(j2 >= i)
            return;
        int k1;
        for(; j2 < i; j2 += k1)
        {
            k1 = Math.min(i - j2, l - i2);
            int l1 = k1 + i2;
            int k2 = j + j2;
            l_a10o_uni_ar(ad, k2, k1, k, l, i3, i1, ad2);
            l_a5to_uni_ar(ad2, ad1, l, l1, i2);
        }

    }

    private void l_a10o_uni_ar(double ad[], int i, int j, int k, int l, int i1, int j1, 
            double ad1[])
    {
        int j4 = 0;
        if(k != j1)
            j4 = 1;
        int i4 = k + 1;
        int l2 = 0;
        if(i1 == 1)
            l2 = i4;
        for(int j2 = 1; j2 <= j; j2++)
        {
            int k1 = j2 - 1;
            int i3 = j2 + l2;
            int k3 = i + j1 + j2;
            ad1[(i3 - 1) * l + (i4 - 1)] = ad[k3 - 1];
            for(int j3 = 1; j3 <= j1; j3++)
            {
                int i2 = j3 - 1;
                k3--;
                int l3 = j3 + j4;
                ad1[(i3 - 1) * l + (l3 - 1)] = ad[k3 - 1];
            }

        }

        if(j4 != 1)
            return;
        for(int k2 = 1; k2 <= j; k2++)
        {
            int l1 = k2 - 1;
            ad1[l1 * l + 0] = 1.0D;
        }

    }



}