
package utility.forecasting;


import java.io.PrintStream;
import java.io.Serializable;
import java.util.Date;

//            Summary, ARMA

public class ARMAMaxLikelihood
    implements Serializable
{
	  private static final long serialVersionUID = 0xe4ef0d5821f7c013L;
    private static final String NEWLINE = System.getProperty("line.separator");
    private ARMA l_arma;
    private double l_f[][];
    private double l_z[];
    private double l_ar[];
    private double l_ma[];
    private double l_alph[];
    private double l_hs[];
    private double l_g[];
    private double l_forecast[];
    private double l_dev[];
    private double l_psiWeights[];
    private double l_residuals[];
    private double l_confidence;
    private double l_aic;
    private double l_constant;
    private double l_likelihood;
    private double l_variance;
    private double l_w_mean;
    private double l_eps3;
    private double l_eps4;
    private double l_epsa;
    private int l_q;
    private int l_p;
    private int l_nobs;
    private int l_init_arlag;
    private int l_init_malag;
    private int l_maxIterations;
    private int l_backwardOrigin;
    private int l_init_mean;
    private int iprint;
    private boolean arSet;
    private boolean maSet;
    private boolean notComputed;
	
    public static class NonStationaryException extends Exception
    {

        private static final long serialVersionUID = 0x640377bfbac4aa69L;

        public NonStationaryException(String s)
        {
            super(s);
        }

        public NonStationaryException(String s, Object aobj[])
        {
            super(s+ aobj);
        }
    }

    public static class NonInvertibleException extends Exception
    {

        private static final long serialVersionUID = 0xd4302db9036e0d15L;

        public NonInvertibleException(String s)
        {
            super(s);
        }

        public NonInvertibleException(String s, Object aobj[])
        {
            super(s+ aobj);
        }
    }

    public static class InitialMAException extends Exception
    {

        public InitialMAException(String s)
        {
            super(s);
        }

        public InitialMAException(String s, Object aobj[])
        {
            super(s+ aobj);
        }
    }

    private class return_m7x
    {

        public double phai;
        public double sigma2;
        public int iswro;
        public int iterations;
        public double g[];
        public double x[];
        public double hs[];
        public final ARMAMaxLikelihood this$0;

        private return_m7x()
        {
			super();
            this$0 = ARMAMaxLikelihood.this;
            
        }

    }

    private class return_m5x
    {

        public double f;
        public double sd;
        public final ARMAMaxLikelihood this$0;

        private return_m5x()
        {
			super();
            this$0 = ARMAMaxLikelihood.this;

        }

    }

    private class return_m4x
    {

        public double f0;
        public double sd;
        public double g[];
        public final ARMAMaxLikelihood this$0;

        private return_m4x()
        {
			super();
            this$0 = ARMAMaxLikelihood.this;

        }

    }

    private class return_m3x
    {

        public double tl;
        public double sigma2;
        public double p0[];
        public double g[];
        public final ARMAMaxLikelihood this$0;

        private return_m3x()
        {
			super();
            this$0 = ARMAMaxLikelihood.this;
       
        }

    }


    public ARMAMaxLikelihood(int i, int j, double ad[])
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException
    {
        l_backwardOrigin = 0;
        l_init_mean = 0;
        iprint = 0;
        arSet = false;
        maSet = false;
        notComputed = false;
        boolean flag = false;
        if(ad == null)
        {
            Object aobj[] = {
                "z", "null"
            };
            System.out.println("CannotBe"+aobj);
        }
        if(i < 0)
        {
            Object aobj1[] = {
                "p", new Integer(i)
            };
            System.out.println("Negative"+aobj1);
        }
        l_p = i;
        if(j < 0)
        {
            Object aobj2[] = {
                "q", new Integer(j)
            };
            System.out.println("Negative"+aobj2);
        }
        l_q = j;
        if(i + j < 1)
        {
            Object aobj3[] = {
                "p+q", new Integer(i + j), new Integer(0)
            };
            System.out.println("NotLargeEnough"+aobj3);
        }
        if(ad.length < 2)
        {
            Object aobj4[] = {
                "z.length", new Integer(ad.length), new Integer(2)
            };
            System.out.println("NotLargeEnough"+aobj4);
        } else
        if(l_isanan(ad.length, ad, 1) != 0)
        {
            Object aobj5[] = {
                (new StringBuilder()).append("z[").append(l_isanan(ad.length, ad, 1) - 1).append("]").toString(), "NaN"
            };
            System.out.println("CannotBe"+aobj5);
        }
        if(i + j + 1 >= ad.length)
        {
            Object aobj6[] = {
                "p+q+1", new Integer(i + j + 1), "z.length", new Integer(ad.length)
            };
            System.out.println("ARMAMaxLikelihood.TooBig"+aobj6);
        }
        l_nobs = ad.length;
        l_z = (double[])(double[])ad.clone();
        l_ar = new double[l_p];
        l_ma = new double[l_q];
        l_g = new double[l_q + l_p];
        if(l_p + l_q > 0)
        {
            l_hs = new double[(l_p + l_q) * (l_p + l_q)];
            l_alph = new double[l_p + l_q];
        } else
        {
            l_hs = null;
            l_alph = null;
        }
        l_init_arlag = 1;
        l_init_malag = 1;
        l_confidence = 0.94999999999999996D;
        l_maxIterations = 300;
        Summary summary = new Summary();
        summary.update(l_z);
        l_w_mean = summary.getMean();
        l_arma = new ARMA(l_p, l_q, l_z);
        l_arma.setMethod(0);

        try
        {
            l_arma.compute();
            l_ar = l_arma.getAR();
            l_ma = l_arma.getMA();
            l_constant = l_arma.getConstant();
            maSet = true;
            arSet = true;
        }
        catch(Exception exception)
        {
            for(int k = 0; k < l_p; k++)
                l_ar[k] = 0.90000000000000002D / (double)l_p;

            for(int l = 0; l < l_q; l++)
                l_ma[l] = 0.90000000000000002D / (double)l_q;

            l_constant = 0.0D;
            maSet = false;
            arSet = false;
        }
        l_eps3 = 2.2204460490000001E-016D;
        l_eps4 = 2.2204460490000001E-016D;
        l_epsa = 0.0001D;
        l_forecast = new double[l_nobs];
        l_dev = null;
        l_psiWeights = null;
        l_backwardOrigin = 0;
    }

    public void compute()
        throws NonStationaryException, NonInvertibleException, InitialMAException
    {
        notComputed = true;
        l_maxarma_driver(l_z, l_ar, l_ma, iprint, l_maxIterations, l_variance, l_likelihood, l_constant, l_init_mean, l_w_mean, l_alph, l_hs);
        notComputed = false;
    }

    private int l_isanan(int i, double ad[], int j)
    {
        int l = 0;
        int i1 = 1;
        int k = 1;
        do
        {
            if(i1 > i)
                break;
            if(Double.isNaN(ad[k - 1]))
            {
                l = i1;
                break;
            }
            k += j;
            i1++;
        } while(true);
        return l;
    }

    public void setConfidence(double d)
    {
        if(d <= 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "confidence", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("GreaterLess"+aobj);
        }
        l_confidence = d;
    }

    public int getBackwardOrigin()
    {
        return l_backwardOrigin;
    }

    public double getConfidence()
    {
        return l_confidence;
    }

    public int getP()
    {
        return l_p;
    }

    public int getQ()
    {
        return l_q;
    }

    public double[] getGradients()
    {
        if(l_g == null)
            return null;
        else
            return (double[])(double[])l_g.clone();
    }

    public double[] getAR()
    {
        if(l_ar == null)
            return null;
        else
            return (double[])(double[])l_ar.clone();
    }

    public void setAR(double ad[])
    {
        int i;
        if(ad == null)
            i = 0;
        else
            i = ad.length;
        if(l_p != i)
        {
            Object aobj[] = {
                new Integer(ad.length), new Integer(l_p)
            };
            System.out.println("ARMAMaxLikelihood.InitialAR1"+aobj);
            return;
        }
        if(i == 0)
            l_ar = null;
        else
            l_ar = (double[])(double[])ad.clone();
        arSet = true;
    }

    public void setBackwardOrigin(int i)
    {
        int j = 0;
        if(i < 0)
        {
            Object aobj[] = {
                "backwardOrigin", new Integer(i)
            };
            System.out.println("NotPositive"+aobj);
        }
        if(l_p > l_q)
            j = l_p;
        else
            j = l_q;
        if(i > l_nobs - j)
        {
            Object aobj1[] = {
                new Integer(i), new Integer(l_nobs - j)
            };
            System.out.println("ARMAMaxLikelihood.BackwardOrigin"+aobj1);
        }
        l_backwardOrigin = i;
    }

    public double[] getMA()
    {
        if(l_ma == null)
            return null;
        else
            return (double[])(double[])l_ma.clone();
    }

    public void setMA(double ad[])
    {
        int i = 0;
        if(ad != null)
            i = ad.length;
        else
            i = 0;
        if(l_q != i)
        {
            Object aobj[] = {
                new Integer(ad.length), new Integer(l_q)
            };
            System.out.println("ARMAMaxLikelihood.InitialMA1"+aobj);
            maSet = false;
            return;
        }
        if(i > 0)
            l_ma = (double[])(double[])ad.clone();
        else
            l_ma = null;
        maSet = true;
    }

    public double getConstant()
    {
        return l_constant;
    }

    public double[] getForecast(int i)
        throws NonStationaryException, NonInvertibleException, InitialMAException, ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException
    {
        boolean flag = false;
        if(i <= 0)
            return null;
        if(notComputed)
            compute();
        double ad[][] = forecast(i);
        l_forecast = new double[i + l_backwardOrigin];
        for(int j = 0; j < l_backwardOrigin; j++)
            l_forecast[j] = ad[0][j];

        for(int k = 0; k < i; k++)
            l_forecast[k + l_backwardOrigin] = ad[k][l_backwardOrigin];

        return (double[])(double[])l_forecast.clone();
    }

    public double[][] forecast(int i)
        throws NonStationaryException, NonInvertibleException, InitialMAException, ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException
    {
        if(i <= 0)
            return (double[][])null;
        if(notComputed)
            compute();
        ARMA arma = new ARMA(l_p, l_q, l_z);
        arma.setArmaInfo(l_constant, l_ar, l_ma, l_variance);
        arma.setBackwardOrigin(l_backwardOrigin);
        arma.setConfidence(l_confidence);

        l_f = arma.forecast(i);

        l_psiWeights = arma.getPsiWeights();
        if(l_backwardOrigin > 0)
        {
            l_dev = new double[l_backwardOrigin + i];
            double ad[] = arma.getDeviations();
            l_residuals = new double[l_backwardOrigin];
            for(int j = 0; j < l_backwardOrigin; j++)
            {
                int l = (l_z.length - l_backwardOrigin) + j;
                l_residuals[j] = l_z[l] - l_f[0][j];
                l_dev[j] = ad[0];
            }

            for(int k = l_backwardOrigin; k < l_backwardOrigin + i; k++)
                l_dev[k] = ad[k - l_backwardOrigin];

        } else
        {
            l_dev = arma.getDeviations();
            l_residuals = null;
        }
        return (double[][])(double[][])l_f.clone();
    }

    public boolean isStationary(double ad[])
    {
        boolean flag = true;
        double ad1[] = null;
        Complex acomplex[] = null;
        if(ad.length <= 0)
            return false;
        ad1 = new double[ad.length + 1];
        ad1[0] = 1.0D;
        for(int i = 0; i < ad.length; i++)
            ad1[i + 1] = -ad[i];

        ZeroPolynomial zeropolynomial = new ZeroPolynomial();
        try
        {
            acomplex = zeropolynomial.computeRoots(ad1);
        }
        catch(Exception exception)
        {
            System.out.println(exception);
        }
        int j = 0;
        do
        {
            if(j >= ad.length)
                break;
            if(Complex.abs(Complex.multiply(acomplex[j], acomplex[j])) <= 1.0D)
            {
                flag = false;
                break;
            }
            j++;
        } while(true);
        return flag;
    }

    public boolean isInvertible(double ad[])
    {
        boolean flag = true;
        double ad1[] = null;
        Complex acomplex[] = null;
        if(ad.length <= 0)
            return false;
        ad1 = new double[ad.length + 1];
        ad1[0] = 1.0D;
        for(int i = 0; i < ad.length; i++)
            ad1[i + 1] = ad[i];

        ZeroPolynomial zeropolynomial = new ZeroPolynomial();
        try
        {
            acomplex = zeropolynomial.computeRoots(ad1);
        }
        catch(Exception exception)
        {
            System.out.println(exception);
        }
        int j = 0;
        do
        {
            if(j >= ad.length)
                break;
            if(Complex.abs(Complex.multiply(acomplex[j], acomplex[j])) <= 1.0D)
            {
                flag = false;
                break;
            }
            j++;
        } while(true);
        return flag;
    }

    public double getLikelihood()
        throws NonStationaryException, NonInvertibleException, InitialMAException
    {
        if(notComputed)
            compute();
        return l_likelihood;
    }

    public int getMaxIterations()
    {
        return l_maxIterations;
    }

    public double[] getResiduals()
    {
        if(l_residuals == null)
            return null;
        else
            return (double[])(double[])l_residuals.clone();
    }

    public void setMaxIterations(int i)
        throws IllegalArgumentException
    {
        if(i <= 0)
        {
            Object aobj[] = {
                "maxIterations", new Integer(i)
            };
            System.out.println("NotPositive"+aobj);
        }
        l_maxIterations = i;
    }

    public void setConstant(double d)
    {
        l_constant = d;
    }

    public void setMean(double d)
    {
        l_w_mean = d;
        l_init_mean = 1;
    }

    public void setTolerance(double d)
    {
        l_eps3 = d;
        l_eps4 = d;
    }

    public double getGradientTolerance()
    {
        return l_epsa;
    }

    public void setGradientTolerance(double d)
    {
        l_epsa = d;
    }

    public double getTolerance()
    {
        return l_eps4;
    }

    public double getMean()
    {
        return l_w_mean;
    }

    public double[] getTimeSeries()
    {
        if(l_z == null)
            return null;
        else
            return (double[])(double[])l_z.clone();
    }

    public double getInnovationVariance()
    {
        return l_variance;
    }

    public double[] getDeviations()
    {
        if(l_dev == null)
            return null;
        else
            return (double[])(double[])l_dev.clone();
    }

    public double[] getPsiWeights()
    {
        if(l_psiWeights == null)
            return null;
        else
            return (double[])(double[])l_psiWeights.clone();
    }

    private void l_maxarma_driver(double ad[], double ad1[], double ad2[], int i, int j, double d, 
            double d1, double d2, int k, double d3, 
            double ad3[], double ad4[])
        throws NonStationaryException, NonInvertibleException, InitialMAException
    {
        Object obj = null;
        Object obj1 = null;
        double ad7[] = null;
        double d4 = 0.0D;
        double d6 = 0.0D;
        double ad8[] = null;
        Object obj2 = null;
        Object obj3 = null;
        return_m3x return_m3x1 = new return_m3x();
        int j5 = i;
        int l5 = 0;
        if(ad1 != null)
            l5 = ad1.length;
        int k5 = 0;
        if(ad2 != null)
            k5 = ad2.length;
        int l6 = ad.length;
        int k6 = j;
        ad8 = new double[l6];
        if(k5 + l5 > 0)
        {
            ad7 = new double[l5 + k5];
        } else
        {
            ad4 = null;
            ad3 = null;
        }
        boolean flag2 = false;
        if(l5 > 0)
        {
            for(int i6 = l5; i6 >= 1; i6--)
            {
                int j6 = i6 - 1;
                double ad5[] = new double[i6 + 1];
                Complex acomplex[] = new Complex[i6];
                ad5[0] = 1.0D;
                for(int l = 2; l <= i6 + 1; l++)
                {
                    int i4 = l - 1;
                    ad5[i4] = -ad1[i4 - 1];
                }

                ZeroPolynomial zeropolynomial = new ZeroPolynomial();
                try
                {
                    acomplex = zeropolynomial.computeRoots(ad5);
                }
                catch(Exception exception)
                {
                    flag2 = true;
                    boolean flag = true;
                    break;
                }
                boolean flag1 = false;
                int i1 = 1;
                do
                {
                    if(i1 > i6)
                        break;
                    int j4 = i1 - 1;
                    if(Complex.abs(Complex.multiply(acomplex[j4], acomplex[j4])) <= 1.0D)
                    {
                        flag2 = true;
                        flag1 = true;
                        break;
                    }
                    i1++;
                } while(true);
                if(!flag1)
                    break;
                if(i6 == 1)
                {
                    ad1[0] = (Math.abs(ad1[0]) / ad1[0]) * 0.80000000000000004D;
                    ad1[0] = ((Math.abs(ad1[0]) / ad1[0]) * 0.80000000000000004D) / (double)l5;
                    ad1[0] = 0.0D;
                } else
                {
                    ad1[j6] = 0.0D;
                    ad1[j6] = 0.90000000000000002D / (double)l5;
                }
            }

            if(flag2)
                System.out.println(this+ "ARMAMaxLikelihood.InitialAR2");
            for(int j1 = 0; j1 < l5; j1++)
                ad1[j1] = -ad1[j1];

        }
        if(k5 > 0)
        {
            double ad6[] = new double[k5 + 1];
            Complex acomplex1[] = new Complex[k5];
            ad6[0] = 1.0D;
            for(int k1 = 1; k1 < k5 + 1; k1++)
                ad6[k1] = ad2[k1 - 1];

            ZeroPolynomial zeropolynomial1 = new ZeroPolynomial();
            try
            {
                acomplex1 = zeropolynomial1.computeRoots(ad6);
            }
            catch(Exception exception1)
            {
                throw new InitialMAException("ARMAMaxLikelihood.InitialMA2", null);
            }
            for(int l1 = 1; l1 <= k5; l1++)
            {
                int k4 = l1 - 1;
                if(Complex.abs(Complex.multiply(acomplex1[k4], acomplex1[k4])) <= 1.0D)
                    throw new InitialMAException("ARMAMaxLikelihood.InitialMA2", null);
            }

        }
        if(l5 > 0)
        {
            for(int i2 = 0; i2 < l5; i2++)
                ad7[i2] = ad1[i2];

        }
        if(k5 > 0)
        {
            for(int j2 = 0; j2 < k5; j2++)
                ad7[j2 + l5] = ad2[j2];

        }
        for(int k2 = 0; k2 < l6; k2++)
            ad8[k2] = ad[k2];

        double d7;
        if(k5 + l5 == 0)
        {
            Summary summary = new Summary();
            summary.update(ad8);
            d7 = summary.getMean();
        } else
        if(k > 0)
        {
            d7 = d3;
        } else
        {
            Summary summary1 = new Summary();
            summary1.update(ad8);
            d7 = summary1.getMean();
        }
        for(int l2 = 0; l2 < l6; l2++)
            ad8[l2] -= d7;

        if(k5 + l5 == 0)
        {
            if(k > 0)
                d3 = d7;
            Summary summary2 = new Summary();
            summary2.update(ad8);
            d = summary2.getVariance();
            double d8 = 6.2831853071795862D;
            d1 = (double)l6 * (Math.log(d8 * d) + 1.0D);
            d2 = d7;
        } else
        {
            return_m3x return_m3x2 = l_m3x_arma(d6, d4, ad8, l6, ad7, l5, k5, j5, k6, ad3, ad4);
            d6 = return_m3x2.tl;
            d4 = return_m3x2.sigma2;
            for(int i3 = 0; i3 < ad7.length; i3++)
            {
                ad7[i3] = return_m3x2.p0[i3];
                l_g[i3] = return_m3x2.g[i3];
            }

            for(int j3 = 0; j3 < l5; j3++)
            {
                int l4 = j3 - 1;
                ad1[j3] = -ad7[j3];
                l_ar[j3] = ad1[j3];
            }

            Date date = new Date();
            if(l5 > 0 && !isStationary(ad1))
                throw new NonStationaryException("ARMAMaxLikelihood.NonStationary", null);
            for(int k3 = l5; k3 < l5 + k5; k3++)
            {
                int i5 = k3 - 1;
                ad2[k3 - l5] = -ad7[k3];
                l_ma[k3 - l5] = ad2[k3 - l5];
            }

            if(k5 > 0 && !isInvertible(ad2))
                throw new NonInvertibleException("ARMAMaxLikelihood.NonInvertible", null);
            if(l5 > 0)
            {
                double d5 = 0.0D;
                for(int l3 = 0; l3 < l5; l3++)
                    d5 += ad1[l3];

                d2 = d7 * (1.0D - d5);
            } else
            {
                d2 = d7;
            }
            d1 = d6;
            d = d4;
        }
        l_likelihood = d1;
        l_variance = d;
        l_constant = d2;
        l_w_mean = d7;
    }

    private int l_m8x_arma(double ad[], int i, double ad1[], double ad2[])
    {
        int i5 = 0;
        double ad3[] = null;
        double d3 = 0.0D;
        double d4 = 1.0D;
        double d5 = 0.99999000000000005D;
        if(i <= 0)
            return 1;
        ad3 = new double[i * i];
        for(int j = 0; j < i; j++)
            ad3[j] = 0.0D;

        for(int k = 1; k <= i; k++)
        {
            int i1 = k - 1;
            int k4 = i - k;
            int l4 = k4 + 1;
            double d = ad[l4 - 1];
            if(d >= d5)
            {
                i5 = 1;
                d = d5;
            } else
            if(d <= -d5)
            {
                i5 = 1;
                d = -d5;
            }
            ad1[l4 - 1] = d;
            if(k4 == 0)
                continue;
            double d1 = d4 / (d4 - Math.pow(d, 2D));
            for(int j2 = 1; j2 <= k4; j2++)
            {
                int l2 = j2 - 1;
                int j4 = l4 - j2;
                ad3[l2] = ad[j4 - 1];
            }

            for(int j3 = 1; j3 <= k4; j3++)
            {
                int l3 = j3 - 1;
                ad[l3] = (ad[l3] - d * ad3[l3]) * d1;
            }

        }

        for(int l = 1; l <= i; l++)
        {
            int j1 = l - 1;
            double d2 = ad1[j1];
            ad[j1] = d2;
            if(l != 1)
            {
                int l1 = l - 1;
                for(int k2 = 1; k2 <= l1; k2++)
                {
                    int i3 = k2 - 1;
                    ad[i3] += d2 * ad3[i3];
                }

            }
            int i2 = l + 1;
            for(int k3 = 1; k3 <= l; k3++)
            {
                int i4 = k3 - 1;
                int k1 = i2 - k3;
                ad3[i4] = ad[k1 - 1];
            }

        }

        return i5;
    }

    private return_m5x l_m5x_arma(double d, double d1, double ad[], int i, double ad1[], 
            int j, int k)
        throws NonStationaryException
    {
        int l12 = 0;
        boolean flag = false;
        double ad2[] = null;
        double ad3[] = null;
        double ad4[] = null;
        double ad5[] = null;
        double ad6[] = null;
        double ad7[] = null;
        double ad8[] = null;
        double ad9[] = null;
        double ad10[] = null;
        double d19 = 0.0D;
        double d20 = 1.0D;
        double d21 = 9.9999999999999995E-007D;
        return_m5x return_m5x1 = new return_m5x();
        return_m5x1.f = 0.0D;
        return_m5x1.sd = 0.0D;
        ad2 = new double[j + k];
        ad3 = new double[j + k + 1];
        ad4 = new double[j + k + 2];
        if(j > k)
            ad5 = new double[j * j];
        else
            ad5 = new double[(k + 1) * (k + 1)];
        ad6 = new double[j + k + 2];
        ad7 = new double[j + k + 1];
        ad8 = new double[j + k + 1];
        ad9 = new double[j + k + 1];
        ad10 = new double[j + k + 1];
        for(int l = 0; l < j + k; l++)
            ad2[l] = d19;

        for(int i1 = 0; i1 < j + k + 2; i1++)
            ad4[i1] = d19;

        for(int j1 = 0; j1 < j + k + 1; j1++)
            ad3[j1] = d19;

        if(j > 0)
        {
            for(int k1 = 1; k1 <= j; k1++)
            {
                int k7 = k1 - 1;
                ad4[k7] = ad1[k7];
            }

        }
        if(k > 0)
        {
            for(int l1 = 1; l1 <= k; l1++)
            {
                int l7 = l1 - 1;
                int j12 = j + l1;
                ad2[l7] = ad1[j12 - 1];
            }

        }
        int j13 = k + 1;
        int k13 = Math.max(j, j13);
        double ad11[] = l_m6x_arma(ad5, ad4, ad2, j, k, k13);
        for(int i2 = 0; i2 < ad5.length; i2++)
            ad5[i2] = ad11[i2];

        double d10 = ad5[0];
        for(int j2 = 1; j2 <= k13; j2++)
        {
            int i8 = j2 - 1;
            ad6[i8] = ad5[i8];
        }

        int l13 = k13 - 1;
        int i14 = k13 + 1;
        if(l13 > 0)
        {
            for(int k2 = 1; k2 <= l13; k2++)
            {
                int j8 = k2 - 1;
                ad3[j8] = ad6[j8 + 1];
            }

        }
        double d14 = d19;
        for(int l2 = 1; l2 <= k13; l2++)
        {
            int k8 = l2 - 1;
            int k6 = i14 - l2;
            d14 += ad4[k8] * ad6[k6 - 1];
        }

        ad3[k13 - 1] = -d14;
        double d2 = -d20 / d10;
        for(int i3 = 1; i3 <= k13; i3++)
        {
            int l8 = i3 - 1;
            ad9[l8] = ad3[l8];
        }

        double d6 = ad[0];
        for(int j3 = 1; j3 <= k13; j3++)
        {
            int i9 = j3 - 1;
            ad10[i9] = d19;
        }

        double d12 = Math.pow(d6, 2D) / d10;
        double d13 = Math.log(d10);
        int j14 = 2;
        do
        {
            if(j14 > i)
                break;
            int l14 = j14 - 1;
            double d17 = ad9[0] * ad9[0];
            double d11 = d10 + d2 * d17;
            l12 = j14;
            if(l13 > 0)
            {
                for(int k3 = 1; k3 <= l13; k3++)
                {
                    int j9 = k3 - 1;
                    ad8[j9] = ad10[j9 + 1];
                }

            }
            double d15 = d19;
            for(int l3 = 1; l3 <= k13; l3++)
            {
                int k9 = l3 - 1;
                int l6 = i14 - l3;
                d15 += ad4[k9] * ad10[l6 - 1];
            }

            ad8[k13 - 1] = -d15;
            double d7 = d6 / d10;
            for(int i4 = 1; i4 <= k13; i4++)
            {
                int l9 = i4 - 1;
                ad10[l9] = ad8[l9] + d7 * ad3[l9];
            }

            if(l13 > 0)
            {
                for(int j4 = 1; j4 <= l13; j4++)
                {
                    int i10 = j4 - 1;
                    ad7[i10] = ad9[i10 + 1];
                }

            }
            d15 = d19;
            for(int k4 = 1; k4 <= k13; k4++)
            {
                int j10 = k4 - 1;
                int i7 = i14 - k4;
                d15 += ad4[j10] * ad9[i7 - 1];
            }

            ad7[k13 - 1] = -d15;
            double d4 = d2 * ad9[0];
            for(int l4 = 1; l4 <= k13; l4++)
            {
                int k10 = l4 - 1;
                ad3[k10] += d4 * ad7[k10];
            }

            double d3 = d2 / d10;
            d2 *= d20 + d3 * d17;
            double d18 = ad9[0] / d11;
            for(int i5 = 1; i5 <= k13; i5++)
            {
                int l10 = i5 - 1;
                ad9[l10] = ad7[l10] - d18 * ad3[l10];
            }

            double d8 = ad[l14] - ad10[0];
            d12 += Math.pow(d8, 2D) / d11;
            d13 += Math.log(d11);
            d10 = d11;
            d6 = d8;
            if(Math.abs(d11 - d20) < d21)
                break;
            j14++;
        } while(true);
        if(l12 < i)
        {
            int i13 = l12 + 1;
            for(int k14 = i13; k14 <= i; k14++)
            {
                int i15 = k14 - 1;
                if(l13 > 0)
                {
                    for(int j5 = 1; j5 <= l13; j5++)
                    {
                        int i11 = j5 - 1;
                        ad8[i11] = ad10[i11 + 1];
                    }

                }
                double d16 = d19;
                for(int k5 = 1; k5 <= k13; k5++)
                {
                    int j11 = k5 - 1;
                    int j7 = i14 - k5;
                    d16 += ad4[j11] * ad10[j7 - 1];
                }

                ad8[k13 - 1] = -d16;
                for(int l5 = 1; l5 <= k13; l5++)
                {
                    int k11 = l5 - 1;
                    ad10[k11] = ad8[k11] + d6 * ad3[k11];
                }

                double d9 = ad[i15] - ad10[0];
                d12 += Math.pow(d9, 2D);
                d6 = d9;
            }

        }
        double d5 = i;
        double d23 = d12 / d5;
        double d22 = d13 + d5 * Math.log(d23);
        if(j > 0)
        {
            for(int i6 = 1; i6 <= j; i6++)
            {
                int l11 = i6 - 1;
                ad1[l11] = ad4[l11];
            }

        }
        if(k > 0)
        {
            for(int j6 = 1; j6 <= k; j6++)
            {
                int i12 = j6 - 1;
                int k12 = j + j6;
                ad1[k12 - 1] = ad2[i12];
            }

        }
        return_m5x1.f = d22;
        return_m5x1.sd = d23;
        return return_m5x1;
    }

    private return_m7x l_m7x_arma(double d, double d1, double ad[], double ad1[], double ad2[], 
            int i, double ad3[], int j, int k, int l, int i1, int j1, 
            int k1, double ad4[], double ad5[])
        throws NonStationaryException, NonInvertibleException
    {
        double ad6[] = null;
        double ad7[] = null;
        double ad8[] = null;
        double ad9[] = null;
        double d17 = 0.0D;
        double d18 = 1.0D;
        double d19 = 4D;
        double d20 = 10D;
        double d21 = 0.5D;
        double d22 = 2D;
        return_m4x return_m4x1 = new return_m4x();
        return_m7x return_m7x1 = new return_m7x();
        double d23 = l_eps3;
        double d24 = l_eps4;
        return_m7x1.phai = d;
        return_m7x1.sigma2 = d1;
        return_m7x1.iswro = l;
        return_m7x1.iterations = 0;
        return_m7x1.g = new double[ad.length];
        for(int l1 = 0; l1 < ad.length; l1++)
            return_m7x1.g[l1] = ad[l1];

        return_m7x1.x = new double[ad3.length];
        for(int i2 = 0; i2 < ad3.length; i2++)
            return_m7x1.x[i2] = ad3[i2];

        return_m7x1.hs = new double[ad5.length];
        for(int j2 = 0; j2 < ad5.length; j2++)
            return_m7x1.hs[j2] = ad5[j2];

        ad6 = new double[j + k];
        ad7 = new double[j + k];
        ad8 = new double[j + k];
        ad9 = new double[j + k];
        for(int k2 = 0; k2 < j + k; k2++)
            ad8[k2] = d17;

        for(int l2 = 0; l2 < j + k; l2++)
            ad6[l2] = d17;

        int j8 = 0;
        boolean flag = true;
        int l7 = j + k;
        int i8 = l7 + l7;
        int l8 = 1;
        int i9 = 0;
        int j10 = 0;
label0:
        do
        {
            int k7;
            double d14;
label1:
            {
label2:
                do
                {
                    byte byte0;
                    double d7;
                    double d12;
                    double d13;
label3:
                    {
label4:
                        do
                        {
                            j10++;
                            i9++;
                            int k8 = 0;
                            do
                            {
                                int i7 = 0;
                                for(int i3 = 0; i3 < l7; i3++)
                                    ad9[i3] = ad3[i3] - ad1[i3];

                                i7 = 0;
                                if(j > 0)
                                {
                                    for(int j3 = 0; j3 < j; j3++)
                                        ad8[j3] = ad9[j3];

                                    i7 = l_m8x_arma(ad8, j, ad4, ad5);
                                    if(i7 > 0)
                                        break label4;
                                }
                                if(k > 0)
                                {
                                    for(int k3 = 0; k3 < k; k3++)
                                        ad8[k3] = ad9[k3 + j];

                                    i7 = l_m8x_arma(ad8, k, ad4, ad5);
                                    if(i7 > 0)
                                        break label4;
                                }
                                if(i7 == 0)
                                    break;
                                if(k8 > i1)
                                {
                                    Object aobj[] = {
                                        "maxit", new Integer(i1)
                                    };
                                    System.out.println(this+ "ARMAMaxLikelihood.IterationLimitExceeded"+aobj);
                                    break label4;
                                }
                                k8++;
                                double d6 = d17;
                                for(int l3 = 0; l3 < l7; l3++)
                                    if(ad5[l3 * (k + j) + l3] > d6)
                                        d6 = ad5[l3 * (k + j) + l3];

                                double d16 = d6 / d19;
                                for(int i4 = 0; i4 < l7; i4++)
                                {
                                    for(int j9 = 0; j9 < l7; j9++)
                                        ad5[i4 * (k + j) + j9] /= d20;

                                    ad5[i4 * (k + j) + i4] += d16;
                                }

                                int j4 = 0;
                                while(j4 < l7) 
                                {
                                    double d11 = d17;
                                    for(int k9 = 0; k9 < l7; k9++)
                                        d11 += ad5[j4 * (k + j) + k9] * ad[k9];

                                    ad1[j4] = d11;
                                    j4++;
                                }
                            } while(true);
                            d13 = 0.0D;
                            d12 = 0.0D;
                            return_m4x return_m4x2 = l_m4x_arma(d13, d12, ad6, ad2, i, ad9, j, k, j1, ad4, ad5);
                            d13 = return_m4x2.f0;
                            d12 = return_m4x2.sd;
                            for(int k4 = 0; k4 < ad6.length; k4++)
                                ad6[k4] = return_m4x2.g[k4];

                            for(int l4 = 0; l4 < l7; l4++)
                            {
                                double d15 = d17;
                                for(int l9 = 0; l9 < l7; l9++)
                                    d15 += ad5[l4 * (k + j) + l9] * ad6[l9];

                                ad7[l4] = d15;
                            }

                            d14 = 0.0D;
                            for(int i5 = 0; i5 < l7; i5++)
                                d14 += ad6[i5] * ad7[i5];

                            if(Math.abs(d14) < d24)
                            {
                                byte0 = -1;
                                d7 = 0.0D;
                                if(d < d13)
                                    break;
                                break label3;
                            }
                            double d5 = 0.0D;
                            for(int j5 = 0; j5 < l7; j5++)
                                d5 += ad[j5] * ad7[j5];

                            double d3 = -d5 / d14;
                            double d4 = d3 + d18;
                            d4 = Math.abs(d4) + d23;
                            d7 = Math.abs(d3) / d4;
                            if(d7 <= d21)
                            {
                                d7 = d21;
                                byte0 = 1;
                            } else
                            if(d7 >= d22)
                            {
                                d7 = d22;
                                byte0 = -1;
                            } else
                            {
                                byte0 = 0;
                            }
                            double d9 = (d7 - d18) / d14;
                            for(int k5 = 0; k5 < l7; k5++)
                            {
                                double d10 = d9 * ad7[k5];
                                for(int i10 = 0; i10 < l7; i10++)
                                    ad5[k5 * (k + j) + i10] += d10 * ad7[i10];

                            }

                            if(d >= d13)
                                break label3;
                            double d8 = d7 - d18;
                            if(Math.abs(d8) < d23)
                                break;
                            double d2 = d3 * d8;
                            for(int l5 = 0; l5 < l7; l5++)
                                ad1[l5] += -d2 * ad7[l5];

                            int j7 = 0;
                            if(d14 <= d24)
                                break;
                            j8 = (j8 + (1 - j7)) * (1 - j7);
                        } while(j8 <= i8);
                        break label2;
                    }
                    for(int i6 = 0; i6 < l7; i6++)
                    {
                        ad3[i6] = ad9[i6];
                        ad[i6] = ad6[i6];
                        ad1[i6] = d7 * ad7[i6];
                    }

                    d = d13;
                    d1 = d12;
                    if(++l8 > i1)
                        break;
                    k7 = 1;
                    if(byte0 != 0)
                        break label1;
                    if(d14 < d24)
                        break;
                    j8 = (j8 + (1 - k7)) * (1 - k7);
                } while(j8 <= i8);
                break label0;
            }
            if(d14 < d24)
                break;
            j8 = (j8 + (1 - k7)) * (1 - k7);
        } while(j8 <= i8);
        l++;
        return_m7x1.phai = d;
        return_m7x1.sigma2 = d1;
        return_m7x1.iswro = l;
        return_m7x1.iterations = l8;
        for(int j6 = 0; j6 < ad.length; j6++)
            return_m7x1.g[j6] = ad[j6];

        for(int k6 = 0; k6 < ad3.length; k6++)
            return_m7x1.x[k6] = ad3[k6];

        for(int l6 = 0; l6 < ad5.length; l6++)
            return_m7x1.hs[l6] = ad5[l6];

        return return_m7x1;
    }

    private return_m4x l_m4x_arma(double d, double d1, double ad[], double ad1[], int i, 
            double ad2[], int j, int k, int l, double ad3[], double ad4[])
        throws NonStationaryException
    {
        double ad5[] = null;
        double ad6[] = null;
        double d5 = 0.0001D;
        double d6 = 0.69999999999999996D;
        d5 = l_epsa;
        return_m4x return_m4x1 = new return_m4x();
        return_m5x return_m5x1 = new return_m5x();
        return_m4x1.f0 = d;
        return_m4x1.sd = d1;
        return_m4x1.g = new double[ad.length];
        for(int i1 = 0; i1 < ad.length; i1++)
            return_m4x1.g[i1] = ad[i1];

        ad5 = new double[j + k];
        ad6 = new double[j + k];
        return_m5x1 = l_m5x_arma(d, d1, ad1, i, ad2, j, k);
        d = return_m5x1.f;
        d1 = return_m5x1.sd;
        int k2 = k + j;
        for(int i3 = 0; i3 < k2; i3++)
            ad5[i3] = ad2[i3];

        int j1 = 0;
label0:
        do
        {
            if(j1 >= k2)
                break;
            double d2 = d5;
            int l2 = 1;
            do
            {
                int l1 = 0;
                ad5[j1] = ad2[j1] + d2;
                if(j1 < j)
                {
                    for(int i2 = 0; i2 < j; i2++)
                        ad6[i2] = ad5[i2];

                    l1 = l_m8x_arma(ad6, j, ad3, ad4);
                    if(l1 > 0)
                        break label0;
                } else
                if(k > 0)
                {
                    for(int j2 = 0; j2 < k; j2++)
                        ad6[j2] = ad5[j2 + j];

                    l1 = l_m8x_arma(ad6, k, ad3, ad4);
                    if(l1 > 0)
                        break label0;
                }
                if(l1 == 0)
                {
                    double d3 = 0.0D;
                    double d4 = 0.0D;
                    return_m5x return_m5x2 = l_m5x_arma(d3, d4, ad1, i, ad5, j, k);
                    d3 = return_m5x2.f;
                    d4 = return_m5x2.sd;
                    ad[j1] = (d3 - d) / d2;
                    ad5[j1] = ad2[j1];
                    break;
                }
                if(l2 >= 10)
                    break label0;
                d2 *= d6;
                d2 = -d2;
                l2++;
            } while(true);
            j1++;
        } while(true);
        return_m4x1.f0 = d;
        return_m4x1.sd = d1;
        for(int k1 = 0; k1 < ad.length; k1++)
            return_m4x1.g[k1] = ad[k1];

        return return_m4x1;
    }

    private return_m3x l_m3x_arma(double d, double d1, double ad[], int i, double ad1[], 
            int j, int k, int l, int i1, double ad2[], double ad3[])
        throws NonStationaryException, NonInvertibleException
    {
        int k8;
        double ad6[];
        return_m3x return_m3x1;
        k8 = 0;
        double ad4[] = null;
        double ad5[] = null;
        ad6 = null;
        double d7 = 10D * l_eps4;
        return_m3x1 = new return_m3x();
        return_m4x return_m4x1 = new return_m4x();
        return_m7x return_m7x1 = new return_m7x();
        return_m3x1.tl = d;
        return_m3x1.sigma2 = d1;
        return_m3x1.p0 = new double[ad1.length];
        for(int k1 = 0; k1 < ad1.length; k1++)
            return_m3x1.p0[k1] = ad1[k1];

        int i7 = k + j;
        return_m3x1.g = new double[i7];
        for(int l1 = 0; l1 < i7; l1++)
            return_m3x1.g[l1] = 0.0D;

        ad4 = new double[i7];
        ad5 = new double[i7];
        ad6 = new double[i7];
        int j1 = 0;
        int j7 = 0;
        if(l == 2)
            j7 = 1;
        for(int i2 = 0; i2 < i7; i2++)
        {
            ad5[i2] = 0.0D;
            for(int l7 = 0; l7 < i7; l7++)
                ad3[i2 * i7 + l7] = 0.0D;

        }

        boolean flag = false;
        if(j > 0)
        {
            for(int j2 = 0; j2 < j; j2++)
                ad6[j2] = ad1[j2];

            int i6 = l_m8x_arma(ad6, j, ad2, ad3);
            if(i6 > 0)
                return return_m3x1;
            for(int k2 = 0; k2 < j; k2++)
                ad1[k2] = ad6[k2];

        }
        if(k > 0)
        {
            for(int l2 = 0; l2 < k; l2++)
                ad6[l2] = ad1[l2 + j];

            int j6 = l_m8x_arma(ad6, k, ad2, ad3);
            if(j6 > 0)
                return return_m3x1;
            for(int i3 = 0; i3 < k; i3++)
                ad1[i3 + j] = ad6[i3];

        }
        int k7 = 0;
        k8 = 0;
        return_m4x1 = l_m4x_arma(d, d1, ad5, ad, i, ad1, j, k, 0, ad2, ad3);
        d = return_m4x1.f0;
        d1 = return_m4x1.sd;
        for(int j3 = 0; j3 < i7; j3++)
            ad5[j3] = return_m4x1.g[j3];

       boolean external =true;
		do
        {
	label0:
            {
                double d6 = 0.0D;
                for(int k3 = 0; k3 < i7; k3++)
                    d6 += ad5[k3] * ad5[k3];

                d6 = Math.sqrt(d6) / (double)i7;
                if(d6 < d7)
                    break label0;
                double d3 = 0.0D;
                for(int l3 = 0; l3 < i7; l3++)
                {
                    double d4 = Math.abs(ad5[l3]);
                    if(d4 > d3)
                        d3 = d4;
                }

                double d2 = 0.0001D / d3;
                for(int i4 = 0; i4 < i7; i4++)
                {
                    for(int i8 = 0; i8 < i7; i8++)
                        ad3[i4 * (k + j) + i8] /= 10D;

                    ad3[i4 * (k + j) + i4] += d2;
                }

                for(int j4 = 0; j4 < i7; j4++)
                {
                    double d5 = 0.0D;
                    for(int j8 = 0; j8 < i7; j8++)
                        d5 += ad3[j4 * (k + j) + j8] * ad5[j8];

                    ad4[j4] = d5;
                }

                return_m7x return_m7x2 = l_m7x_arma(d, d1, ad5, ad4, ad, i, ad1, j, k, k7, i1, j7, j1, ad2, ad3);
                if(return_m7x2.iterations > k8)
                    k8 = return_m7x2.iterations;
                d = return_m7x2.phai;
                d1 = return_m7x2.sigma2;
                for(int k4 = 0; k4 < j + k; k4++)
                {
                    ad5[k4] = return_m7x2.g[k4];
                    return_m3x1.g[k4] = ad5[k4];
                }

                k7 = return_m7x2.iswro;
                for(int l4 = 0; l4 < j + k; l4++)
                {
                    ad1[l4] = return_m7x2.x[l4];
                    return_m3x1.p0[l4] = ad1[l4];
                }

                for(int i5 = 0; i5 < ad3.length; i5++)
                    ad3[i5] = return_m7x2.hs[i5];

                if(k7 >= i7){
						external=false;
						break label0;

					}
                int j5 = 0;
                do
                {
                    if(j5 >= i7){
						external=false;
						break label0;

					}
                        
                    if(Math.abs(ad6[j5] - ad1[j5]) >= 5.0000000000000002E-005D)
                        break;
                    j5++;
                } while(true);
            }
        } while(external);


        if(k8 >= i1)
        {
            Object aobj[] = {
                "maxit", new Integer(i1)
            };
            System.out.println(this+ "ARMAMaxLikelihood.IterationLimitExceeded"+aobj);
        }

        boolean flag1 = false;
        if(j > 0)
        {
            for(int k5 = 0; k5 < j; k5++)
                ad6[k5] = ad1[k5];

            int k6 = l_m8x_arma(ad6, j, ad2, ad3);
            if(k6 > 0)
                return return_m3x1;
        }
        if(k > 0)
        {
            for(int l5 = 0; l5 < k; l5++)
                ad6[l5] = ad1[l5 + j];

            int l6 = l_m8x_arma(ad6, k, ad2, ad3);
            if(l6 > 0)
                return return_m3x1;
        }
        return_m3x1.tl = d;
        return_m3x1.sigma2 = d1;
        return return_m3x1;
    }

    private double[] l_m6x_arma(double ad[], double ad1[], double ad2[], int i, int j, int k)
        throws NonStationaryException
    {
        Object obj = null;
        double ad4[] = null;
        double ad5[] = null;
        double ad6[] = null;
        Object obj1 = null;
        double ad7[] = new double[ad.length];
        if(i > 0)
            ad4 = new double[i * i];
        ad5 = new double[i + j + 2];
        ad6 = new double[i + j + 1];
        if(i > 0)
        {
            double ad3[] = new double[i + 1];
            Complex acomplex[] = new Complex[i];
            ad3[0] = 1.0D;
            for(int l = 0; l < i; l++)
                ad3[l + 1] = ad1[l];

            ZeroPolynomial zeropolynomial = new ZeroPolynomial();
            try
            {
                acomplex = zeropolynomial.computeRoots(ad3);
            }
            catch(Exception exception)
            {
                throw new NonStationaryException("ARMAMaxLikelihood.NonStationary", null);
            }
            for(int i1 = 0; i1 < i; i1++)
                if(Complex.abs(Complex.multiply(acomplex[i1], acomplex[i1])) <= 1.0D)
                    throw new NonStationaryException("ARMAMaxLikelihood.NonStationary", null);

        }
        if(i > 0)
        {
            for(int j1 = 0; j1 < i * i; j1++)
                ad4[j1] = 0.0D;

        }
        for(int k1 = 0; k1 < i + j + 2; k1++)
            ad5[k1] = 0.0D;

        ad6[0] = 1.0D;
        int i9 = j + 1;
        int j9 = i + 1;
        int l9 = k - 1;
        if(l9 > 0)
        {
            for(int l1 = 2; l1 <= k; l1++)
            {
                int k3 = l1 - 1;
                int l5 = l1 - 1;
                int k7 = Math.min(i, l5);
                double d2 = 0.0D;
                if(k7 > 0)
                {
                    for(int j11 = 1; j11 <= k7; j11++)
                    {
                        int k12 = j11 - 1;
                        int i7 = l1 - j11;
                        d2 += -ad1[k12] * ad6[i7 - 1];
                    }

                }
                if(l1 <= i9)
                    d2 += ad2[l5 - 1];
                ad6[k3] = d2;
            }

        }
        int i10 = k + 1;
        ad5[i10 - 1] = 0.0D;
        if(l9 > 0)
        {
            ad5[k - 1] = ad2[l9 - 1];
            for(int i2 = 1; i2 <= l9; i2++)
            {
                int l3 = i2 - 1;
                int i6 = i2 - 1;
                int l8 = j - i6;
                double d3 = 0.0D;
                if(l8 > 0)
                {
                    for(int k11 = 1; k11 <= l8; k11++)
                    {
                        int l12 = k11 - 1;
                        int l13 = i6 + k11;
                        d3 += ad2[l13 - 1] * ad6[l12 + 1];
                    }

                }
                if(i2 == 1)
                    d3++;
                if(i2 > 1)
                    d3 += ad2[i6 - 1];
                ad5[l3] = d3;
            }

        } else
        {
            ad5[0] = 1.0D;
        }
        if(i > 0)
        {
            int k9 = i + 1;
            for(int i16 = 1; i16 <= i; i16++)
            {
                int j16 = i16 - 1;
                ad4[j16] = ad1[j16];
            }

            int j10 = i;
            int j2 = i;
            do
            {
                int l7 = j2 + 1;
                int i8 = l7 + 1;
                int l4 = i8 / 2;
                int i5 = l4 + 1;
                double d = ad4[j10 - 1];
                double d1 = 1.0D / (1.0D - d * d);
                for(int i14 = 1; i14 <= l4; i14++)
                {
                    int i15 = i14 - 1;
                    int j8 = i8 - i14;
                    ad5[i15] = (ad5[i15] - d * ad5[j8 - 1]) * d1;
                }

                if(j2 > 2)
                {
                    for(int j14 = i5; j14 <= j2; j14++)
                    {
                        int j15 = j14 - 1;
                        int k8 = i8 - j14;
                        ad5[j15] += -d * ad5[k8 - 1];
                    }

                }
                int j6 = j2 - 1;
                if(j6 == 0)
                    break;
                for(int k14 = 1; k14 <= j6; k14++)
                {
                    int k15 = k14 - 1;
                    int i11 = j10 + k14;
                    int l6 = (j10 - j2) + k14;
                    int k10 = j10 - k14;
                    ad4[i11 - 1] = (ad4[l6 - 1] - d * ad4[k10 - 1]) * d1;
                }

                j2 = j6;
                j10 += j2;
            } while(true);
            if(i > 1)
            {
                int l10 = j10 + 1;
                for(int k2 = 2; k2 <= i; k2++)
                {
                    int i4 = k2 - 1;
                    double d4 = ad5[i4];
                    int k6 = k2 - 1;
                    for(int l11 = 1; l11 <= k6; l11++)
                    {
                        int i13 = l11 - 1;
                        l10--;
                        d4 += -ad4[l10 - 1] * ad5[i13];
                    }

                    ad5[i4] = d4;
                }

            }
            for(int l2 = k9; l2 <= i10; l2++)
            {
                int j4 = l2 - 1;
                double d5 = ad5[j4];
                for(int i12 = 1; i12 <= i; i12++)
                {
                    int j13 = i12 - 1;
                    int j7 = l2 - i12;
                    d5 += -ad4[j13] * ad5[j7 - 1];
                }

                ad5[j4] = d5;
            }

        }
        for(int i3 = 1; i3 <= k; i3++)
        {
            int k4 = i3 - 1;
            for(int j12 = 1; j12 <= i3; j12++)
            {
                int k13 = j12 - 1;
                double d6 = 0.0D;
                if(j12 != 1)
                {
                    for(int l14 = 1; l14 <= j12; l14++)
                    {
                        int l15 = l14 - 1;
                        int k5 = (l14 + i3) - j12;
                        d6 += ad6[k5 - 1] * ad6[l15];
                    }

                }
                int j5 = (i3 - j12) + 1;
                ad[k13 * k + k4] = ad5[j5 - 1] - d6;
                ad[k4 * k + k13] = ad[k13 * k + k4];
            }

        }

        for(int j3 = 0; j3 < ad.length; j3++)
            ad7[j3] = ad[j3];

        return ad7;
    }

  

}