
package utility.forecasting;


import utility.forecasting.ARMAMaxLikelihood.InitialMAException;
import utility.forecasting.Cholesky.NotSPDException;
import utility.forecasting.SVD.DidNotConvergeException;
import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;

//            Sort, Cdf, Summary, LinearRegression, 
//            Difference, ARMA, ARMAMaxLikelihood

public class ARMAOutlierIdentification
    implements Serializable, Cloneable
{

    private static final long serialVersionUID = 0x36adf590d658e7c6L;
    public static final int INNOVATIONAL = 0;
    public static final int ADDITIVE = 1;
    public static final int LEVEL_SHIFT = 2;
    public static final int TEMPORARY_CHANGE = 3;
    public static final int UNABLE_TO_IDENTIFY = 4;
    private boolean isComputed;
    private boolean isForecasted;
    private double l_z[];
    private int l_p;
    private int l_q;
    private int l_s;
    private int l_d;
    private int n_obs;
    private double res_sigma;
    private double res_series[];
    private double out_free_series[];
    private int outlier_stat[][];
    private int num_outliers[] = {
        0
    };
    private double l_delta;
    private double l_critical;
    private double l_epsilon;
    private double l_relative_error;
    private double l_confidence;
    private double l_aic[];
    private double l_aicc[];
    private double l_bic[];
    private double l_parameters[];
    private double l_outfree_forecast[];
    private double l_outlier_forecast[];
    private double l_psi_weights[];
    private double l_dev[];
    private double sigma_arr[];
    private double tau_arr[];
    private double omega_arr[];
    private int l_model[];
    private static final int EMPTY_INTEGER_ARRAY[][] = new int[0][0];
    private static final double EMPTY_DOUBLE_VECTOR[] = new double[0];
    private static final int EMPTY_INTEGER_VECTOR[] = new int[0];

    public ARMAOutlierIdentification(double ad[])
    {
        isComputed = false;
        isForecasted = false;
        l_s = 1;
        l_d = 0;
        l_delta = 0.69999999999999996D;
        l_critical = 3D;
        l_epsilon = 0.001D;
        l_relative_error = 1E-010D;
        l_confidence = 95D;
        l_aic = new double[1];
        l_aicc = new double[1];
        l_bic = new double[1];
        sigma_arr = new double[5];
        l_model = new int[4];
        if(ad.length < 2)
        {
            Object aobj[] = {
                "z.length", new Integer(ad.length), "1"
            };
            System.out.println( "NotLargeEnough"+ aobj);
        }
        l_z = new double[ad.length];
        System.arraycopy(ad, 0, l_z, 0, ad.length);
        n_obs = ad.length;
        l_aic[0] = (1.0D / 0.0D);
        l_aicc[0] = (1.0D / 0.0D);
        l_bic[0] = (1.0D / 0.0D);
    }

    public final void compute(int ai[])
        throws ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.InitialMAException, ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, SingularMatrixException, Cholesky.NotSPDException, NotSPDException, DidNotConvergeException
    {
        int k1 = 0;
        int ai3[] = {
            0
        };
        int ai4[] = {
            0
        };
        double ad4[] = null;
        double ad11[] = null;
        double ad12[] = null;
        if(ai.length < 4)
        {
            Object aobj[] = {
                "model.length", new Integer(ai.length), "3"
            };
            System.out.println("NotLargeEnough"+ aobj);
        }
        if(ai[0] < 0)
        {
            Object aobj1[] = {
                "p=model[0]", new Integer(ai[0])
            };
            System.out.println( "Negative"+ aobj1);
        }
        l_p = ai[0];
        if(ai[1] < 0)
        {
            Object aobj2[] = {
                "q=model[1]", new Integer(ai[1])
            };
            System.out.println( "Negative"+ aobj2);
        }
        l_q = ai[1];
        if(ai[2] <= 0)
        {
            Object aobj3[] = {
                "s=model[2]", new Integer(ai[2])
            };
            System.out.println("NotPositive"+ aobj3);
        }
        l_s = ai[2];
        if(ai[3] < 0)
        {
            Object aobj4[] = {
                "d=model[3]", new Integer(ai[3])
            };
            System.out.println( "Negative"+ aobj4);
        }
        l_d = ai[3];
        if(l_z.length < l_p + l_q + l_s * l_d + 2)
        {
            Object aobj5[] = {
                "z.length", new Integer(l_z.length), "p+q+s*d+2"
            };
            System.out.println("NotLargeEnough"+ aobj5);
        }
        for(int i = 0; i < 4; i++)
            l_model[i] = ai[i];

        double ad9[] = new double[n_obs];
        double ad10[] = new double[n_obs];
        double ad[] = new double[n_obs];
        double ad1[] = new double[n_obs];
        int ai1[] = new int[n_obs];
        int ai6[] = new int[n_obs];
        int ai2[] = new int[n_obs];
        double ad3[] = new double[n_obs];
        double ad2[] = new double[n_obs];
        res_series = new double[n_obs];
        out_free_series = new double[n_obs];
        if(l_d > 0)
            ad4 = new double[n_obs];
        double ad6[];
        if(l_p > 0)
        {
            ad11 = new double[l_p];
            ad6 = new double[l_p];
        } else
        {
            ad6 = new double[1];
        }
        double ad7[];
        if(l_q > 0)
        {
            ad12 = new double[l_q];
            ad7 = new double[l_q];
        } else
        {
            ad7 = new double[1];
        }
        l_parameters = new double[l_p + l_q + 1];
        double ad8[] = new double[l_p + l_s * l_d + 1 + l_q];
        BLAS.copy(n_obs, l_z, ad);
        BLAS.set(n_obs, 0.0D, res_series, 0);
        ai4[0] = 0;
        try {
            l_ts_outlier_identification_driver(n_obs, ai, l_z, l_delta, l_critical, l_epsilon, l_aic, l_aicc, l_bic, out_free_series, res_series, ad9, ad10, ad, ad1, ad4, ad6, ad7, ad11, ad12, l_parameters, ad8, sigma_arr, ai1, ad3, ad2, ai2, ai6, num_outliers, ai4, ai3, l_relative_error);
        } catch (ZeroPolynomial.DidNotConvergeException ex) {
            Logger.getLogger(ARMAOutlierIdentification.class.getName()).log(Level.SEVERE, null, ex);
        }
        k1 = num_outliers[0];
        if(k1 > 0)
        {
            int ai7[][] = new int[2][k1];
            outlier_stat = new int[k1][2];
            omega_arr = new double[k1];
            tau_arr = new double[k1];
            int ai5[] = new int[k1];
            double ad5[] = new double[k1];
            int l1 = ai4[0];
            for(int j = 0; j < k1; j++)
            {
                ad5[j] = ai1[l1];
                l1 = ai2[l1];
            }

            Sort.ascending(ad5, ai5);
            l1 = ai4[0];
            for(int k = 0; k < k1; k++)
            {
                ai7[0][k] = ai1[l1];
                ai7[1][k] = ai6[l1];
                l1 = ai2[l1];
            }

            l_permute_vector(k1, ai7[0], ai5, 1, ai7[0]);
            l_permute_vector(k1, ai7[1], ai5, 1, ai7[1]);
            for(int l = 0; l < k1; l++)
            {
                outlier_stat[l][0] = ai7[0][l];
                outlier_stat[l][1] = ai7[1][l];
            }

            l1 = ai4[0];
            for(int i1 = 0; i1 < k1; i1++)
            {
                omega_arr[i1] = ad3[l1];
                l1 = ai2[l1];
            }

            l_permute_vector(k1, omega_arr, ai5, 1, omega_arr);
            l1 = ai4[0];
            for(int j1 = 0; j1 < k1; j1++)
            {
                tau_arr[j1] = ad2[l1];
                l1 = ai2[l1];
            }

            l_permute_vector(k1, tau_arr, ai5, 1, tau_arr);
        }
        isComputed = true;
    }

    public double[] getMA()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getMA"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(l_q > 0)
        {
            double ad[] = new double[l_q];
            for(int i = 0; i < l_q; i++)
                ad[i] = l_parameters[1 + l_p + i];

            return ad;
        } else
        {
            return EMPTY_DOUBLE_VECTOR;
        }
    }

    public double[] getAR()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getAR"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(l_p > 0)
        {
            double ad[] = new double[l_p];
            for(int i = 0; i < l_p; i++)
                ad[i] = l_parameters[1 + i];

            return ad;
        } else
        {
            return EMPTY_DOUBLE_VECTOR;
        }
    }

    public double getConstant()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getConstant"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_parameters[0];
    }

    public double getResidualStandardError()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getResidualStandardError"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return sigma_arr[0];
    }

    public double[] getResidual()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getResidual"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        int i = res_series.length;
        double ad[] = new double[i];
        System.arraycopy(res_series, 0, ad, 0, i);
        return ad;
    }

    public int getNumberOfOutliers()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getNumberOfOutliers"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return num_outliers[0];
    }

    public int[][] getOutlierStatistics()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getOutlierStatistics"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(num_outliers[0] > 0)
        {
            int ai[][] = new int[num_outliers[0]][2];
            for(int i = 0; i < num_outliers[0]; i++)
            {
                ai[i][0] = outlier_stat[i][0];
                ai[i][1] = outlier_stat[i][1];
            }

            return ai;
        } else
        {
            return EMPTY_INTEGER_ARRAY;
        }
    }

    public double[] getTauStatistics()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getTauStatistics"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(num_outliers[0] > 0)
        {
            double ad[] = new double[num_outliers[0]];
            System.arraycopy(tau_arr, 0, ad, 0, num_outliers[0]);
            return ad;
        } else
        {
            return EMPTY_DOUBLE_VECTOR;
        }
    }

    public double[] getOmegaWeights()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getOmegaWeights"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(num_outliers[0] > 0)
        {
            double ad[] = new double[num_outliers[0]];
            System.arraycopy(omega_arr, 0, ad, 0, num_outliers[0]);
            return ad;
        } else
        {
            return EMPTY_DOUBLE_VECTOR;
        }
    }

    public double getAIC()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getAIC"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_aic[0];
    }

    public double getAICC()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getAICC"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_aicc[0];
    }

    public double getBIC()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getBIC"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_bic[0];
    }

    public double[] getOutlierFreeSeries()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getOutlierFreeSeries"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        double ad[] = new double[n_obs];
        System.arraycopy(out_free_series, 0, ad, 0, n_obs);
        return ad;
    }

    public final void computeForecasts(int i)
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "computeForecasts"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(i <= 0)
        {
            Object aobj1[] = {
                "nForecast", new Integer(i), new Integer(0)
            };
            System.out.println( "NotLargeEnough"+ aobj1);
        }
        double ad[] = new double[l_p + l_s * l_d + 1];
        double ad1[] = new double[n_obs + i + 1];
        double ad2[] = new double[i];
        double ad3[] = new double[i];
        double ad4[][] = new double[i][3];
        double ad5[][] = new double[i][3];
        l_outfree_forecast = new double[i];
        l_outlier_forecast = new double[i];
        l_psi_weights = new double[i];
        l_dev = new double[i];
        l_ts_outlier_forecast_driver(n_obs, out_free_series, l_model, outlier_stat, omega_arr, res_series, ad1, ad, ad2, ad3, i, num_outliers[0], l_delta, ad5, ad4, l_confidence, l_parameters);
        for(int j = 0; j < i; j++)
        {
            l_outfree_forecast[j] = ad4[j][0];
            l_outlier_forecast[j] = ad5[j][0];
            l_dev[j] = ad4[j][1];
            l_psi_weights[j] = ad4[j][2];
        }

        isForecasted = true;
    }

    private void l_ts_outlier_forecast_driver(int i, double ad[], int ai[], int ai1[][], double ad1[], double ad2[], double ad3[], 
            double ad4[], double ad5[], double ad6[], int j, int k, double d, 
            double ad7[][], double ad8[][], double d1, double ad9[])
    {
        int l = ai[0];
        int i1 = ai[1];
        int j1 = ai[2];
        int k1 = ai[3];
        double d6 = 0.0D;
        for(int l1 = Math.max(l + j1 * k1, i1); l1 < i; l1++)
            d6 += ad2[l1] * ad2[l1];

        if(i - Math.max(l + j1 * k1, i1) > 0)
            d6 /= i - Math.max(l + j1 * k1, i1);
        double d2 = (100D - d1) * 0.01D;
        double d4 = Cdf.inverseNormal(1.0D - d2 / 2D);
        double d3 = 0.0D;
        l_compute_psi(ad9, ad4, ad3, ai, i + j);
        l_compute_forecasts(i, j, d3, ai, ad5, ad, ad2, ad9, ad4);
        BLAS.copy(j, ad5, ad6);
        for(int k3 = 0; k3 < k; k3++)
        {
            int l3 = ai1[k3][1];
            int i4 = ai1[k3][0];
            if(l3 == 0 || l3 == 4)
            {
                for(int i2 = 0; i2 < j; i2++)
                    ad6[i2] += ad1[k3] * ad3[(i - i4) + i2 + 1];

            }
            if(l3 == 2)
            {
                for(int j2 = 0; j2 < j; j2++)
                    ad6[j2] += ad1[k3];

            }
            if(l3 != 3)
                continue;
            for(int k2 = 0; k2 < j; k2++)
                ad6[k2] += ad1[k3] * Math.pow(d, (i - i4) + k2 + 1);

        }

        for(int l2 = 0; l2 < j; l2++)
        {
            ad7[l2][2] = ad3[l2 + 1];
            ad8[l2][2] = ad3[l2 + 1];
        }

        double d5 = 0.0D;
        ad7[0][1] = d4 * Math.sqrt(d6);
        ad8[0][1] = d4 * Math.sqrt(d6);
        for(int i3 = 2; i3 <= j; i3++)
        {
            d5 += Math.pow(ad7[i3 - 2][2], 2D);
            ad7[i3 - 1][1] = d4 * Math.sqrt((1.0D + d5) * d6);
            ad8[i3 - 1][1] = d4 * Math.sqrt((1.0D + d5) * d6);
        }

        for(int j3 = 0; j3 < j; j3++)
        {
            ad7[j3][0] = ad6[j3];
            ad8[j3][0] = ad5[j3];
        }

    }

    public void setConfidence(double d)
    {
        if(d <= 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "confidence", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println( "GreaterLess"+ aobj);
        }
        d *= 100D;
        l_confidence = d;
    }

    public double[] getDeviations()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "getDeviations"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeForecastsMethodRequired"+ aobj);
        }
        int i = l_dev.length;
        double ad[] = new double[i];
        System.arraycopy(l_dev, 0, ad, 0, i);
        return ad;
    }

    public double[] getForecast()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "getForecast"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeForecastsMethodRequired"+ aobj);
        }
        int i = l_outlier_forecast.length;
        double ad[] = new double[i];
        System.arraycopy(l_outlier_forecast, 0, ad, 0, i);
        return ad;
    }

    public double[] getOutlierFreeForecast()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "getOutlierFreeForecast"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeForecastsMethodRequired"+ aobj);
        }
        int i = l_outfree_forecast.length;
        double ad[] = new double[i];
        System.arraycopy(l_outfree_forecast, 0, ad, 0, i);
        return ad;
    }

    public double[] getPsiWeights()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "getPsiWeights"
            };
            System.out.println( "ARMAOutlierIdentification.ComputeForecastsMethodRequired"+ aobj);
        }
        int i = l_psi_weights.length;
        double ad[] = new double[i];
        System.arraycopy(l_psi_weights, 0, ad, 0, i);
        return ad;
    }

    public void setDelta(double d)
    {
        if(d <= 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "delta", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println( "GreaterLess"+ aobj);
        }
        l_delta = d;
    }

    public void setCriticalValue(double d)
    {
        if(d <= 0.0D)
        {
            Object aobj[] = {
                "critical", new Double(d)
            };
            System.out.println( "NotPositive"+ aobj);
        }
        l_critical = d;
    }

    public void setAccuracyTolerance(double d)
    {
        if(d <= 0.0D)
        {
            Object aobj[] = {
                "epsilon", new Double(d)
            };
            System.out.println( "NotPositive"+ aobj);
        }
        l_epsilon = d;
    }

    public void setRelativeError(double d)
    {
        if(d <= 0.0D)
        {
            Object aobj[] = {
                "relativeError", new Double(d)
            };
            System.out.println( "NotPositive"+ aobj);
        }
        l_relative_error = d;
    }

    private double l_binom(int i, int j)
    {
        boolean flag1 = false;
        double d8 = Math.log(1.7976931348623157E+308D) - 0.0001D;
        double d = (0.0D / 0.0D);
        int l = Math.min(j, i - j);
        d = 1.0D;
        if(l == 0 || i == 1)
            return d;
        double d5 = i;
        double d7 = d5 + 1.0D;
        double d3 = l;
        double d4 = d3 + 1.0D;
        boolean flag;
        if(l <= 20)
        {
            flag = false;
            if(d3 * Math.log(d5) > d8)
                flag = true;
        } else
        {
            flag = true;
        }
        if(!flag)
        {
            double d2 = 0.0D;
            for(int k = 1; k <= l; k++)
            {
                d2++;
                d = (d * (d7 - d2)) / d2;
            }

        } else
        {
            if(l < 9)
            {
                System.out.println( "ANOVAFactorial.BinomialOverflow");
                d = (0.0D / 0.0D);
                return d;
            }
            double d6 = d7 - d3;
            double d1 = Sfun.r9lgmc(d7) - Sfun.r9lgmc(d4) - Sfun.r9lgmc(d6);
            d = (((d4 * Math.log(d6 / d4) - d7 * dlnrel(-d3 / d7) - 0.5D * Math.log((d7 * d6) / d4)) + 1.0D) - 0.91893853320467278D) + d1;
            if(d > d8)
            {
                System.out.println( "ANOVAFactorial.BinomialOverflow");
                d = (0.0D / 0.0D);
                return d;
            }
            d = Math.exp(d);
        }
        if(d < 8106479329266893D)
            d = (int)d;
        return d;
    }

    private double csevl(double d, double ad[])
    {
        double d2 = 0.0D;
        double d1 = 0.0D;
        double d3 = 0.0D;
        double d4 = 2D * d;
        for(int i = ad.length - 1; i >= 0; i--)
        {
            d3 = d2;
            d2 = d1;
            d1 = (d4 * d2 - d3) + ad[i];
        }

        return 0.5D * (d1 - d3);
    }

    private double dlnrel(double d)
    {
        double ad[] = {
            1.037869356274377D, -0.13364301504908918D, 0.019408249135520562D, -0.0030107551127535777D, 0.00048694614797154852D, -8.1054881893175362E-005D, 1.3778847799559525E-005D, -2.380221089435897E-006D, 4.1640416213865184E-007D, -7.3595828378075992E-008D, 
            1.3117611876241675E-008D, -2.3546709317742423E-009D, 4.2522773276035E-010D, -7.7190894134840799E-011D, 1.407574648135907E-011D, -2.5769072058024682E-012D, 4.7342406666294419E-013D, -8.7249012674742641E-014D, 1.6124614902740551E-014D, -2.9875652015665774E-015D, 
            5.5480701209082887E-016D, -1.0324619158271569E-016D, 1.9250239203049852E-017D, -3.5955073465265147E-018D, 6.726454253787686E-019D, -1.260262416873522E-019D
        };
        double d1;
        if(d <= -1D)
            d1 = (0.0D / 0.0D);
        else
        if(Math.abs(d) <= 0.375D)
            d1 = d * (1.0D - d * csevl(d / 0.375D, ad));
        else
            d1 = Math.log(1.0D + d);
        return d1;
    }

    private void l_compute_psi(double ad[], double ad1[], double ad2[], int ai[], int i)
    {
        int j = ai[0];
        int k = ai[1];
        int l = ai[2];
        int i1 = ai[3];
        ad1[0] = 1.0D;
        for(int j1 = 1; j1 <= j + l * i1; j1++)
        {
            if(j1 <= j)
                ad1[j1] = ad[j1];
            else
                ad1[j1] = 0.0D;
            double d = 1.0D;
            for(int l1 = 1; l1 <= i1; l1++)
            {
                if(j1 - l * l1 == 0)
                    ad1[j1] += l_binom(i1, l1) * d;
                if(j1 - l * l1 > 0 && j1 - l * l1 <= j)
                    ad1[j1] += l_binom(i1, l1) * d * -ad[j1 - l * l1];
                d *= -1D;
            }

        }

        ad2[0] = 1.0D;
        for(int k1 = 1; k1 <= i; k1++)
        {
            if(k1 <= k)
                ad2[k1] = -ad[j + k1];
            else
                ad2[k1] = 0.0D;
            for(int i2 = 1; i2 <= k1; i2++)
                if(i2 <= j + l * i1)
                    ad2[k1] += ad2[k1 - i2] * ad1[i2];

        }

    }

    private void l_compute_forecasts(int i, int j, double d, int ai[], double ad[], double ad1[], 
            double ad2[], double ad3[], double ad4[])
    {
        int k = ai[0];
        int l = ai[1];
        int i1 = ai[2];
        int j1 = ai[3];
        double d1 = ad3[0];
        for(int l1 = 1; l1 <= j; l1++)
        {
            int k1 = l1 - 1;
            ad[k1] = d1;
            for(int i2 = 1; i2 <= k + i1 * j1; i2++)
                if(l1 - i2 <= 0)
                    ad[k1] += ad4[i2] * ad1[((i - 1) + l1) - i2];
                else
                    ad[k1] += ad4[i2] * ad[l1 - i2 - 1];

            for(int j2 = l1; j2 <= l; j2++)
                ad[k1] -= ad3[k + j2] * ad2[((i - 1) + l1) - j2];

        }

    }

    private void l_permute_vector(int i, double ad[], int ai[], int j, double ad1[])
    {
        byte byte0 = 1;
        int ai1[] = new int[i];
        for(int k = 0; k < i; k++)
            ai1[k] = ai[k] + 1;

        switch(j)
        {
        case 1: // '\001'
            byte0 = 1;
            break;

        case 2: // '\002'
            byte0 = 2;
            break;
        }
        imsls_permu(i, ad, ai1, byte0, ad1);
    }

    private void l_permute_vector(int i, int ai[], int ai1[], int j, int ai2[])
    {
        byte byte0 = 1;
        int ai3[] = new int[i];
        for(int k = 0; k < i; k++)
            ai3[k] = ai1[k] + 1;

        switch(j)
        {
        case 1: // '\001'
            byte0 = 1;
            break;

        case 2: // '\002'
            byte0 = 2;
            break;
        }
        imsls_permu(i, ai, ai3, byte0, ai2);
    }

    private void imsls_permu(int i, double ad[], int ai[], int j, double ad1[])
    {
        BLAS.copy(i, ad, ad1);
        if(i == 1)
            return;
        for(int k = 1; k <= i; k++)
            if(ai[k - 1] < 1 || ai[k - 1] > i)
            {
                String s = (new StringBuilder()).append("permutation[").append(k - 1).append("]").toString();
                Object aobj[] = {
                    s, new Integer(ai[k - 1] - 1), new Integer(0), new Integer(i - 1)
                };
                System.out.println( "GreaterEqual"+ aobj);
            } else
            {
                ai[k - 1] = -ai[k - 1];
            }

        if(j == 1)
        {
            for(int l = 1; l <= i; l++)
                if(ai[l - 1] <= 0)
                {
                    int j1 = l;
                    ai[j1 - 1] = -ai[j1 - 1];
                    for(int l1 = ai[j1 - 1]; ai[l1 - 1] <= 0; l1 = ai[l1 - 1])
                    {
                        double d = ad1[j1 - 1];
                        ad1[j1 - 1] = ad1[l1 - 1];
                        ad1[l1 - 1] = d;
                        ai[l1 - 1] = -ai[l1 - 1];
                        j1 = l1;
                    }

                }

        } else
        {
            for(int i1 = 1; i1 <= i; i1++)
            {
                if(ai[i1 - 1] > 0)
                    continue;
                ai[i1 - 1] = -ai[i1 - 1];
                for(int k1 = ai[i1 - 1]; k1 != i1; k1 = ai[k1 - 1])
                {
                    double d1 = ad1[i1 - 1];
                    ad1[i1 - 1] = ad1[k1 - 1];
                    ad1[k1 - 1] = d1;
                    ai[k1 - 1] = -ai[k1 - 1];
                }

            }

        }
    }

    private void imsls_permu(int i, int ai[], int ai1[], int j, int ai2[])
    {
        BLAS.copy(i, ai, ai2);
        if(i == 1)
            return;
        for(int k = 1; k <= i; k++)
            if(ai1[k - 1] < 1 || ai1[k - 1] > i)
            {
                String s = (new StringBuilder()).append("permutation[").append(k - 1).append("]").toString();
                Object aobj[] = {
                    s, new Integer(ai1[k - 1] - 1), new Integer(0), new Integer(i - 1)
                };
                System.out.println( "GreaterEqual"+ aobj);
            } else
            {
                ai1[k - 1] = -ai1[k - 1];
            }

        if(j == 1)
        {
            for(int l = 1; l <= i; l++)
                if(ai1[l - 1] <= 0)
                {
                    int j1 = l;
                    ai1[j1 - 1] = -ai1[j1 - 1];
                    for(int l1 = ai1[j1 - 1]; ai1[l1 - 1] <= 0; l1 = ai1[l1 - 1])
                    {
                        int i2 = ai2[j1 - 1];
                        ai2[j1 - 1] = ai2[l1 - 1];
                        ai2[l1 - 1] = i2;
                        ai1[l1 - 1] = -ai1[l1 - 1];
                        j1 = l1;
                    }

                }

        } else
        {
            for(int i1 = 1; i1 <= i; i1++)
            {
                if(ai1[i1 - 1] > 0)
                    continue;
                ai1[i1 - 1] = -ai1[i1 - 1];
                for(int k1 = ai1[i1 - 1]; k1 != i1; k1 = ai1[k1 - 1])
                {
                    int j2 = ai2[i1 - 1];
                    ai2[i1 - 1] = ai2[k1 - 1];
                    ai2[k1 - 1] = j2;
                    ai1[k1 - 1] = -ai1[k1 - 1];
                }

            }

        }
    }

    private int l_root_check(double ad[], int i, int j, int k)
        throws ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ZeroPolynomial.DidNotConvergeException, DidNotConvergeException
    {
        double ad1[] = new double[i + 1];
        double ad2[] = new double[j + 1];
        int l1 = 0;
        ad1[0] = ad2[0] = 1.0D;
        for(int l = 1; l <= i; l++)
            ad1[l] = -ad[l];

        for(int i1 = 1; i1 <= j; i1++)
            ad2[i1] = -ad[i + i1];

        if(i > 0)
        {
            ZeroPolynomial zeropolynomial = new ZeroPolynomial();
            Complex acomplex[] = zeropolynomial.computeRoots(ad1);
            for(int j1 = 0; j1 < i; j1++)
                if(Complex.abs(Complex.multiply(acomplex[j1], acomplex[j1])) <= 1.0D)
                    throw new ARMAMaxLikelihood.NonStationaryException("ARMAMaxLikelihood.NonStationary", null);

        }
        if(j > 0)
        {
            ZeroPolynomial zeropolynomial1 = new ZeroPolynomial();
            Complex acomplex1[] = zeropolynomial1.computeRoots(ad2);
            for(int k1 = 0; k1 < j; k1++)
                if(Complex.abs(Complex.multiply(acomplex1[k1], acomplex1[k1])) <= 1.0D)
                    throw new ARMAMaxLikelihood.NonInvertibleException("ARMAMaxLikelihood.NonInvertible", null);

        }
        return l1;
    }

    private void l_comp_coeff(double ad[], double ad1[], int i, int j, int k, int l, int i1, 
            int j1)
    {
        double ad2[] = new double[i1];
        ad2[0] = 1.0D;
        for(int l1 = 1; l1 < i1; l1++)
        {
            ad2[l1] = 0.0D;
            for(int k1 = 1; k1 <= Math.min(j, l1 - 1); k1++)
                ad2[l1] += ad1[i + k1] * ad2[l1 - k1];

            if(l1 <= i)
                ad2[l1] += ad1[l1];
            if(l1 <= j)
                ad2[l1] -= ad1[i + l1];
        }

        if(l == 0)
        {
            for(int i2 = 0; i2 < i1; i2++)
                ad[i2] = ad2[i2];

        }
        if(l > 0)
        {
            ad[0] = 1.0D;
            for(int j2 = 1; j2 < i1; j2++)
            {
                ad[j2] = 0.0D;
                double d = -1D;
                for(int k2 = 0; k2 <= l; k2++)
                {
                    if(j2 - k2 * k == 0)
                        ad[j2] += d * l_binom(l, k2);
                    if(j2 - k2 * k > 0)
                        ad[j2] -= d * l_binom(l, k2) * ad2[j2 - k2 * k];
                    d *= -1D;
                }

            }

        }
        if(j1 >= 0)
        {
            for(int l2 = j1; l2 < i1; l2++)
                ad[l2] = 0.0D;

        }
    }

    private int l_comp_num_coeff(double ad[], int i, int j, int k, int l, int i1)
    {
        if(j == 0)
            return i + k * l + 1;
        for(int j1 = i + j + k * l + 1; j1 < i1; j1++)
            if(Math.abs(ad[j1 - 1]) <= 0.0001D && Math.abs(ad[j1]) <= 0.0001D && Math.abs(ad[j1 - 1] - ad[j1]) <= 9.9999999999999995E-008D)
            {
                if(j1 > i + j + k * l + (i1 - i - j - k * l) / 4)
                    j1 = i + j + k * l + (i1 - i - j - k * l) / 4;
                return j1 + 1;
            }

        return i + j + k * l + (i1 - i - j - k * l) / 4 + 1;
    }

    private void l_comp_max_eta(int i, int j, double ad[], int k, double ad1[], double d, 
            double d1, int ai[], int l, int ai1[], double ad2[], int ai2[], 
            double ad3[], double ad4[], int i1, int ai3[], double ad5[], double ad6[])
    {
        double ad7[] = new double[4];
        double ad8[] = new double[4];
        int k2 = (j - i) + 1;
        double d2 = 1.0D;
        ad5[0] = 1.0D;
        ad6[0] = 1.0D;
        for(int k1 = 1; k1 < (j - i) + 1; k1++)
        {
            d2 += ad1[k1] * ad1[k1];
            ad5[k1] = ad5[k1 - 1] - ad1[k1];
            ad6[k1] = d1 * ad6[k1 - 1] - ad1[k1];
        }

        int i3 = 0;
        for(int j1 = i - 1; j1 < j; j1++)
        {
            ad7[0] = ad7[1] = ad[(j1 - i) + 1 + k];
            ad8[0] = ad7[0] / d;
            double d6;
            double d5 = d6 = 1.0D;
            int l2 = 1;
            for(int i2 = j1 + 1; i2 < j; i2++)
            {
                ad7[1] -= ad[(i2 - i) + 1 + k] * ad1[i2 - j1];
                double d3 = ad5[l2];
                d5 += d3 * d3;
                double d4 = ad6[l2];
                l2++;
                d6 += d4 * d4;
            }

            ad7[2] = BLAS.dot(j - j1, ad, i3 + k, ad5, 0);
            ad7[3] = BLAS.dot(j - j1, ad, i3 + k, ad6, 0);
            i3++;
            ad8[1] = ad7[1] / (Math.sqrt(d2) * d);
            ad7[1] /= d2;
            ad8[2] = ad7[2] / (Math.sqrt(d5) * d);
            ad7[2] /= d5;
            ad8[3] = ad7[3] / (Math.sqrt(d6) * d);
            ad7[3] /= d6;
            k2--;
            d2 -= ad1[k2] * ad1[k2];
            double d9 = Math.abs(ad8[0]);
            int j2 = 0;
            double d7 = d9;
            double d8 = ad7[0];
            double d10 = ad8[0];
            for(int l1 = 1; l1 <= 3; l1++)
                if(Math.abs(ad8[l1]) > d9)
                {
                    d9 = Math.abs(ad8[l1]);
                    j2 = l1;
                    d7 = d9;
                    d8 = ad7[l1];
                    d10 = ad8[l1];
                }

            if(j1 == i - 1 && l_out_list_search((j1 - i) + 1, i1, ai3, ai, l) == 1)
            {
                ai1[0] = 0;
                ad2[0] = 0.0D;
                ai2[0] = j2;
                ad3[0] = d8;
                ad4[0] = d10;
            } else
            if(j1 == i - 1)
            {
                ai1[0] = 0;
                ad2[0] = d7;
                ai2[0] = j2;
                ad3[0] = d8;
                ad4[0] = d10;
            }
            if(j1 <= i - 1 || l_out_list_search((j1 - i) + 1, i1, ai3, ai, l) != 0 || d7 <= ad2[0])
                continue;
            ai1[0] = (j1 - i) + 1;
            ad2[0] = d7;
            if(j1 == j - 1)
                ai2[0] = 4;
            else
                ai2[0] = j2;
            ad3[0] = d8;
            ad4[0] = d10;
        }

    }

    private void l_rem_from_residuals(int i, double d, int j, int k, int l, double ad[], 
            int i1, double ad1[], double d1)
    {
        int k2 = i;
        double d4 = d;
        int l2 = j;
        ad[k2 + i1] = ad[k2 + i1] - d4;
label0:
        switch(l2)
        {
        default:
            break;

        case 1: // '\001'
            for(int j1 = k2 + 1; j1 < (k - l) + 1; j1++)
                ad[j1 + i1] += ad1[j1 - k2] * d4;

            break;

        case 2: // '\002'
            int k1 = k2 + 1;
            do
            {
                if(k1 >= (k - l) + 1)
                    break label0;
                ad[k1 + i1] -= d4;
                double d2 = 0.0D;
                for(int i2 = 1; i2 <= k1 - k2; i2++)
                    d2 += ad1[i2];

                ad[k1 + i1] += d4 * d2;
                k1++;
            } while(true);

        case 3: // '\003'
            int l1 = k2 + 1;
            do
            {
                if(l1 >= (k - l) + 1)
                    break label0;
                ad[l1 + i1] -= d4 * Math.pow(d1, l1 - k2);
                double d3 = 0.0D;
                for(int j2 = 1; j2 < l1 - k2; j2++)
                    d3 += ad1[j2] * Math.pow(d1, l1 - k2 - j2);

                d3 += ad1[l1 - k2];
                ad[l1 + i1] += d4 * d3;
                l1++;
            } while(true);
        }
    }

    private void l_rem_from_values(int i, int j, double d, double ad[], double ad1[], int k, 
            int l, double d1)
    {
        ad[(i + l) - 1] -= d;
        switch(j)
        {
        case 1: // '\001'
        default:
            break;

        case 0: // '\0'
            for(int i1 = i + 1; i1 < (k - l) + 1; i1++)
                ad[(i1 + l) - 1] -= ad1[i1 - i] * d;

            break;

        case 2: // '\002'
            for(int j1 = i + 1; j1 < (k - l) + 1; j1++)
                ad[(j1 + l) - 1] -= d;

            break;

        case 3: // '\003'
            for(int k1 = i + 1; k1 < (k - l) + 1; k1++)
                ad[(k1 + l) - 1] -= d * Math.pow(d1, k1 - i);

            break;
        }
    }

    private void l_comp_psi(double ad[], double ad1[], int i, int j)
    {
        ad[0] = 1.0D;
        for(int l = 1; l < i; l++)
        {
            ad[l] = 0.0D;
            for(int k = 1; k <= Math.min(l, j - 1); k++)
                ad[l] += ad1[k] * ad[l - k];

        }

    }

    private double l_rse(int i, int j, double ad[], int k)
    {
        double d = 0.0D;
        for(int l = 0; l < (i - j) + 1; l++)
            d += ad[l + k] * ad[l + k];

        d = Math.sqrt(d);
        return d;
    }

    private double l_MAD(int i, int j, double ad[], int k)
    {
        double ad1[] = new double[(i - j) + 1];
        double d = 0.0D;
        double d1 = (1.0D / 0.0D);
        int l = (i - j) + 1;
        double d2 = Cdf.inverseNormal(0.75D);
        for(int i1 = 0; i1 < l; i1++)
            ad1[i1] = ad[i1 + k];

        d = Summary.median(ad1);
        for(int j1 = 0; j1 < l; j1++)
            ad1[j1] = Math.abs(ad1[j1] - d);

        if(l > 0)
        {
            Sort.ascending(ad1);
            int k1 = l / 2;
            int l1 = l % 2;
            if(l1 != 0)
                d1 = ad1[k1];
            else
                d1 = (ad1[k1 - 1] + ad1[k1]) / 2D;
            d1 /= d2;
        }
        return d1;
    }

    private void l_matrix_sol(int ai[], int ai1[], int ai2[], double ad[], double ad1[], double ad2[], double d, int i, int j, int k, double ad3[], int l, double d1, double ad4[], int ai3[], int i1)
        throws SingularMatrixException, Cholesky.NotSPDException, NotSPDException
    {
        int j4 = 0;
        double ad8[] = null;
        double ad9[][] = (double[][])null;
        double d6 = 0.0D;
        int k4 = 0;
        int l4 = 0;
        double ad11[][] = (double[][])null;
        double ad7[] = new double[(j - k) + 1];
        for(int j1 = 0; j1 < (j - k) + 1; j1++)
            ad7[j1] = ad3[j1 + l];

        double ad5[][] = new double[(j - k) + 1][i];
        int k1 = 0;
        for(int i5 = i1; k1 < i; i5 = ai2[i5])
        {
            int l3 = ai[i5];
            int i4 = ai1[i5];
            ad5[l3][k1] = 1.0D;
            if(i4 == 1)
            {
                for(int i2 = l3 + 1; i2 < (j - k) + 1 && i2 - l3 <= k - 1; i2++)
                    ad5[i2][k1] -= ad2[i2 - l3];

            }
            if(i4 == 2)
            {
                for(int j2 = l3 + 1; j2 < (j - k) + 1; j2++)
                {
                    double d2 = 1.0D;
                    for(int i3 = 1; i3 <= j2 - l3 && i3 <= k - 1; i3++)
                    {
                        d2 -= ad2[i3];
                        ad5[j2][k1] = d2;
                    }

                }

            }
            if(i4 == 3)
            {
                for(int k2 = l3 + 1; k2 < (j - k) + 1; k2++)
                {
                    double d3 = Math.pow(d, k2 - l3);
                    for(int j3 = 1; j3 < k2 - l3 && j3 <= k - 1; j3++)
                        d3 -= ad2[j3] * Math.pow(d, k2 - l3 - j3);

                    if(k2 - l3 <= k - 1)
                        d3 -= ad2[k2 - l3];
                    ad5[k2][k1] = d3;
                }

            }
            k1++;
        }

        LinearRegression linearregression = new LinearRegression(i, false);
        linearregression.update(ad5, ad7);
        double ad6[] = linearregression.getCoefficients();
        double d7 = 2.2204460492503131E-014D;
        try
        {
            SVD svd = new SVD(ad5, d7);
            ad8 = svd.getS();
            ad9 = svd.getV();
        }
        catch(STATException imslexception)
        {
            k4 = 1;
        }
        if(k4 > 0)
        {
            double ad10[][] = Matrix.multiply(Matrix.transpose(ad5), ad5);
            try
            {
                Cholesky cholesky = new Cholesky(ad10);
                ad11 = cholesky.inverse();
            }
            catch(STATException imslexception1)
            {
                l4 = 1;
            }
            if(l4 > 0)
            {
                for(int l1 = 0; l1 < i; l1++)
                    ad10[l1][l1] += 1E-008D;

                Cholesky cholesky1 = new Cholesky(ad10);
                ad11 = cholesky1.inverse();
            }
        }
        int l2 = 0;
        int j5 = i1;
        for(; l2 < i; l2++)
        {
            double d4;
            if(k4 > 0)
            {
                d4 = ad11[l2][l2];
            } else
            {
                d4 = 0.0D;
                for(int k3 = 0; k3 < i; k3++)
                    d4 += (ad9[l2][k3] / ad8[k3]) * (ad9[l2][k3] / ad8[k3]);

            }
            ad[j5] = ad6[l2];
            double d5 = ad[j5] / (d1 * Math.sqrt(d4));
            ad1[j5] = d5;
            if(l2 == 0)
            {
                d6 = Math.abs(d5);
                j4 = ai[j5];
            } else
            if(Math.abs(d5) < d6)
            {
                d6 = Math.abs(d5);
                j4 = ai[j5];
            }
            j5 = ai2[j5];
        }

        ad4[0] = d6;
        ai3[0] = j4;
    }

    private void l_list_append(int ai[], int ai1[], double ad[], double ad1[], int ai2[], int i, int j, 
            int k, double d, double d1)
    {
        if(i <= 0)
            return;
        ai[i - 1] = j;
        ai1[i - 1] = k;
        ad[i - 1] = d;
        ad1[i - 1] = d1;
        ai2[i - 1] = -1;
        if(i >= 2)
            ai2[i - 2] = i - 1;
    }

    private void l_list_delete(int ai[], int i, int j, int ai1[], int ai2[])
    {
        if(j <= 0)
            return;
        if(ai1[ai[0]] == i)
        {
            ai[0] = ai2[ai[0]];
        } else
        {
            int l = ai[0];
            int k = 0;
            do
            {
                if(k >= j - 1)
                    break;
                int i1 = ai2[l];
                if(ai1[i1] == i)
                {
                    ai2[l] = ai2[i1];
                    break;
                }
                l = i1;
                k++;
            } while(true);
        }
    }

    private void l_res_update(int i, int j, double ad[], int k, double ad1[], double ad2[])
    {
        for(int l = i - 1; l < j; l++)
        {
            ad[(l - i) + 1 + k] = ad1[l];
            for(int i1 = 1; i1 < i; i1++)
                ad[(l - i) + 1 + k] -= ad2[i1] * ad1[l - i1];

        }

    }

    private void l_compute_residuals(double ad[], int i, int j, int k, int l, double ad1[], double d, int i1, double ad2[], double ad3[], double ad4[], double ad5[], double ad6[])
    {
        int ai[] = new int[4];
        for(int j1 = 0; j1 < 4; j1++)
            ai[j1] = 0;

        for(int k1 = 0; k1 < 5; k1++)
            ad2[k1] = 0.0D;

        ad3[0] = (1.0D / 0.0D);
        ad4[0] = (1.0D / 0.0D);
        ad5[0] = (1.0D / 0.0D);
        double d2 = 1.0D;
        for(int k2 = 1; k2 <= i + k * l; k2++)
            d2 -= ad[k2];

        d2 *= d;
        for(int l2 = 0; l2 < i1; l2++)
        {
            if(l2 < Math.max(i + k * l, j))
            {
                ad6[l2] = 0.0D;
                continue;
            }
            double d1 = d2;
            for(int l1 = 0; l1 < i + k * l; l1++)
                d1 += ad[l1 + 1] * ad1[l2 - l1 - 1];

            for(int i2 = 0; i2 < j; i2++)
                d1 -= ad[i + 1 + i2] * ad6[l2 - i2 - 1];

            ad6[l2] = ad1[l2] - d1;
        }

        for(int i3 = 0; i3 < i1; i3++)
        {
            if(i3 >= Math.max(i + k * l, j))
                ad2[0] += ad6[i3] * ad6[i3];
            if((double)i3 / (double)i1 <= 0.25D)
            {
                ad2[1] += ad6[i3] * ad6[i3];
                ai[0]++;
            }
            if((double)i3 / (double)i1 > 0.25D && (double)i3 / (double)i1 <= 0.5D)
            {
                ad2[2] += ad6[i3] * ad6[i3];
                ai[1]++;
            }
            if((double)i3 / (double)i1 > 0.5D && (double)i3 / (double)i1 <= 0.75D)
            {
                ad2[3] += ad6[i3] * ad6[i3];
                ai[2]++;
            }
            if((double)i3 / (double)i1 > 0.75D)
            {
                ad2[4] += ad6[i3] * ad6[i3];
                ai[3]++;
            }
        }

        if(i1 - Math.max(i + k * l, j) > 0)
            ad2[0] = Math.sqrt(ad2[0] / (double)(i1 - Math.max(i + k * l, j)));
        for(int j2 = 0; j2 < 4; j2++)
            if(ai[j2] > 0)
                ad2[j2 + 1] = Math.sqrt(ad2[j2 + 1] / (double)ai[j2]);

        int j3 = i1 - Math.max(i + k * l, j);
        if(ad2[0] > 0.0D)
            ad3[0] = 2D * (double)i1 * Math.log(6.2831853999999998D * ad2[0]) + (double)i1 + 2D * (double)(i + j + 3);
        else
            ad3[0] = -1E+030D;
        if(j3 - i - j - 4 > 0)
            if(ad2[0] > 0.0D)
                ad4[0] = 2D * (double)j3 * (Math.log(ad2[0]) + (double)((i + j + 3) / (j3 - i - j - 4))) + (double)j3 * (Math.log(6.2831853999999998D) + 1.0D);
            else
                ad4[0] = -1E+030D;
        if(j3 > 0)
            if(ad2[0] > 0.0D)
                ad5[0] = 2D * (double)j3 * Math.log(ad2[0]) + (double)(i + j + 3) * Math.log(j3);
            else
                ad5[0] = -1E+030D;
    }

    private int l_out_list_search(int i, int j, int ai[], int ai1[], int k)
    {
        int l = 0;
        int i1 = j;
        for(; l < k; l++)
        {
            if(ai1[i1] == i)
                return 1;
            i1 = ai[i1];
        }

        return 0;
    }

    private int l_stage_1_comp(int i, int j, int ai[], double d, int ai1[], double ad[], 
            double ad1[], double ad2[], int k, double ad3[], double ad4[], double ad5[], double ad6[], 
            double ad7[], double ad8[], double ad9[], double d1, double d2, 
            double ad10[], int l, int i1, int ai2[], int ai3[], int ai4[], double ad11[], 
            double ad12[], int ai5[], int j1, double d3, double ad13[], double ad14[])
        throws ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ZeroPolynomial.DidNotConvergeException, ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, DidNotConvergeException, InitialMAException
    {
        int k1 = 0;
        double d6 = 0.0D;
        int ai6[] = {
            0
        };
        double ad15[] = {
            0.0D
        };
        int ai7[] = {
            0
        };
        double ad16[] = {
            0.0D
        };
        double ad17[] = {
            0.0D
        };
        int ai8[] = {
            1
        };
        int ai9[] = {
            0
        };
        double ad24[] = null;
        int k4 = ai[0];
        int l4 = ai[1];
        int i5 = ai[2];
        int j5 = ai[3];
        if(j5 > 0)
        {
            ai8[0] = i5;
            ai9[0] = j5;
        }
        ai2[0] = 0;
        double d4 = l_MAD(i, j, ad2, k);
        if(d4 == 0.0D)
        {
            double d10 = Math.pow(l_rse(i, j, ad2, k), 2D) / (double)((i - j) + 1);
            double d12 = Math.sqrt(d10);
            if(d12 <= 1.0000000000000001E-015D)
                return k1;
            d4 = d12;
        }
        l_comp_max_eta(j, i, ad2, k, ad3, d4, d1, ai3, ai1[0], ai6, ad15, ai7, ad16, ad17, j1, ai5, ad13, ad14);
        double d5;
        for(; ad15[0] > d2; l_comp_max_eta(j, i, ad2, k, ad3, d5, d1, ai3, ai1[0], ai6, ad15, ai7, ad16, ad17, j1, ai5, ad13, ad14))
        {
            ai1[0]++;
            l_rem_from_residuals(ai6[0], ad16[0], ai7[0], i, j, ad2, k, ad3, d1);
            l_rem_from_values(ai6[0], ai7[0], ad16[0], ad, ad4, i, j, d1);
            k1 = 1;
            l_list_append(ai3, ai4, ad11, ad12, ai5, ai1[0], ai6[0], ai7[0], ad16[0], ad17[0]);
            d5 = l_MAD(i, j, ad2, k);
            if(d5 != 0.0D)
                continue;
            double d11 = Math.pow(l_rse(i, j, ad2, k), 2D) / (double)((i - j) + 1);
            double d13 = Math.sqrt(d11);
            if(d13 <= 1.0000000000000001E-015D)
                break;
            d5 = d13;
        }

        if(k1 == 1 && l == 1)
        {
            if(j5 > 0)
            {
                Difference difference = new Difference();
                difference.excludeFirst(true);
                difference.setOrders(ai9);
                ad24 = difference.compute(ad, ai8);
                int k5 = difference.getObservationsLost();
                int l5 = i - k5;
                for(int i2 = 0; i2 < l5; i2++)
                    ad1[i2] = ad24[i2];

            }
            double d7;
            double ad18[];
            double ad21[];
            if(j5 > 0)
            {
                ARMA arma = new ARMA(k4, l4, ad24);
                arma.setCenter(false);
                arma.setMethod(1);
                arma.setConvergenceTolerance(d);
                arma.setRelativeError(d3);
                arma.setMaxIterations(10000);
                arma.setBackcasting(0, 0.001D);
                arma.compute();
                d7 = arma.getConstant();
                ad18 = arma.getAR();
                ad21 = arma.getMA();
            } else
            {
                ARMA arma1 = new ARMA(k4, l4, ad);
                arma1.setCenter(false);
                arma1.setMethod(1);
                arma1.setConvergenceTolerance(d);
                arma1.setRelativeError(d3);
                arma1.setMaxIterations(10000);
                arma1.setBackcasting(0, 0.001D);

                arma1.compute();

                d7 = arma1.getConstant();
                ad18 = arma1.getAR();
                ad21 = arma1.getMA();
            }
            ad10[0] = d7;
            for(int j2 = 1; j2 < k4 + l4 + 1; j2++)
            {
                if(j2 <= k4)
                    ad10[j2] = ad18[j2 - 1];
                if(j2 > k4)
                    ad10[j2] = ad21[j2 - k4 - 1];
            }

            int l1 = 0;
            int j4 = l_root_check(ad10, k4, l4, l1);
            for(int k2 = 1; k2 < k4 + l4 + 1; k2++)
            {
                if(k2 <= k4)
                    ad8[k2 - 1] = ad10[k2];
                if(k2 > k4)
                    ad9[k2 - k4 - 1] = ad10[k2];
            }


            if(j5 > 0)
            {
                double d14 = 0.0D;
                try
                {
                    ARMAMaxLikelihood armamaxlikelihood = new ARMAMaxLikelihood(k4, l4, ad24);
                    armamaxlikelihood.setMean(d14);
                    armamaxlikelihood.setAR(ad8);
                    armamaxlikelihood.setMA(ad9);
                    armamaxlikelihood.setMaxIterations(10000);
                    armamaxlikelihood.compute();
                    double d8 = armamaxlikelihood.getConstant();
                    double ad19[] = armamaxlikelihood.getAR();
                    double ad22[] = armamaxlikelihood.getMA();
                    for(int l2 = 0; l2 < k4; l2++)
                        ad8[l2] = ad19[l2];

                    for(int i3 = 0; i3 < l4; i3++)
                        ad9[i3] = ad22[i3];

                    ad10[0] = d8;
                    for(int j3 = 1; j3 < k4 + l4 + 1; j3++)
                    {
                        if(j3 <= k4)
                            ad10[j3] = ad8[j3 - 1];
                        if(j3 > k4)
                            ad10[j3] = ad9[j3 - k4 - 1];
                    }

                }
                catch(Exception ex) {System.out.println(ex.toString()); }
            } else
            {
                double d15 = 0.0D;
                try
                {
                    ARMAMaxLikelihood armamaxlikelihood1 = new ARMAMaxLikelihood(k4, l4, ad);
                    armamaxlikelihood1.setMean(d15);
                    armamaxlikelihood1.setAR(ad8);
                    armamaxlikelihood1.setMA(ad9);
                    armamaxlikelihood1.setMaxIterations(10000);
                    armamaxlikelihood1.compute();
                    double d9 = armamaxlikelihood1.getConstant();
                    double ad20[] = armamaxlikelihood1.getAR();
                    double ad23[] = armamaxlikelihood1.getMA();
                    for(int k3 = 0; k3 < k4; k3++)
                        ad8[k3] = ad20[k3];

                    for(int l3 = 0; l3 < l4; l3++)
                        ad9[l3] = ad23[l3];

                    ad10[0] = d9;
                    for(int i4 = 1; i4 < k4 + l4 + 1; i4++)
                    {
                        if(i4 <= k4)
                            ad10[i4] = ad8[i4 - 1];
                        if(i4 > k4)
                            ad10[i4] = ad9[i4 - k4 - 1];
                    }

                }
                catch(Exception ex) { }
            }
            l_comp_coeff(ad3, ad10, k4, l4, i5, j5, i, j);
            l_comp_psi(ad4, ad3, i, j);
            l_res_update(j, i, ad2, k, ad, ad3);
        }
        return k1;
    }

    private int l_stage_2_comp(int i, int j, int ai[], double d, double d1, 
            int ai1[], double ad[], double ad1[], double ad2[], int k, double ad3[], double ad4[], 
            double ad5[], double ad6[], double ad7[], double ad8[], double ad9[], double d2, 
            double d3, double ad10[], int l, int i1, int ai2[], int ai3[], 
            int ai4[], double ad11[], double ad12[], int ai5[], int ai6[], double d4)
        throws ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ZeroPolynomial.DidNotConvergeException, ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, SingularMatrixException, Cholesky.NotSPDException, NotSPDException, DidNotConvergeException, InitialMAException
    {
        double ad13[] = {
            0.0D
        };
        int ai7[] = {
            0
        };
        boolean flag = false;
        double d11 = 10000000D;
        int l1 = 1;
        int ai8[] = {
            1
        };
        int ai9[] = {
            0
        };
        double ad20[] = null;
        int l4 = ai[0];
        int i5 = ai[1];
        int j5 = ai[2];
        int k5 = ai[3];
        if(k5 > 0)
        {
            ai8[0] = j5;
            ai9[0] = k5;
        }
        l_comp_coeff(ad3, ad10, l4, i5, j5, k5, i, j);
        l_comp_psi(ad4, ad3, i, j);
        l_res_update(j, i, ad2, k, ad, ad3);
        double d5 = l_MAD(i, j, ad2, k);
        l_matrix_sol(ai3, ai4, ai5, ad11, ad12, ad3, d2, ai1[0], i, j, ad2, k, d5, ad13, ai7, ai6[0]);
        if(ad13[0] <= d3)
        {
            l_list_delete(ai6, ai7[0], ai1[0], ai3, ai5);
            ai1[0]--;
            l1 = 0;
        }
        if(l == 2 && ad13[0] > d3)
        {
            int k4 = ai6[0];
            for(int i2 = 0; i2 < ai1[0]; i2++)
            {
                l_rem_from_values(ai3[k4], ai4[k4], ad11[k4], ad, ad4, i, j, d2);
                k4 = ai5[k4];
            }

            l_res_update(j, i, ad2, k, ad, ad3);
            double d7 = l_rse(i, j, ad2, k);
            if(k5 > 0)
            {
                Difference difference = new Difference();
                difference.excludeFirst(true);
                difference.setOrders(ai9);
                ad20 = difference.compute(ad, ai8);
                int i6 = difference.getObservationsLost();
                int l5 = i - i6;
                for(int j2 = 0; j2 < l5; j2++)
                    ad1[j2] = ad20[j2];

            }
            double d8;
            double ad14[];
            double ad17[];
            if(k5 > 0)
            {
                ARMA arma = new ARMA(l4, i5, ad20);
                arma.setCenter(false);
                arma.setMethod(1);
                arma.setConvergenceTolerance(d);
                arma.setRelativeError(d4);
                arma.setMaxIterations(10000);
                arma.setBackcasting(0, 0.001D);
                arma.compute();

                d8 = arma.getConstant();
                ad14 = arma.getAR();
                ad17 = arma.getMA();
            } else
            {
                ARMA arma1 = new ARMA(l4, i5, ad);
                arma1.setCenter(false);
                arma1.setMethod(1);
                arma1.setConvergenceTolerance(d);
                arma1.setRelativeError(d4);
                arma1.setMaxIterations(10000);
                arma1.setBackcasting(0, 0.001D);
 
                arma1.compute();

                d8 = arma1.getConstant();
                ad14 = arma1.getAR();
                ad17 = arma1.getMA();
            }
            ad10[0] = d8;
            for(int k2 = 1; k2 < l4 + i5 + 1; k2++)
            {
                if(k2 <= l4)
                    ad10[k2] = ad14[k2 - 1];
                if(k2 > l4)
                    ad10[k2] = ad17[k2 - l4 - 1];
            }

            int j1 = 0;
            int k1 = l_root_check(ad10, l4, i5, j1);
            for(int l2 = 1; l2 < l4 + i5 + 1; l2++)
            {
                if(l2 <= l4)
                    ad8[l2 - 1] = ad10[l2];
                if(l2 > l4)
                    ad9[l2 - l4 - 1] = ad10[l2];
            }

            if(k5 > 0)
            {
                double d13 = 0.0D;
                try
                {
                    ARMAMaxLikelihood armamaxlikelihood = new ARMAMaxLikelihood(l4, i5, ad20);
                    armamaxlikelihood.setMean(d13);
                    armamaxlikelihood.setAR(ad8);
                    armamaxlikelihood.setMA(ad9);
                    armamaxlikelihood.setMaxIterations(10000);
                    armamaxlikelihood.compute();
                    double d9 = armamaxlikelihood.getConstant();
                    double ad15[] = armamaxlikelihood.getAR();
                    double ad18[] = armamaxlikelihood.getMA();
                    for(int i3 = 0; i3 < l4; i3++)
                        ad8[i3] = ad15[i3];

                    for(int j3 = 0; j3 < i5; j3++)
                        ad9[j3] = ad18[j3];

                    ad10[0] = d9;
                    for(int k3 = 1; k3 < l4 + i5 + 1; k3++)
                    {
                        if(k3 <= l4)
                            ad10[k3] = ad8[k3 - 1];
                        if(k3 > l4)
                            ad10[k3] = ad9[k3 - l4 - 1];
                    }

                }
                catch(Exception ex) { System.out.println(ex.toString());}
            } else
            {
                double d14 = 0.0D;
                try
                {
                    ARMAMaxLikelihood armamaxlikelihood1 = new ARMAMaxLikelihood(l4, i5, ad);
                    armamaxlikelihood1.setMean(d14);
                    armamaxlikelihood1.setAR(ad8);
                    armamaxlikelihood1.setMA(ad9);
                    armamaxlikelihood1.setMaxIterations(10000);
                    armamaxlikelihood1.compute();
                    double d10 = armamaxlikelihood1.getConstant();
                    double ad16[] = armamaxlikelihood1.getAR();
                    double ad19[] = armamaxlikelihood1.getMA();
                    for(int l3 = 0; l3 < l4; l3++)
                        ad8[l3] = ad16[l3];

                    for(int i4 = 0; i4 < i5; i4++)
                        ad9[i4] = ad19[i4];

                    ad10[0] = d10;
                    for(int j4 = 1; j4 < l4 + i5 + 1; j4++)
                    {
                        if(j4 <= l4)
                            ad10[j4] = ad8[j4 - 1];
                        if(j4 > l4)
                            ad10[j4] = ad9[j4 - l4 - 1];
                    }

                }
                catch(Exception ex) {System.out.println(ex.toString()); }
            }

            l_comp_coeff(ad3, ad10, l4, i5, j5, k5, i, j);
            l_res_update(j, i, ad2, k, ad, ad3);
            l_comp_psi(ad4, ad3, i, j);
            double d6 = l_rse(i, j, ad2, k);
            double d12 = Math.abs(d6 - d7) / Math.abs(d7);
            if(d12 > d1)
                l1 = 0;
        }
        return l1;
    }

    private void l_ts_outlier_identification_driver(int i, int ai[], double ad[], double d, double d1, 
            double d2, double ad1[], double ad2[], double ad3[], double ad4[], double ad5[], 
            double ad6[], double ad7[], double ad8[], double ad9[], double ad10[], double ad11[], double ad12[], 
            double ad13[], double ad14[], double ad15[], double ad16[], double ad17[], int ai1[], double ad18[], 
            double ad19[], int ai2[], int ai3[], int ai4[], int ai5[], int ai6[], double d3)
        throws ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException,ZeroPolynomial.DidNotConvergeException, ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, SingularMatrixException, Cholesky.NotSPDException, NotSPDException, DidNotConvergeException, InitialMAException
    {
        int i7;
        int j7;
        int l7;
        double d13;
label0:
        {
            int i9 = 0;
            int j9 = 0;
            int k9 = 0;
            double ad20[] = {
                0.0D
            };
            int ai7[] = {
                0
            };
            double ad21[] = {
                0.0D
            };
            double ad22[] = {
                0.0D
            };
            int ai8[] = {
                0
            };
            double ad23[] = {
                0.0D
            };
            int ai9[] = {
                0
            };
            int ai10[] = {
                1
            };
            int ai11[] = {
                0
            };
            double ad28[] = null;
            double d12 = 1.9984014443252818E-016D;
            i7 = ai[0];
            j7 = ai[1];
            int k7 = ai[2];
            l7 = ai[3];
            if(l7 > 0)
            {
                ai10[0] = k7;
                ai11[0] = l7;
            }
            d13 = Summary.median(ad8);
            BLAS.add(i, -d13, ad8, 0, 1);
            if(i7 == 0 && j7 == 0)
            {
                BLAS.copy(i, ad8, ad9);
                ad15[0] = d13;
                l_comp_coeff(ad6, ad15, i7, j7, k7, l7, i, -1);
                int i8 = k7 * l7 + 1;
                l_comp_psi(ad7, ad6, i, i8);
                l_res_update(i8, i, ad5, i8 - 1, ad9, ad6);
                ai4[0] = 0;
                double d7 = l_MAD(i, i8, ad5, i8 - 1);
                int l11 = (i - i8) + 1;
                double ad24[] = new double[l11];
                double ad26[] = new double[l11];
                l_comp_max_eta(i8, i, ad5, i8 - 1, ad6, d7, d, ai1, ai4[0], ai8, ad20, ai7, ad21, ad22, ai5[0], ai2, ad24, ad26);
                double d8;
                for(; ad20[0] > d1; l_comp_max_eta(i8, i, ad5, i8 - 1, ad6, d8, d, ai1, ai4[0], ai8, ad20, ai7, ad21, ad22, ai5[0], ai2, ad24, ad26))
                {
                    ai4[0]++;
                    l_rem_from_residuals(ai8[0], ad21[0], ai7[0], i, i8, ad5, i8 - 1, ad6, d);
                    l_rem_from_values(ai8[0], ai7[0], ad21[0], ad9, ad7, i, i8, d);
                    l_list_append(ai1, ai3, ad18, ad19, ai2, ai4[0], ai8[0], ai7[0], ad21[0], ad22[0]);
                    d8 = l_MAD(i, i8, ad5, i8 - 1);
                    if(d8 != 0.0D)
                        continue;
                    double d14 = Math.pow(l_rse(i, i8, ad5, i8 - 1), 2D) / (double)((i - i8) + 1);
                    double d16 = Math.sqrt(d14);
                    if(d16 <= 1.0000000000000001E-015D)
                        break;
                    d8 = d16;
                }

                if(ai4[0] == 0)
                {
                    l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                    l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                    for(int j = 0; j < i; j++)
                        ad4[j] = ad8[j] + d13;

                } else
                {
                    for(int k = 0; k < i; k++)
                        ad9[k] = ad8[k];

                    l_res_update(i8, i, ad5, i8 - 1, ad9, ad6);
                    double d9 = l_MAD(i, i8, ad5, i8 - 1);
                    l_matrix_sol(ai1, ai3, ai2, ad18, ad19, ad6, d, ai4[0], i, i8, ad5, i8 - 1, d9, ad23, ai9, ai5[0]);
                    while(ad23[0] <= d1) 
                    {
                        l_list_delete(ai5, ai9[0], ai4[0], ai1, ai2);
                        ai4[0]--;
                        if(ai4[0] == 0)
                        {
                            l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                            l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                            for(int l = 0; l < i; l++)
                                ad4[l] = ad8[l] + d13;

                            return;
                        }
                        l_matrix_sol(ai1, ai3, ai2, ad18, ad19, ad6, d, ai4[0], i, i8, ad5, i8 - 1, d9, ad23, ai9, ai5[0]);
                    }
                    int l10 = ai5[0];
                    for(int i1 = 0; i1 < ai4[0];)
                    {
                        l_rem_from_values(ai1[l10], ai3[l10], ad18[l10], ad9, ad7, i, i8, d);
                        i1++;
                        l10 = ai2[l10];
                    }

                    l_res_update(i8, i, ad5, i8 - 1, ad9, ad6);
                    for(int j1 = 0; j1 < i; j1++)
                        ad4[j1] = ad9[j1] + d13;

                    int k10 = ai5[0];
                    for(int k1 = 0; k1 < ai4[0];)
                    {
                        ai1[k10] += i8;
                        k1++;
                        k10 = ai2[k10];
                    }

                    if(ad15 != null)
                    {
                        l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                        l_compute_residuals(ad16, i7, j7, k7, l7, ad9, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                    }
                }
            }
            if(i7 + j7 <= 0)
                break label0;
            if(l7 > 0)
            {
                Difference difference = new Difference();
                difference.excludeFirst(true);
                difference.setOrders(ai11);
                ad28 = difference.compute(ad8, ai10);
                int k11 = difference.getObservationsLost();
                int j11 = i - k11;
                for(int l1 = 0; l1 < j11; l1++)
                    ad10[l1] = ad28[l1];

            }
            double d4;
            double ad29[];
            double ad32[];
            if(l7 > 0)
            {
                ARMA arma = new ARMA(i7, j7, ad28);
                arma.setCenter(false);
                arma.setMethod(1);
                arma.setConvergenceTolerance(d12);
                arma.setRelativeError(d3);
                arma.setMaxIterations(10000);
                arma.setBackcasting(0, 0.001D);

                arma.compute();

                d4 = arma.getConstant();
                ad29 = arma.getAR();
                ad32 = arma.getMA();
            } else
            {
                ARMA arma1 = new ARMA(i7, j7, ad8);
                arma1.setCenter(false);
                arma1.setMethod(1);
                arma1.setConvergenceTolerance(d12);
                arma1.setRelativeError(d3);
                arma1.setMaxIterations(10000);
                arma1.setBackcasting(0, 0.001D);

                arma1.compute();

                d4 = arma1.getConstant();
                ad29 = arma1.getAR();
                ad32 = arma1.getMA();
            }
            ad15[0] = d4;
            for(int i2 = 1; i2 < i7 + j7 + 1; i2++)
            {
                if(i2 <= i7)
                    ad15[i2] = ad29[i2 - 1];
                if(i2 > i7)
                    ad15[i2] = ad32[i2 - i7 - 1];
            }

            int k8 = 0;
            int l8 = l_root_check(ad15, i7, j7, k8);
            for(int j2 = 1; j2 < i7 + j7 + 1; j2++)
            {
                if(j2 <= i7)
                    ad13[j2 - 1] = ad15[j2];
                if(j2 > i7)
                    ad14[j2 - i7 - 1] = ad15[j2];
            }


            if(l7 > 0)
            {
                double d18 = 0.0D;
                try
                {
                    ARMAMaxLikelihood armamaxlikelihood = new ARMAMaxLikelihood(i7, j7, ad28);
                    armamaxlikelihood.setMean(d18);
                    armamaxlikelihood.setAR(ad13);
                    armamaxlikelihood.setMA(ad14);
                    armamaxlikelihood.setMaxIterations(10000);
                    armamaxlikelihood.compute();
                    double d5 = armamaxlikelihood.getConstant();
                    double ad30[] = armamaxlikelihood.getAR();
                    double ad33[] = armamaxlikelihood.getMA();
                    for(int k2 = 0; k2 < i7; k2++)
                        ad13[k2] = ad30[k2];

                    for(int l2 = 0; l2 < j7; l2++)
                        ad14[l2] = ad33[l2];

                    ad15[0] = d5;
                    for(int i3 = 1; i3 < i7 + j7 + 1; i3++)
                    {
                        if(i3 <= i7)
                            ad15[i3] = ad13[i3 - 1];
                        if(i3 > i7)
                            ad15[i3] = ad14[i3 - i7 - 1];
                    }

                }
                catch(Exception ex) {ex.toString(); }
            } else
            {
                double d19 = 0.0D;
                try
                {
                    ARMAMaxLikelihood armamaxlikelihood1 = new ARMAMaxLikelihood(i7, j7, ad8);
                    armamaxlikelihood1.setMean(d19);
                    armamaxlikelihood1.setAR(ad13);
                    armamaxlikelihood1.setMA(ad14);
                    armamaxlikelihood1.setMaxIterations(10000);
                    armamaxlikelihood1.compute();
                    double d6 = armamaxlikelihood1.getConstant();
                    double ad31[] = armamaxlikelihood1.getAR();
                    double ad34[] = armamaxlikelihood1.getMA();
                    for(int j3 = 0; j3 < i7; j3++)
                        ad13[j3] = ad31[j3];

                    for(int k3 = 0; k3 < j7; k3++)
                        ad14[k3] = ad34[k3];

                    ad15[0] = d6;
                    for(int l3 = 1; l3 < i7 + j7 + 1; l3++)
                    {
                        if(l3 <= i7)
                            ad15[l3] = ad13[l3 - 1];
                        if(l3 > i7)
                            ad15[l3] = ad14[l3 - i7 - 1];
                    }

                }
                catch(Exception ex) {System.out.println(ex.toString()); }
            }

            int j8 = -1;
            l_comp_coeff(ad6, ad15, i7, j7, k7, l7, i, j8);
            j8 = l_comp_num_coeff(ad6, i7, j7, k7, l7, i);
            l_comp_psi(ad7, ad6, i, j8);
            l_res_update(j8, i, ad5, j8 - 1, ad8, ad6);
            double d10 = l_MAD(i, j8, ad5, j8 - 1);
            if(d10 == 0.0D)
            {
                double d15 = Math.pow(l_rse(i, j8, ad5, j8 - 1), 2D) / (double)((i - j8) + 1);
                double d17 = Math.sqrt(d15);
                if(d17 <= 1.0000000000000001E-015D)
                    return;
                double d11 = d17;
            }
            BLAS.copy(i, ad8, ad9);
            ai4[0] = 0;
            int i12 = (i - j8) + 1;
            double ad25[] = new double[i12];
            double ad27[] = new double[i12];
            int j10 = 1;
            do
                j9 = l_stage_1_comp(i, j8, ai, d12, ai4, ad9, ad10, ad5, j8 - 1, ad6, ad7, ad17, ad11, ad12, ad13, ad14, d, d1, ad15, j10, i9, ai6, ai1, ai3, ad18, ad19, ai2, ai5[0], d3, ad25, ad27);
            while(j9 > 0 && ai4[0] > 0);
            if(ai4[0] == 0)
            {
                for(int i4 = 0; i4 < i; i4++)
                    ad4[i4] = ad8[i4] + d13;

                if(ad15 != null)
                {
                    l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                    l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                }
                break label0;
            }
            j10 = 2;
            int l9 = ai4[0];
            int i10 = 0;
            for(int j4 = 0; j4 < i; j4++)
                ad9[j4] = ad8[j4];

            k9 = l_stage_2_comp(i, j8, ai, d12, d2, ai4, ad9, ad10, ad5, j8 - 1, ad6, ad7, ad17, ad11, ad12, ad13, ad14, d, d1, ad15, j10, i9, ai6, ai1, ai3, ad18, ad19, ai2, ai5, d3);
            do
            {
                if(k9 != 0)
                    break;
                if(ai4[0] == 0)
                {
                    for(int k4 = 0; k4 < i; k4++)
                        ad4[k4] = ad8[k4] + d13;

                    if(ad15 != null)
                    {
                        l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                        l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                    }
                    break label0;
                }
                if(l9 > ai4[0])
                {
                    i10 = 0;
                    l9 = ai4[0];
                } else
                {
                    i10++;
                }
                for(int l4 = 0; l4 < i; l4++)
                    ad9[l4] = ad8[l4];

                k9 = l_stage_2_comp(i, j8, ai, d12, d2, ai4, ad9, ad10, ad5, j8 - 1, ad6, ad7, ad17, ad11, ad12, ad13, ad14, d, d1, ad15, j10, i9, ai6, ai1, ai3, ad18, ad19, ai2, ai5, d3);
                if(i10 > 10)
                    k9 = 1;
            } while(true);
            j10 = 3;
            l_comp_coeff(ad6, ad15, i7, j7, k7, l7, i, j8);
            l_comp_psi(ad7, ad6, i, j8);
            BLAS.copy(i, ad8, ad9);
            l_res_update(j8, i, ad5, j8 - 1, ad9, ad6);
            ai5[0] = 0;
            ai4[0] = 0;
            do
                j9 = l_stage_1_comp(i, j8, ai, d12, ai4, ad9, ad10, ad5, j8 - 1, ad6, ad7, ad17, ad11, ad12, ad13, ad14, d, d1, ad15, j10, i9, ai6, ai1, ai3, ad18, ad19, ai2, ai5[0], d3, ad25, ad27);
            while(j9 > 0 && ai4[0] > 0);
            if(ai4[0] == 0)
            {
                for(int i5 = 0; i5 < i; i5++)
                    ad4[i5] = ad8[i5] + d13;

                if(ad15 != null)
                {
                    l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                    l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                }
                break label0;
            }
            l_comp_coeff(ad6, ad15, i7, j7, k7, l7, i, j8);
            l_comp_psi(ad7, ad6, i, j8);
            BLAS.copy(i, ad8, ad9);
            k9 = l_stage_2_comp(i, j8, ai, d12, d2, ai4, ad9, ad10, ad5, j8 - 1, ad6, ad7, ad17, ad11, ad12, ad13, ad14, d, d1, ad15, j10, i9, ai6, ai1, ai3, ad18, ad19, ai2, ai5, d3);
            do
            {
                if(k9 != 0)
                    break;
                for(int j5 = 0; j5 < i; j5++)
                    ad9[j5] = ad8[j5];

                if(ai4[0] == 0)
                {
                    for(int k5 = 0; k5 < i; k5++)
                        ad4[k5] = ad8[k5] + d13;

                    if(ad15 != null)
                    {
                        l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                        l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                    }
                    break label0;
                }
                k9 = l_stage_2_comp(i, j8, ai, d12, d2, ai4, ad9, ad10, ad5, j8 - 1, ad6, ad7, ad17, ad11, ad12, ad13, ad14, d, d1, ad15, j10, i9, ai6, ai1, ai3, ad18, ad19, ai2, ai5, d3);
            } while(true);
            if(ai4[0] == 0)
            {
                for(int l5 = 0; l5 < i; l5++)
                    ad4[l5] = ad8[l5] + d13;

                if(ad15 != null)
                {
                    l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                    l_compute_residuals(ad16, i7, j7, k7, l7, ad8, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                }
            } else
            {
                int i11 = ai5[0];
                for(int i6 = 0; i6 < ai4[0];)
                {
                    l_rem_from_values(ai1[i11], ai3[i11], ad18[i11], ad9, ad7, i, j8, d);
                    i6++;
                    ai1[i11] += j8;
                    i11 = ai2[i11];
                }

                for(int j6 = 0; j6 < i; j6++)
                    ad4[j6] = ad9[j6] + d13;

                if(ad15 != null)
                {
                    l_comp_coeff_phi(i7, j7, k7, l7, ad15, ad16);
                    l_compute_residuals(ad16, i7, j7, k7, l7, ad9, 0.0D, i, ad17, ad1, ad2, ad3, ad5);
                }
            }
        }
        if(i7 + j7 > 0)
            if(l7 == 0)
            {
                ad15[0] = 1.0D;
                for(int k6 = 1; k6 <= i7; k6++)
                    ad15[0] -= ad15[k6];

                ad15[0] = ad15[0] * d13;
            } else
            {
                ad15[0] = 0.0D;
            }
        Object obj = null;
        Object obj1 = null;
        if(ai6[0] == 1)
        {
            for(int l6 = 0; l6 < 5; l6++)
                ad17[l6] = (1.0D / 0.0D);

        }
        if(ad15 == null)
            ad17[0] = (1.0D / 0.0D);
    }

    private void l_comp_coeff_phi(int i, int j, int k, int l, double ad[], double ad1[])
    {
        ad1[0] = 1.0D;
        if(l > 0)
        {
            for(int i1 = 1; i1 <= i + k * l; i1++)
            {
                ad1[i1] = 0.0D;
                int i2 = -1;
                for(int k1 = 0; k1 <= l; k1++)
                {
                    if(i1 - k * k1 == 0)
                        ad1[i1] += (double)i2 * l_binom(l, k1);
                    if(i1 - k * k1 > 0 && i1 - k * k1 <= i)
                        ad1[i1] -= (double)i2 * l_binom(l, k1) * ad[i1 - k * k1];
                    i2 *= -1;
                }

            }

        } else
        {
            for(int l1 = 1; l1 <= i; l1++)
                ad1[l1] = ad[l1];

        }
        for(int j1 = 1; j1 <= j; j1++)
            ad1[i + k * l + j1] = ad[i + j1];

    }



}