
package utility.forecasting;

// Referenced classes of package com.imsl.stat:
//            Sort

public final class Cdf
{

    private static final double EPSILON_SMALL = 1.1102230246251999E-016D;
    private static final double EPSILON_LARGE = 2.2204460492503E-016D;
    private Cdf()
    {
    }

    private static double logCombination(int i, int j)
    {
        double d = Sfun.logGamma(i + 1);
        double d1 = Sfun.logGamma(j + 1);
        double d2 = Sfun.logGamma((i - j) + 1);
        return d - d1 - d2;
    }

    public static double binomial(int i, int j, double d)
    {
        if(j <= 0 || d > 1.0D || d < 0.0D || Double.isNaN(d))
            return (0.0D / 0.0D);
        if(i < 0)
            return 0.0D;
        if(i > j)
            return 1.0D;
        if(d == 0.0D)
            return 1.0D;
        if(d == 1.0D)
            return 0.0D;
        double d1 = Math.log(d);
        double d2 = Math.log(1.0D - d);
        double d3 = Sfun.logGamma(j + 1);
        double d4 = 0.0D;
        if((double)i < (double)j * d)
        {
            for(int k = 0; k <= i; k++)
            {
                double d5 = Sfun.logGamma(k + 1);
                double d7 = Sfun.logGamma((j - k) + 1);
                double d9 = d3 - d5 - d7;
                d4 += Math.exp(d9 + (double)k * d1 + (double)(j - k) * d2);
            }

        } else
        {
            for(int l = i; l >= 0; l--)
            {
                double d6 = Sfun.logGamma(l + 1);
                double d8 = Sfun.logGamma((j - l) + 1);
                double d10 = d3 - d6 - d8;
                d4 += Math.exp(d10 + (double)l * d1 + (double)(j - l) * d2);
            }

        }
        return d4;
    }

    public static double bivariateNormal(double d, double d1, double d2)
    {
        double d3 = 0.0D;
        double d4 = 0.0D;
        double d6 = 0.0D;
        double d7 = 0.0D;
        if(Math.abs(d2) > 1.0D)
            return (0.0D / 0.0D);
        if(Math.abs(d2) == 1.0D)
            if(d2 > 0.0D)
            {
                double d8 = Math.min(d, d1);
                return normal(d8);
            } else
            {
                return Math.max((normal(d) + normal(d1)) - 1.0D, 0.0D);
            }
        double d9 = 1.0D / Math.sqrt(1.0D - d2 * d2);
        double d10 = d * d1;
        boolean flag = false;
        boolean flag1 = false;
        boolean flag2 = false;
        boolean flag3 = false;
        if(d10 != 0.0D)
        {
            d3 = d9 * (d1 / d - d2);
            d4 = d9 * (d / d1 - d2);
        } else
        if(d == 0.0D && d1 == 0.0D)
            d3 = d4 = d9 * (1.0D - d2);
        else
        if(d != 0.0D)
        {
            d7 = d >= 0.0D ? 0.25D : -0.25D;
            d3 = -d9 * d2;
            flag2 = true;
        } else
        {
            d6 = d1 >= 0.0D ? 0.25D : -0.25D;
            d4 = -d9 * d2;
            if(d4 < 0.0D)
            {
                flag1 = true;
                d4 = -d4;
            }
            d7 = OwenT(d1, d4);
            if(flag1)
                d7 = -d7;
            flag3 = true;
        }
        if(!flag3)
        {
            if(d3 < 0.0D)
            {
                flag = true;
                d3 = -d3;
            }
            d6 = OwenT(d, d3);
            if(flag)
                d6 = -d6;
            if(!flag2)
            {
                if(d4 < 0.0D)
                {
                    flag1 = true;
                    d4 = -d4;
                }
                d7 = OwenT(d1, d4);
                if(flag1)
                    d7 = -d7;
            }
        }
        double d11;
        if(d <= 0.0D)
            d11 = normal(d);
        else
            d11 = 1.0D - normal(-d);
        double d12;
        if(d1 <= 0.0D)
            d12 = normal(d1);
        else
            d12 = 1.0D - normal(-d1);
        double d5 = 0.5D * (d11 + d12) - d6 - d7;
        if(d10 <= 0.0D && (d10 != 0.0D || d + d1 < 0.0D))
            d5 -= 0.5D;
        d5 = Math.min(Math.max(0.0D, d5), 1.0D);
        return d5;
    }

    private static double OwenT(double d, double d1)
    {
        double ad[] = {
            0.99999999999999989D, -0.99999999999988798D, 0.99999999998290745D, -0.99999999896282499D, 0.99999996660459367D, -0.99999933986272471D, 0.99999125611136963D, -0.99991777624463385D, 0.99942835555870135D, -0.99697311720723003D, 
            0.98751448037275302D, -0.95915857980572883D, 0.89246305511006707D, -0.76893425990464004D, 0.5889352846848469D, -0.38380345160440255D, 0.20317601701045299D, -0.082813631607004987D, 0.024167984735759578D, -0.0044676566663971826D, 
            0.00039141169402373836D
        };
        double ad1[] = {
            0.0035082039676451716D, 0.031279042338030756D, 0.085266826283219452D, 0.16245071730812277D, 0.25851196049125436D, 0.36807553840697532D, 0.48501092905604698D, 0.60277514152618572D, 0.71477884217753229D, 0.81475510988760103D, 
            0.89711029755948968D, 0.95723808085944262D, 0.99178832974629705D
        };
        double ad2[] = {
            0.018831438115323503D, 0.018567086243977651D, 0.018042093461223385D, 0.017263829606398752D, 0.016243219975989858D, 0.014994592034116705D, 0.013535474469662089D, 0.011886351605820165D, 0.010070377242777432D, 0.0081130545742299585D, 
            0.0060419009528470237D, 0.0038862217010742057D, 0.0016793031084546091D
        };
        int ai[] = {
            1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 
            2, 3, 4, 4, 4, 4, 5, 6
        };
        int ai1[] = {
            2, 3, 4, 5, 7, 10, 12, 18, 10, 20, 
            30, 20, 4, 7, 8, 20, 13, 0
        };
        double ad3[] = {
            0.02D, 0.059999999999999998D, 0.089999999999999997D, 0.125D, 0.26000000000000001D, 0.40000000000000002D, 0.59999999999999998D, 1.6000000000000001D, 1.7D, 2.3300000000000001D, 
            2.3999999999999999D, 3.3599999999999999D, 3.3999999999999999D, 4.7999999999999998D
        };
        double ad4[] = {
            0.025000000000000001D, 0.089999999999999997D, 0.14999999999999999D, 0.35999999999999999D, 0.5D, 0.90000000000000002D, 0.99999000000000005D
        };
        int ai2[] = {
            1, 1, 2, 13, 13, 13, 13, 13, 13, 13, 
            13, 16, 16, 16, 9, 1, 2, 2, 3, 3, 
            5, 5, 14, 14, 15, 15, 16, 16, 16, 9, 
            2, 2, 3, 3, 3, 5, 5, 15, 15, 15, 
            15, 16, 16, 16, 10, 2, 2, 3, 5, 5, 
            5, 5, 7, 7, 16, 16, 16, 16, 16, 10, 
            2, 3, 3, 5, 5, 6, 6, 8, 8, 17, 
            17, 17, 12, 12, 11, 2, 3, 5, 5, 5, 
            6, 6, 8, 8, 17, 17, 17, 12, 12, 12, 
            2, 3, 4, 4, 6, 6, 8, 8, 17, 17, 
            17, 17, 17, 12, 12, 2, 3, 4, 4, 6, 
            6, 18, 18, 18, 18, 17, 17, 17, 12, 12
        };
        double d2 = Math.abs(d);
        double d3 = Math.abs(d1);
        double d4 = d3 * d2;
        double d5;
        double d6;
        double d7;
        if(d3 <= 1.0D)
        {
            d6 = d2;
            d5 = d3;
            d7 = d4;
        } else
        {
            d6 = d4;
            d5 = 1.0D / d3;
            d7 = d2;
        }
        int i;
        for(i = 0; i < 14 && d6 > ad3[i]; i++);
        int j;
        for(j = 0; j < 7 && d5 > ad4[j]; j++);
        int k = ai2[15 * j + i] - 1;
        int l = ai1[k];
        double d8;
        switch(ai[k])
        {
        case 1: // '\001'
        default:
            double d9 = -0.5D * d6 * d6;
            double d13 = Math.exp(d9);
            double d20 = d5 * d5;
            int l2 = 1;
            int i3 = 1;
            double d30 = 0.15915494309189535D * d5;
            d8 = 0.15915494309189535D * Math.atan(d5);
            double d34 = d13 - 1.0D;
            double d37 = d9 * d13;
            do
            {
                d8 += (d34 * d30) / (double)i3;
                if(l2 >= l)
                    break;
                l2++;
                i3 += 2;
                d30 *= d20;
                d34 = d37 - d34;
                d37 = (d37 * d9) / (double)l2;
            } while(true);
            break;

        case 2: // '\002'
            int i1 = l + l + 1;
            int l1 = 1;
            d8 = 0.0D;
            double d14 = d6 * d6;
            double d21 = -d5 * d5;
            double d27 = 0.3989422804014327D * d5 * Math.exp(-0.5D * d7 * d7);
            double d31 = (0.5D * Sfun.erf(0.70710678118654757D * d7)) / d6;
            double d35 = 1.0D / d14;
            do
            {
                d8 += d31;
                if(l1 >= i1)
                    break;
                d31 = d35 * (d27 - (double)l1 * d31);
                d27 = d21 * d27;
                l1 += 2;
            } while(true);
            d8 *= 0.3989422804014327D * Math.exp(-0.5D * d14);
            break;

        case 3: // '\003'
            int j1 = 1;
            int i2 = 1;
            d8 = 0.0D;
            double d15 = d6 * d6;
            double d22 = d5 * d5;
            double d28 = 0.3989422804014327D * d5 * Math.exp(-0.5D * d7 * d7);
            double d32 = (0.5D * Sfun.erf(0.70710678118654757D * d7)) / d6;
            double d36 = 1.0D / d15;
            do
            {
                d8 += d32 * ad[j1 - 1];
                if(j1 > l)
                    break;
                d32 = d36 * ((double)i2 * d32 - d28);
                d28 = d22 * d28;
                j1++;
                i2 += 2;
            } while(true);
            d8 = d8 * 0.3989422804014327D * Math.exp(-0.5D * d15);
            break;

        case 4: // '\004'
            int k1 = l + l + 1;
            int j2 = 1;
            double d16 = d6 * d6;
            double d23 = -d5 * d5;
            d8 = 0.0D;
            double d29 = 0.15915494309189535D * d5 * Math.exp(-0.5D * d16 * (1.0D - d23));
            double d33 = 1.0D;
            do
            {
                d8 += d29 * d33;
                if(j2 >= k1)
                    break;
                j2 += 2;
                d33 = (1.0D - d16 * d33) / (double)j2;
                d29 *= d23;
            } while(true);
            break;

        case 5: // '\005'
            d8 = 0.0D;
            double d10 = d5 * d5;
            double d17 = -0.5D * d6 * d6;
            for(int k2 = 0; k2 < l; k2++)
            {
                double d26 = 1.0D + d10 * ad1[k2];
                d8 += (ad2[k2] * Math.exp(d17 * d26)) / d26;
            }

            d8 = d5 * d8;
            break;

        case 6: // '\006'
            double d11 = 0.5D * Sfun.erfc(0.70710678118654757D * d6);
            d8 = 0.5D * d11 * (1.0D - d11);
            double d18 = 1.0D - d5;
            double d24 = Math.atan(d18 / (1.0D + d5));
            if(d24 != 0.0D)
                d8 -= 0.15915494309189535D * d24 * Math.exp((-0.5D * d18 * d6 * d6) / d24);
            break;
        }
        double d12;
        if(d3 <= 1.0D)
            d12 = d8;
        else
        if(d2 <= 0.67000000000000004D)
        {
            d12 = 0.25D - 0.5D * Sfun.erf(0.70710678118654757D * d2) * 0.5D * Sfun.erf(0.70710678118654757D * d4) - d8;
        } else
        {
            double d19 = 0.5D * Sfun.erfc(0.70710678118654757D * d2);
            double d25 = 0.5D * Sfun.erfc(0.70710678118654757D * d4);
            d12 = 0.5D * (d19 + d25) - d19 * d25 - d8;
        }
        if(d1 < 0.0D)
            d12 = -d12;
        return d12;
    }

    public static double poisson(int i, double d)
    {
        double d12 = 1E-300D;
        double d13 = Math.log(d12);
        if(Double.isNaN(d) || d < 0.0D)
            return (0.0D / 0.0D);
        if(i < 0)
            return 0.0D;
        int i1 = i + 1;
        if(d <= 2.2204460492503E-016D)
            return 1.0D;
        double d8 = d;
        double d10 = 1.0D;
        int l = 1;
        double d3 = -d;
        int j = (int)(d3 / d13);
        d3 -= (double)j * d13;
        d3 = Math.exp(d3);
        double d9 = i;
        double d11 = d;
        double d1 = d9 * Math.log(d11);
        double d2 = i1;
        d2 = Sfun.logGamma(d2);
        double d6 = (-d11 + d1) - d2;
        int j1 = (int)(d6 / d13);
        d6 -= (double)j1 * d13;
        d6 = Math.exp(d6);
        d1 = 1.0D;
        d2 = 1.0D;
        if(j == 0)
            d1 = 1.0D - d3;
        if(j1 == 0)
            d2 = 1.0D - d6;
        double d7 = 0.0D;
        do
        {
            int k = j - j1;
            if(k > 0 || k == 0 && d3 <= d6)
            {
                if(j == 0)
                    d7 += d3;
                if(l == i1)
                    break;
                d3 *= d8 / d10;
                if(d3 >= d2)
                {
                    double d4 = d3 * d12;
                    if(d4 != 0.0D)
                    {
                        d3 = d4;
                        j--;
                    }
                }
                l++;
                d10++;
                continue;
            }
            if(j1 == 0)
                d7 += d6;
            if(l == i1)
                break;
            d6 *= d9 / d11;
            if(d6 >= d1)
            {
                double d5 = d6 * d12;
                if(d5 != 0.0D)
                {
                    d6 = d5;
                    j1--;
                }
            }
            i1--;
            d9--;
        } while(true);
        if(d7 > 1.0D)
            d7 = 1.0D;
        return d7;
    }

    public static double betaMean(double d, double d1)
    {
        return d / (d - d1);
    }

    public static double betaVariance(double d, double d1)
    {
        return (betaMean(d, d1) * d1) / ((d + d1) * (d + d1 + 1.0D));
    }

    public static double F(double d, double d1, double d2)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2) || d1 <= 0.0D || d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 0.0D;
        else
            return Sfun.betaIncomplete((d1 * d) / (d2 + d1 * d), 0.5D * d1, 0.5D * d2);
    }

    public static double complementaryF(double d, double d1, double d2)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2) || d1 <= 0.0D || d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 1.0D;
        else
            return Sfun.betaIncomplete(d2 / (d2 + d1 * d), 0.5D * d2, 0.5D * d1);
    }

    public static double complementaryF2(double d, double d1, double d2)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2) || d1 <= 0.0D || d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 1.0D;
        else
            return beta(d2 / (d2 + d1 * d), 0.5D * d2, 0.5D * d1);
    }

    public static double hypergeometric(int i, int j, int k, int l)
    {
        double d8 = 0.0D;
        if(j <= 0 || k <= 0)
            return (0.0D / 0.0D);
        if(l < j || l < k)
            return (0.0D / 0.0D);
        if(i < 0)
            return 0.0D;
        if(i > j)
            return 1.0D;
        if(i > k)
            return 1.0D;
        if(j - i > l - k)
            return 0.0D;
        double d9 = 1.0D;
        double d10 = l;
        int k1 = Math.min(j, k);
        double d11 = k1;
        int l1 = Math.max(j, k);
        double d12 = l1;
        double d13 = i * (l + 2);
        double d14 = (k + 1) * (j + 1);
        boolean flag = d13 > d14;
        int i1;
        int j1;
        double d;
        double d1;
        double d2;
        double d4;
        double d5;
        if(!flag)
        {
            d2 = d12 - d11;
            d5 = d10 - d12 - d11;
            int i2 = j + (k - l);
            if(i2 < 0)
                i2 = 0;
            double d3 = i2;
            d1 = d11 - d3;
            d4 = d3 + 1.0D;
            j1 = i - i2;
            d = (d10 - d12) + d3;
            i1 = k1 - i2;
        } else
        {
            d2 = d10 - d12 - d11;
            d5 = d12 - d11;
            d1 = d11;
            d4 = 1.0D;
            j1 = k1 - i;
            i1 = l - l1;
            d = d10 - d11;
            if(k1 < i1)
            {
                i1 = k1;
                d = d12;
            }
        }
        int j2 = 0;
        double d15 = 4.9406564584124654E-323D;
        if(i1 != 0)
        {
            for(int k2 = 0; k2 < i1; k2++)
            {
                double d6 = d / d10;
                if(d6 < d15 / d9)
                {
                    d9 /= d15;
                    j2++;
                }
                d9 *= d6;
                d--;
                d10--;
            }

        }
        d8 = 0.0D;
        if(j1 != 0)
        {
            for(int l2 = 0; l2 < j1; l2++)
            {
                if(j2 == 0)
                    d8 += d9;
                double d7 = (d1 * (d1 + d2)) / (d4 * (d4 + d5));
                d9 *= d7;
                if(d9 >= 1.0D)
                {
                    d9 *= d15;
                    j2--;
                }
                d1--;
                d4++;
            }

        }
        if(j2 != 0)
            d9 = 0.0D;
        if(!flag)
            d8 += d9;
        else
            d8 = 1.0D - d8;
        return d8;
    }

    public static double gamma(double d, double d1)
    {
        double ad[] = new double[6];
        double ad1[] = new double[4];
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 0.0D;
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(Math.log(d) >= Math.log(1.7976900000000001E+308D) / 3D)
            return d1 < 0.5D * d ? 1.0D : (0.0D / 0.0D);
        double d9 = Sfun.logGamma(d1);
        double d4 = d1 * Math.log(d);
        double d14 = d + d9;
        double d2 = Math.exp(d4 - d14);
        double d3 = 1E+030D;
        double d7 = 1E-008D;
        double d8;
        if(d > 1.0D && d >= d1)
        {
            double d13 = 1.0D - d1;
            double d16 = d + d13 + 1.0D;
            double d5 = 0.0D;
            ad[0] = 1.0D;
            ad[1] = d;
            ad[2] = d + 1.0D;
            ad[3] = d16 * d;
            d8 = ad[2] / ad[3];
            ad1[0] = ad[2];
            ad1[1] = ad[3];
            ad1[2] = ad[4];
            ad1[3] = ad[5];
            do
            {
                d5++;
                d13++;
                d16 += 2D;
                double d15 = d13 * d5;
                ad[4] = ad1[0] * d16 - ad[0] * d15;
                ad1[2] = ad[4];
                ad[5] = ad1[1] * d16 - ad[1] * d15;
                ad1[3] = ad[5];
                if(ad[5] == 0.0D)
                {
                    for(int i = 0; i < 4; i++)
                        ad[i] = ad1[i];

                    ad1[0] = ad[2];
                    ad1[1] = ad[3];
                    if(Math.abs(ad[4]) >= d3 || Math.abs(ad[5]) >= d3)
                    {
                        for(int j = 0; j < 4; j++)
                            ad[j] /= d3;

                        ad1[0] = ad[2];
                        ad1[1] = ad[3];
                    }
                    continue;
                }
                double d10 = ad[4] / ad[5];
                double d12 = Math.abs(d8 - d10);
                if(d12 <= d10 * d7 * 10D && d12 <= d7)
                {
                    d8 = 1.0D - d8 * d2;
                    break;
                }
                d8 = d10;
                for(int k = 0; k < 4; k++)
                    ad[k] = ad1[k];

                ad1[0] = ad[2];
                ad1[1] = ad[3];
                if(Math.abs(ad[4]) >= d3 || Math.abs(ad[5]) >= d3)
                {
                    for(int l = 0; l < 4; l++)
                        ad[l] /= d3;

                    ad1[0] = ad[2];
                    ad1[1] = ad[3];
                }
            } while(true);
        } else
        {
            double d11 = d1;
            double d6 = 1.0D;
            d8 = 1.0D;
            do
            {
                d11++;
                d6 *= d / d11;
                d8 += d6;
                if(d6 <= d7)
                    d8 = (d8 * d2) / d1;
            } while(d6 > d7);
        }
        return d8;
    }

    public static double exponential(double d, double d1)
    {
        double d2 = 1.0D;
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        else
            return gamma(d / d1, d2);
    }

    public static double normal(double d)
    {
        return 0.5D * Sfun.erfc(-d * 0.70710678118654757D);
    }

    public static double chiMean(double d)
    {
        if(d < 0.5D)
            return (0.0D / 0.0D);
        else
            return d;
    }

    public static double chiVariance(double d)
    {
        if(d <= 0.5D)
            return (0.0D / 0.0D);
        else
            return 2D * d;
    }

    public static double chi(double d, double d1)
    {
        if(Double.isNaN(d) || Double.isNaN(d1))
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 0.0D;
        double d5 = 2.2204460492503001E-015D;
        double d11 = 1E-300D;
        double d12 = Math.log(d11);
        double d13 = 343D;
        double d6;
        if(d1 > d13)
        {
            double d14 = 2D / (9D * d1);
            double d9 = (Math.pow(d / d1, 0.33333333333333331D) - (1.0D - d14)) / Math.sqrt(d14);
            d6 = normal(d9);
        } else
        {
            double d2 = 0.5D * d1;
            double d10 = 0.5D * d;
            double d15 = d2 * Math.log(d10) - d10;
            double d16 = Sfun.logGamma(d2);
            if(d10 < d2)
            {
                double d17 = d15 - Math.log(d2) - d16;
                double d7;
                if(d17 < d12)
                    d7 = 1.0D;
                else
                    d7 = Math.exp(d17);
                double d8 = d7;
                double d3 = 0.0D;
                int i = 1;
                do
                {
                    if(i > 50000)
                        break;
                    double d4 = i;
                    d8 *= d10 / (d2 + d4);
                    if(d8 <= d5 * d7)
                        break;
                    d7 += d8;
                    i++;
                } while(true);
                if(d17 < d12)
                    d6 = d7 * Math.exp(d17);
                else
                    d6 = d7;
            } else
            {
                double d18 = Math.exp(d15 - d16);
                double d19 = d11;
                double d20 = d19;
                double d21 = 0.0D;
                double d23 = 1.0D;
                int j = 1;
                int k = 1;
                do
                {
                    if(k > 0x186a0)
                        break;
                    double d24;
                    if(j == 1)
                    {
                        d24 = d10;
                        if(k > 1)
                            d23 = k / 2;
                    } else
                    {
                        d24 = 1.0D;
                        d23 = (double)(k / 2) - d2;
                    }
                    d21 = d24 + d23 * d21;
                    if(Math.abs(d21) < d11)
                        d21 = d11;
                    d20 = d24 + d23 / d20;
                    if(Math.abs(d20) < d11)
                        d20 = d11;
                    d21 = 1.0D / d21;
                    double d22 = d20 * d21;
                    d19 *= d22;
                    if(Math.abs(d22 - 1.0D) < d5)
                        break;
                    j = 1 - j;
                    k++;
                } while(true);
                d6 = 1.0D - d18 * d19;
            }
        }
        return d6;
    }

    public static double complementaryChi(double d, double d1)
    {
        if(Double.isNaN(d) || Double.isNaN(d1))
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 1.0D;
        double d5 = 2.2204460492503001E-015D;
        double d11 = 1E-300D;
        double d12 = Math.log(d11);
        double d13 = 343D;
        double d6;
        if(d1 > d13)
        {
            double d14 = 2D / (9D * d1);
            double d9 = (Math.pow(d / d1, 0.33333333333333331D) - (1.0D - d14)) / Math.sqrt(d14);
            d6 = 1.0D - normal(d9);
        } else
        {
            double d2 = 0.5D * d1;
            double d10 = 0.5D * d;
            double d15 = d2 * Math.log(d10) - d10;
            double d16 = Sfun.logGamma(d2);
            if(d10 < d2)
            {
                double d17 = d15 - Math.log(d2) - d16;
                double d7;
                if(d17 < d12)
                    d7 = 1.0D;
                else
                    d7 = Math.exp(d17);
                double d8 = d7;
                double d3 = 0.0D;
                int i = 1;
                do
                {
                    if(i > 50000)
                        break;
                    double d4 = i;
                    d8 *= d10 / (d2 + d4);
                    if(d8 <= d5 * d7)
                        break;
                    d7 += d8;
                    i++;
                } while(true);
                if(d17 < d12)
                    d6 = d7 * Math.exp(d17);
                else
                    d6 = d7;
                d6 = 1.0D - d6;
            } else
            {
                double d18 = Math.exp(d15 - d16);
                double d19 = d11;
                double d20 = d19;
                double d21 = 0.0D;
                double d23 = 1.0D;
                int j = 1;
                int k = 1;
                do
                {
                    if(k > 0x186a0)
                        break;
                    double d24;
                    if(j == 1)
                    {
                        d24 = d10;
                        if(k > 1)
                            d23 = k / 2;
                    } else
                    {
                        d24 = 1.0D;
                        d23 = (double)(k / 2) - d2;
                    }
                    d21 = d24 + d23 * d21;
                    if(Math.abs(d21) < d11)
                        d21 = d11;
                    d20 = d24 + d23 / d20;
                    if(Math.abs(d20) < d11)
                        d20 = d11;
                    d21 = 1.0D / d21;
                    double d22 = d20 * d21;
                    d19 *= d22;
                    if(Math.abs(d22 - 1.0D) < d5)
                        break;
                    j = 1 - j;
                    k++;
                } while(true);
                d6 = d18 * d19;
            }
        }
        return d6;
    }

    public static double noncentralBeta(double d, double d1, double d2, double d3)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2) || Double.isNaN(d3))
            return (0.0D / 0.0D);
        double d4 = 0.0D;
        double d5 = 0.0D;
        if(d2 <= 0.0D || d1 <= 0.0D || d3 < 0.0D)
            return (0.0D / 0.0D);
        double d7 = 1E-300D;
        double d8 = Math.log(d7);
        if(d < 0.0D)
            return 0.0D;
        if(d == 0.0D)
            return 0.0D;
        double d9 = 1.1102230246251999E-014D;
        if(d > 1.0D + d9)
            return 1.0D;
        if(d > 1.0D - d9)
        {
            return 1.0D;
        } else
        {
            double d10 = (d2 / d1) * (d / (1.0D - d));
            double d6 = noncentralF(d10, 2D * d1, 2D * d2, d3);
            return d6;
        }
    }

    public static double noncentralchi(double d, double d1, double d2)
    {
        int j1 = 0x186a0;
        double d25 = 1E-300D;
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2))
            return (0.0D / 0.0D);
        if(d1 <= 0.0D || d1 > 200000D)
            return (0.0D / 0.0D);
        if(d2 < 0.0D)
            return (0.0D / 0.0D);
        if(d1 + d2 > 200000D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
            return 0.0D;
        double d14 = 2.2204460492503001E-014D;
        char c = '\u03E8';
        double d10 = (0.0D / 0.0D);
        double d15;
        if(d2 <= 1E-010D)
        {
            d15 = chi(d, d1);
        } else
        {
            double d24 = d2 * 0.5D;
            int k = (int)d24;
            if(k == 0)
                k = 1;
            double d23 = k;
            double d9 = d * 0.5D;
            double d8 = 0.0D;
            double d16 = 0.0D;
            double d7 = 0.0D;
            for(int i = 0; i <= j1; i++)
            {
                int k1 = k + i;
                double d6 = Sfun.logGamma(k1 + 1);
                double d4 = (-d24 + (double)k1 * Math.log(d24)) - d6;
                d8 = Math.exp(d4);
                double d26 = d1 + 2D * (double)k1;
                d16 = chi(d, d26);
                double d11 = d26 * 0.5D;
                d6 = Sfun.logGamma(1.0D + d11);
                double d3 = d11 * Math.log(d9) - d9 - d6;
                d7 = Math.exp(d3);
                if(d7 > d25)
                {
                    k = k1;
                    break;
                }
                k1 = k - i;
                if(k1 < 0)
                    continue;
                d6 = Sfun.logGamma(k1 + 1);
                d4 = (-d24 + (double)k1 * Math.log(d24)) - d6;
                d8 = Math.exp(d4);
                d26 = d1 + 2D * (double)k1;
                d16 = chi(d, d26);
                d11 = d26 * 0.5D;
                d6 = Sfun.logGamma(1.0D + d11);
                d3 = d11 * Math.log(d9) - d9 - d6;
                d7 = Math.exp(d3);
                if(d7 <= d25)
                    continue;
                k = k1;
                break;
            }

            double d19 = d8 * d16;
            int l = 0;
            double d20 = 0.0D;
            double d5 = d7;
            double d22 = d8;
            int j = k;
            double d12 = (d1 + 2D * (double)j) * 0.5D;
            d5 = (d5 * d12) / d9;
            d20 += d5;
            double d17 = d16 + d20;
            d22 *= (double)j / d24;
            double d21 = d22 * d17;
            d19 += d21;
            j--;
            for(l++; l <= c && d21 >= d14 * d19 && j != 0; l++)
            {
                d12 = (d1 + 2D * (double)j) * 0.5D;
                d5 = (d5 * d12) / d9;
                d20 += d5;
                d17 = d16 + d20;
                d22 *= (double)j / d24;
                d21 = d22 * d17;
                d19 += d21;
                j--;
            }

            int i1 = 0;
            d20 = d7;
            d5 = d7;
            d22 = d8;
            j = k;
            d22 *= d24 / (double)(j + 1);
            d17 = d16 - d20;
            d21 = d22 * d17;
            d19 += d21;
            j++;
            d12 = (d1 + 2D * (double)j) * 0.5D;
            d5 = (d5 * d9) / d12;
            d20 += d5;
            for(i1++; i1 <= c && d21 >= d14 * d19; i1++)
            {
                d22 *= d24 / (double)(j + 1);
                double d18 = d16 - d20;
                d21 = d22 * d18;
                d19 += d21;
                j++;
                double d13 = (d1 + 2D * (double)j) * 0.5D;
                d5 = (d5 * d9) / d13;
                d20 += d5;
            }

            d15 = d19;
            if(l > c || i1 > c)
                System.out.println(new Cdf()+"com.imsl.stat"+ "noncentralchi.NoConvergence");
        }
        if(d15 > 1.0D)
            d15 = 1.0D;
        d10 = d15;
        return d10;
    }

    public static double studentsT(double d, double d1)
    {
        if(Double.isNaN(d) || d1 < 1.0D)
            return (0.0D / 0.0D);
        double d3 = d * d;
        double d7;
        if(d1 > d3)
        {
            d3 = d;
            double d11 = d1;
            int i = (int)d11;
            d3 *= d3;
            double d5 = d3 / d11;
            double d12 = 1.0D + d5;
            if(d11 != (double)i || d11 >= 20D || d11 > 200D)
            {
                double d4 = d12 - 1.0D;
                if(d4 != 0.0D)
                    d5 *= Math.log(d12) / d4;
                double d8 = d11 - 0.5D;
                d12 = 48D * d8 * d8;
                d5 *= d8;
                d5 = (((((-0.40000000000000002D * d5 - 3.2999999999999998D) * d5 - 24D) * d5 - 85.5D) / (0.80000000000000004D * (d5 * d5) + 100D + d12) + d5 + 3D) / d12 + 1.0D) * Math.sqrt(d5);
                d7 = Sfun.erfc(d5 * Math.sqrt(0.5D));
            } else
            if(d11 < 20D && d3 < 4D)
            {
                d5 = Math.sqrt(d5);
                double d9 = d5;
                if(d11 == 1.0D)
                    d9 = 0.0D;
                do
                {
                    d11 -= 2D;
                    if(d11 > 1.0D)
                        d9 = ((d11 - 1.0D) / (d12 * d11)) * d9 + d5;
                } while(d11 > 1.0D);
                if(d11 == 0.0D)
                    d9 /= Math.sqrt(d12);
                if(d11 != 0.0D)
                    d9 = (Math.atan(d5) + d9 / d12) * 0.63661977236758005D;
                d7 = 1.0D - d9;
            } else
            {
                double d10 = 1.0D;
                double d6 = d11;
                double d13 = 0.0D;
                for(double d14 = 0.0D; d10 != d14; d10 += d6 / (d11 + d13))
                {
                    d13 += 2D;
                    d14 = d10;
                    d6 *= (d13 - 1.0D) / (d12 * d13);
                }

                for(; d11 > 1.0D; d11 -= 2D)
                    d10 *= (d11 - 1.0D) / (d12 * d11);

                if(d11 != 0.0D)
                    d10 *= 0.63661977236758005D / Math.sqrt(d12);
                d7 = d10;
            }
        } else
        {
            d7 = beta(d1 / (d1 + d3), 0.5D * d1, 0.5D);
        }
        double d2;
        if(d > 0.0D)
            d2 = 1.0D - 0.5D * d7;
        else
            d2 = 0.5D * d7;
        return d2;
    }

    public static double complementaryStudentsT(double d, double d1)
    {
        if(Double.isNaN(d) || d1 < 1.0D)
            return (0.0D / 0.0D);
        double d3 = d * d;
        double d7;
        if(d1 > d3)
        {
            d3 = d;
            double d11 = d1;
            int i = (int)d11;
            d3 *= d3;
            double d5 = d3 / d11;
            double d12 = 1.0D + d5;
            if(d11 != (double)i || d11 >= 20D || d11 > 200D)
            {
                double d4 = d12 - 1.0D;
                if(d4 != 0.0D)
                    d5 *= Math.log(d12) / d4;
                double d8 = d11 - 0.5D;
                d12 = 48D * d8 * d8;
                d5 *= d8;
                d5 = (((((-0.40000000000000002D * d5 - 3.2999999999999998D) * d5 - 24D) * d5 - 85.5D) / (0.80000000000000004D * (d5 * d5) + 100D + d12) + d5 + 3D) / d12 + 1.0D) * Math.sqrt(d5);
                d7 = Sfun.erfc(d5 * Math.sqrt(0.5D));
            } else
            if(d11 < 20D && d3 < 4D)
            {
                d5 = Math.sqrt(d5);
                double d9 = d5;
                if(d11 == 1.0D)
                    d9 = 0.0D;
                do
                {
                    d11 -= 2D;
                    if(d11 > 1.0D)
                        d9 = ((d11 - 1.0D) / (d12 * d11)) * d9 + d5;
                } while(d11 > 1.0D);
                if(d11 == 0.0D)
                    d9 /= Math.sqrt(d12);
                if(d11 != 0.0D)
                    d9 = (Math.atan(d5) + d9 / d12) * 0.63661977236758005D;
                d7 = 1.0D - d9;
            } else
            {
                double d10 = 1.0D;
                double d6 = d11;
                double d13 = 0.0D;
                for(double d14 = 0.0D; d10 != d14; d10 += d6 / (d11 + d13))
                {
                    d13 += 2D;
                    d14 = d10;
                    d6 *= (d13 - 1.0D) / (d12 * d13);
                }

                for(; d11 > 1.0D; d11 -= 2D)
                    d10 *= (d11 - 1.0D) / (d12 * d11);

                if(d11 != 0.0D)
                    d10 *= 0.63661977236758005D / Math.sqrt(d12);
                d7 = d10;
            }
        } else
        {
            d7 = beta(d1 / (d1 + d3), 0.5D * d1, 0.5D);
        }
        double d2;
        if(d <= 0.0D)
            d2 = 1.0D - 0.5D * d7;
        else
            d2 = 0.5D * d7;
        return d2;
    }

    public static double noncentralstudentsT(double d, int i, double d1)
    {
        if(Double.isNaN(d) || i < 1)
            return (0.0D / 0.0D);
        double d22 = 3.1415926535897931D;
        double d25 = 2.5066282746310002D;
        double d24 = 0.3989422804014327D;
        double d23 = 0.15915494309189535D;
        double d21 = (0.0D / 0.0D);
        double d9 = Math.sqrt(-1.8999999999999999D * Math.log(2.2250738585072014E-308D));
        double d7 = i;
        int j = i - 2 * (i / 2);
        double d2 = d / Math.sqrt(d7);
        double d4 = d7 / (d7 + d * d);
        double d19 = Math.sqrt(d4);
        double d5 = d1 * d2;
        double d8 = d1 * d19;
        double d6 = d2 * d8;
        double d15 = normal(d6);
        double d11 = 0.0D;
        if(Math.abs(d8) < d9)
        {
            d11 = d2 * d19 * Math.exp(-0.5D * d8 * d8) * d15 * d24;
        } else
        {
            System.out.println(new Cdf()+"com.imsl.stat noncentralstudentsT.UnderflowPossible");
            return d21;
        }
        double d10 = d4 * d5 * d11;
        if(Math.abs(d1) < d9)
            d10 += d4 * d2 * d23 * Math.exp(-0.5D * d1 * d1);
        double d20 = 0.0D;
        d10 = d4 * d5 * d11;
        if(Math.abs(d1) < d9)
            d10 += d4 * d2 * d23 * Math.exp(-0.5D * d1 * d1);
        d20 = 0.0D;
        double d14;
        if(i != 1)
        {
            if(j <= 0)
                d20 = d11;
            else
                d20 = d10;
            if(i >= 4)
            {
                int k = i - 2;
                double d3 = 1.0D;
                double d13 = 2D;
                for(int l = 2; l <= k; l += 2)
                {
                    double d12 = d13 - 1.0D;
                    d11 = (d4 * (d5 * d3 * d10 + d11) * d12) / d13;
                    d3 = 1.0D / (d3 * d12);
                    d10 = (d4 * (d5 * d3 * d11 + d10) * d13) / (d13 + 1.0D);
                    if(j <= 0)
                        d20 += d11;
                    else
                        d20 += d10;
                    d3 = 1.0D / (d3 * d13);
                    d13 += 2D;
                }

            }
            if(j <= 0)
            {
                double d16 = normal(-d1);
                d14 = d16 + d20 * d25;
            } else
            {
                double d17 = normal(-d8);
                d14 = OwenT(d8, d2);
                d14 = d17 + 2D * (d14 + d20);
            }
        } else
        {
            double d18 = normal(-d8);
            d14 = OwenT(d8, d2);
            d14 = d18 + 2D * (d14 + d20);
        }
        d21 = d14;
        return d21;
    }

    public static double Weibull(double d, double d1, double d2)
    {
        if(d < 0.0D)
            return (0.0D / 0.0D);
        else
            return 1.0D - Math.exp(-Math.pow(d / d2, d1));
    }

    public static double logNormal(double d, double d1, double d2)
    {
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d < 0.0D)
            return (0.0D / 0.0D);
        else
            return normal((1.0D / d2) * (Math.log(d) - d1));
    }

    public static double extremeValue(double d, double d1, double d2)
    {
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        else
            return 1.0D - Math.exp(-Math.exp((d - d1) / d2));
    }

    public static double Rayleigh(double d, double d1)
    {
        if(d < 0.0D)
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return 0.0D;
        else
            return Weibull(d, 2D, Math.sqrt(2D) * d1);
    }

    public static double uniform(double d, double d1, double d2)
    {
        if(d1 >= d2)
            return (0.0D / 0.0D);
        if(d < d1)
            return 0.0D;
        if(d >= d2)
            return 1.0D;
        if(d >= d1 && d <= d2)
            return (d - d1) / (d2 - d1);
        else
            return (0.0D / 0.0D);
    }

    public static double discreteUniform(int i, int j)
    {
        if((double)i <= 0.0D)
            return (0.0D / 0.0D);
        if((double)j <= 0.0D)
            return (0.0D / 0.0D);
        if(i >= j)
            return 1.0D;
        else
            return (double)i / (double)j;
    }

    public static double geometric(int i, double d)
    {
        if(d <= 0.0D || d >= 1.0D)
            return (0.0D / 0.0D);
        if((double)i < 0.0D)
            return (0.0D / 0.0D);
        else
            return 1.0D - Math.pow(1.0D - d, i + 1);
    }

    public static double logistic(double d, double d1, double d2)
    {
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(Double.isNaN(d2) || Double.isNaN(d1) || Double.isNaN(d))
        {
            return (0.0D / 0.0D);
        } else
        {
            double d3 = -(d - d1) / d2;
            return 1.0D / (1.0D + Math.exp(d3));
        }
    }

    public static double Pareto(double d, double d1, double d2)
    {
        if(Double.isNaN(d2) || Double.isNaN(d1) || Double.isNaN(d))
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d < d1)
            return (0.0D / 0.0D);
        if(d2 == (1.0D / 0.0D))
        {
            return 1.0D;
        } else
        {
            double d3 = Math.pow(d1 / d, d2);
            return 1.0D - d3;
        }
    }

    /**
     * @deprecated Method empiricalCdf is deprecated
     */

    public static void empiricalCdf(int i, double ad[], double ad1[])
    {
        double ad2[] = new double[i];
        int ai[] = new int[i];
        for(int j = 0; j < i; j++)
        {
            ad2[j] = ad[j];
            ai[j] = j;
        }

        Sort.ascending(ad2, ai);
        double d = i + 1;
        ad1[ai[i - 1]] = (double)i / d;
        for(int k = i - 2; k >= 0; k--)
            if(ad2[k] == ad2[k + 1])
                ad1[ai[k]] = ad1[ai[k + 1]];
            else
                ad1[ai[k]] = (double)(k + 1) / d;

    }

    /**
     * @deprecated Method binomialProb is deprecated
     */

    public static double binomialProb(int i, int j, double d)
    {
        if(j <= 0 || d > 1.0D || d < 0.0D || i < 0 || i > j || Double.isNaN(d))
            return (0.0D / 0.0D);
        if(d == 1.0D || d == 0.0D)
            return 0.0D;
        else
            return Math.exp(logCombination(j, i) + (double)i * Math.log(d) + (double)(j - i) * Math.log(1.0D - d));
    }

    /**
     * @deprecated Method poissonProb is deprecated
     */

    public static double poissonProb(int i, double d)
    {
        return Math.exp((-d + (double)i * Math.log(d)) - Sfun.logGamma(i + 1));
    }

    /**
     * @deprecated Method betaProb is deprecated
     */

    public static double betaProb(double d, double d1, double d2)
    {
        double d3 = d1;
        double d4 = d2;
        if(d3 <= 0.0D || d4 <= 0.0D || d <= 0.0D || d >= 1.0D)
            return (0.0D / 0.0D);
        else
            return (1.0D / Sfun.beta(d3, d4)) * Math.pow(1.0D - d, d4 - 1.0D) * Math.pow(d, d3 - 1.0D);
    }

    /**
     * @deprecated Method FProb is deprecated
     */

    public static double FProb(double d, double d1, double d2)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2) || d1 <= 0.0D || d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D)
        {
            return 0.0D;
        } else
        {
            double d3 = 0.5D * d1;
            double d4 = 0.5D * d2;
            double d5 = d3 / d4;
            double d6 = Sfun.beta(d3, d4);
            double d7 = d5 * d;
            return (1.0D / (d * d6)) * Math.pow(d7 / (1.0D + d7), d3) * Math.pow(1.0D + d7, -d4);
        }
    }

    /**
     * @deprecated Method hypergeometricProb is deprecated
     */

    public static double hypergeometricProb(int i, int j, int k, int l)
    {
        if(j < 0 || j > l || k < 0 || k > l || l < 0)
            return (0.0D / 0.0D);
        if(i < 0)
            return 0.0D;
        int i1 = (j - l) + k;
        if(i1 < 0)
            i1 = 0;
        if(i < i1)
        {
            return (0.0D / 0.0D);
        } else
        {
            double d = (logCombination(k, i) + logCombination(l - k, j - i)) - logCombination(l, j);
            return Math.exp(d);
        }
    }

    /**
     * @deprecated Method gammaProb is deprecated
     */

    public static double gammaProb(double d, double d1, double d2)
    {
        if(d1 <= 0.0D || d2 <= 0.0D || d <= 0.0D)
        {
            return (0.0D / 0.0D);
        } else
        {
            double d3 = 1.0D / (Math.pow(d2, d1) * Sfun.gamma(d1));
            double d4 = d / d2;
            return d3 * Math.pow(d, d1 - 1.0D) * Math.exp(-d4);
        }
    }

    /**
     * @deprecated Method exponentialProb is deprecated
     */

    public static double exponentialProb(double d, double d1)
    {
        if(Double.isNaN(d1) || Double.isNaN(d))
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        else
            return gammaProb(d, 1.0D, d1);
    }

    /**
     * @deprecated Method chiProb is deprecated
     */

    public static double chiProb(double d, double d1)
    {
        if(Double.isNaN(d) || Double.isNaN(d1))
            return (0.0D / 0.0D);
        if(d1 <= 0.0D || d <= 0.0D)
            return (0.0D / 0.0D);
        else
            return gammaProb(d, d1 / 2D, 2D);
    }

    /**
     * @deprecated Method WeibullProb is deprecated
     */

    public static double WeibullProb(double d, double d1, double d2)
    {
        double d3 = d2;
        double d4 = d1;
        if(d < 0.0D)
            return (0.0D / 0.0D);
        if(d4 <= 0.0D || d3 <= 0.0D)
        {
            return (0.0D / 0.0D);
        } else
        {
            double d5 = d / d3;
            return (d4 / d3) * Math.pow(d5, d4 - 1.0D) * Math.exp(-Math.pow(d5, d4));
        }
    }

    /**
     * @deprecated Method logNormalProb is deprecated
     */

    public static double logNormalProb(double d, double d1, double d2)
    {
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d < 0.0D)
            return (0.0D / 0.0D);
        double d3 = 1.0D / (Math.sqrt(2D) * d2);
        double d4 = 1.0D / Math.sqrt(6.2831853071795862D);
        if(d == 0.0D)
            return 0.0D;
        else
            return (d4 / (d * d2)) * Math.exp(-Math.pow((Math.log(d) - d1) * d3, 2D));
    }

    /**
     * @deprecated Method extremeValueProb is deprecated
     */

    public static double extremeValueProb(double d, double d1, double d2)
    {
        if(Double.isNaN(d1) || Double.isNaN(d2) || Double.isNaN(d))
            return (0.0D / 0.0D);
        if(d2 <= 0.0D)
        {
            return (0.0D / 0.0D);
        } else
        {
            double d3 = (d - d1) / d2;
            double d4 = Math.exp(d3);
            return (d4 * Math.exp(-d4)) / d2;
        }
    }

    /**
     * @deprecated Method RayleighProb is deprecated
     */

    public static double RayleighProb(double d, double d1)
    {
        if(d < 0.0D)
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return 0.0D;
        else
            return WeibullProb(d, 2D, Math.sqrt(2D) * d1);
    }

    /**
     * @deprecated Method discreteUniformProb is deprecated
     */

    public static double discreteUniformProb(int i, int j)
    {
        if((double)i <= 0.0D)
            return (0.0D / 0.0D);
        if((double)j <= 0.0D)
            return (0.0D / 0.0D);
        if(i > j)
            return 0.0D;
        else
            return 1.0D / (double)j;
    }

    /**
     * @deprecated Method geometricProb is deprecated
     */

    public static double geometricProb(int i, double d)
    {
        if(d <= 0.0D || d >= 1.0D)
            return (0.0D / 0.0D);
        if((double)i < 0.0D)
            return (0.0D / 0.0D);
        else
            return d * Math.exp((double)i * Sfun.dlnrel(-d));
    }

    /**
     * @deprecated Method inverseBeta is deprecated
     */

    public static double inverseBeta(double d, double d1, double d2)
    {
        double d14 = (0.0D / 0.0D);
        if(d1 <= 0.0D || d2 <= 0.0D || d <= 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d == 1.0D)
            return 1.0D;
        double d8 = d;
        double d3 = beta(d8, d1, d2) - d;
        double d9;
        if(d3 == 0.0D)
        {
            d9 = d8;
        } else
        {
            d9 = d + 0.050000000000000003D;
            double d11 = 0.050000000000000003D;
            double d4;
            if(d9 <= 0.0D)
                d4 = -d;
            else
            if(d9 >= 1.0D)
                d4 = 1.0D - d;
            else
                d4 = beta(d9, d1, d2) - d;
            double d7 = Math.max(0.01D, (d4 - d3) / d11);
            double d15 = -d3 / d7;
            int i = 1;
            do
            {
                if(i >= 102)
                    break;
                d15 = 2D * d15;
                if(i <= 100)
                {
                    d9 = d8 + d15;
                    if(d9 <= 0.0D)
                        d4 = -d;
                    else
                    if(d9 >= 1.0D)
                        d4 = 1.0D - d;
                    else
                        d4 = beta(d9, d1, d2) - d;
                    if(d3 * d4 < 0.0D)
                        break;
                    d8 = d9;
                } else
                {
                    d9 = 1.0D;
                    break;
                }
                i++;
            } while(true);
            boolean flag = false;
            for(int j = 1; j <= 100; j++)
            {
                double d13 = (d8 + d9) * 0.5D;
                double d6 = d4 - d3;
                double d12 = d9 - d8;
                if(d13 != 0.0D ? Math.abs(d12) < Math.abs(d13 * 2.2204460492503E-016D) : Math.abs(d12) < 2.2204460492503E-016D)
                    break;
                double d10;
                if(flag)
                    d10 = d13;
                else
                    d10 = d9 - (d4 * d12) / d6;
                flag = false;
                double d5;
                if(d10 <= 0.0D)
                    d5 = -d;
                else
                if(d10 >= 1.0D)
                    d5 = 1.0D - d;
                else
                    d5 = beta(d10, d1, d2) - d;
                if(d5 * d4 <= 0.0D)
                {
                    d8 = d9;
                    d3 = d4;
                    d9 = d10;
                    d4 = d5;
                    continue;
                }
                d9 = d10;
                d4 = d5;
                d3 *= 0.5D;
                if(Math.abs(d4) > Math.abs(d3))
                {
                    d3 = 2D * d3;
                    flag = true;
                }
            }

        }
        d14 = 0.5D * (d8 + d9);
        return d14;
    }

    /**
     * @deprecated Method inverseF is deprecated
     */

    public static double inverseF(double d, double d1, double d2)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2))
            return (0.0D / 0.0D);
        if(d == 1.0D)
            return 0.0D;
        if(d == 0.0D)
            return (1.0D / 0.0D);
        double d3;
        if(d < 0.0D || d > 1.0D || d1 <= 0.0D || d2 <= 0.0D)
        {
            d3 = (0.0D / 0.0D);
        } else
        {
            double d4 = 0.99999999999999978D;
            double d5 = 0.5D * d1;
            double d6 = 0.5D * d2;
            if(d <= 0.5D)
            {
                d3 = inverseBeta(d, d5, d6);
                if(d3 >= d4)
                    d3 = (1.0D / 0.0D);
                else
                    d3 = (d2 * d3) / (d1 * (1.0D - d3));
            } else
            {
                double d7 = 1.0D - d;
                d3 = inverseBeta(d7, d6, d5);
                if(d3 == 0.0D)
                    d3 = (1.0D / 0.0D);
                else
                    d3 = ((1.0D / d3 - 1.0D) * d2) / d1;
            }
        }
        return d3;
    }

    /**
     * @deprecated Method inverseGamma is deprecated
     */

    public static double inverseGamma(double d, double d1)
    {
        double d19 = 1.7976931348623157E+308D / Math.log(1.7976931348623157E+308D);
        if(d < 0.0D || d > 1.0D || d1 <= 0.0D || d1 > d19)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            d = 1.1102230246251999E-016D;
        if(d == 1.0D)
            return (1.0D / 0.0D);
        double d9 = inverseNormal(d);
        double d2 = 1.0D / (9D * d1);
        double d3 = d9 / Math.sqrt(9D * d1);
        double d4 = Math.pow(1.7976931348623157E+308D, 0.33333333333333331D);
        double d20 = Math.abs((1.0D - d2) + d3);
        if(d20 > d4)
            return (0.0D / 0.0D);
        double d18;
        if(9D * d1 <= Math.pow(d9, 2D))
            d18 = d1 * Math.pow((1.0D - 1.0D / (9D * d1)) + d9 / Math.sqrt(9D * d1), 3D);
        else
            d18 = 0.0D;
        double d13 = d18;
        if(d13 <= 0.0D)
            d9 = -d;
        else
            d9 = gamma(d13, d1) - d;
        double d14;
        if(d9 == 0.0D)
        {
            d14 = d13;
        } else
        {
            double d16;
            if(Math.abs(d18) >= 1.0D)
            {
                d14 = d18 * 1.05D;
                d16 = d14 - d13;
            } else
            {
                d14 = d18 + 0.050000000000000003D;
                d16 = 0.050000000000000003D;
            }
            double d10;
            if(d14 <= 0.0D)
                d10 = -d;
            else
                d10 = gamma(d14, d1) - d;
            double d12 = Math.max(0.01D, (d10 - d9) / d16);
            double d8 = -d9 / d12;
            int i = 0;
            do
            {
                if(i >= 102)
                    break;
                d8 = 2D * d8;
                d14 = d13 + d8;
                double d7;
                if(d14 > 0.0D)
                {
                    double d5 = Math.log(d14);
                    double d6 = Math.log(1.7976931348623157E+308D) / 3D;
                    d7 = d5 - d6;
                } else
                {
                    d7 = -1D;
                }
                if(i > 100 || d7 >= 0.0D && d1 >= 0.5D * d14)
                    return (0.0D / 0.0D);
                if(d14 <= 0.0D)
                    d10 = -d;
                else
                    d10 = gamma(d14, d1) - d;
                if(d9 * d10 < 0.0D)
                    break;
                d13 = d14;
                i++;
            } while(true);
            boolean flag = false;
            for(int j = 1; j < 102; j++)
            {
                double d21 = (d13 + d14) * 0.5D;
                double d22 = d10 - d9;
                double d17 = d14 - d13;
                if(d21 != 0.0D ? Math.abs(d17) < Math.abs(d21 * 2.2204460492503001E-015D) : Math.abs(d17) < 2.2204460492503001E-015D)
                    break;
                double d15;
                if(flag)
                    d15 = d21;
                else
                    d15 = d14 - (d10 * d17) / d22;
                flag = false;
                double d11;
                if(d15 <= 0.0D)
                    d11 = -d;
                else
                    d11 = gamma(d15, d1) - d;
                if(d11 * d10 <= 0.0D)
                {
                    d13 = d14;
                    d9 = d10;
                    d14 = d15;
                    d10 = d11;
                    continue;
                }
                d14 = d15;
                d10 = d11;
                d9 *= 0.5D;
                if(Math.abs(d10) > Math.abs(d9))
                {
                    d9 = 2D * d9;
                    flag = true;
                }
            }

        }
        return (d13 + d14) * 0.5D;
    }

    /**
     * @deprecated Method inverseNormal is deprecated
     */

    public static double inverseNormal(double d)
    {
        if(d < 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return (-1.0D / 0.0D);
        if(d == 1.0D)
            return (1.0D / 0.0D);
        else
            return -1.4142135623730951D * Sfun.erfcInverse(2D * d);
    }

    /**
     * @deprecated Method inverseChi is deprecated
     */

    public static double inverseChi(double d, double d1)
    {
        if(Double.isNaN(d) || Double.isNaN(d1))
            return (0.0D / 0.0D);
        if(d <= 0.0D || d > 1.0D || d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d == 1.0D)
            d -= 1.1102230246251999E-016D;
        double d14 = normal(d);
        double d8 = 2D / (9D * d1);
        double d9 = d1 * Math.pow((1.0D - d8) + d14 * Math.sqrt(d8), 3D);
        if(d9 < 0.0D)
            d9 = 0.0D;
        double d3 = chi(d9, d1) - d;
        double d10;
        if(d3 == 0.0D)
        {
            d10 = d9;
        } else
        {
            double d12;
            if(Math.abs(d14) >= 1.0D)
            {
                d10 = d14 * 1.05D;
                d12 = d10 - d9;
            } else
            {
                d10 = d14 + 0.050000000000000003D;
                d12 = 0.050000000000000003D;
            }
            if(d10 < 0.0D)
                d10 = 0.0D;
            double d4 = chi(d10, d1) - d;
            double d7 = Math.max(0.01D, (d4 - d3) / d12);
            double d2 = -d3 / d7;
            int i = 0;
            do
            {
                if(i > 100)
                    break;
                d2 *= 2D;
                if(i > 100)
                    return (0.0D / 0.0D);
                d10 = d9 + d2;
                if(d10 < 0.0D)
                    d10 = 0.0D;
                d4 = chi(d10, d1) - d;
                if((d3 < 0.0D || d4 < 0.0D) && (d3 > 0.0D || d4 > 0.0D))
                    break;
                d9 = d10;
                i++;
            } while(true);
            boolean flag = false;
            for(int j = 1; j <= 0x186a0; j++)
            {
                double d15 = (d9 + d10) / 2D;
                double d6 = d4 - d3;
                double d13 = d10 - d9;
                if(Math.abs(d15) == 0.0D)
                {
                    if(Math.abs(d13) < 2.2204460492503001E-015D)
                        return (d9 + d10) / 2D;
                } else
                if(Math.abs(d13) < d15 * 10D * 2.2204460492503E-016D)
                    return (d9 + d10) / 2D;
                double d11;
                if(flag)
                    d11 = d15;
                else
                    d11 = d10 - (d4 * d13) / d6;
                flag = false;
                if(d11 < 0.0D)
                    d11 = 0.0D;
                double d5 = chi(d11, d1) - d;
                if(d5 >= 0.0D && d4 <= 0.0D || d5 <= 0.0D && d4 >= 0.0D)
                {
                    d9 = d10;
                    d3 = d4;
                    d10 = d11;
                    d4 = d5;
                    continue;
                }
                d10 = d11;
                d4 = d5;
                d3 /= 2D;
                if(Math.abs(d4) > Math.abs(d3))
                {
                    d3 = 2D * d3;
                    flag = true;
                }
            }

        }
        return (d9 + d10) / 2D;
    }

    /**
     * @deprecated Method inverseNoncentralchi is deprecated
     */

    public static double inverseNoncentralchi(double d, double d1, double d2)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2))
            return (0.0D / 0.0D);
        if(d <= 0.0D || d >= 1.0D || d1 < 0.5D || d1 > 200000D)
            return (0.0D / 0.0D);
        if(d2 < 0.0D || d2 + d1 > 200000D)
            return (0.0D / 0.0D);
        double d5 = 2.2204460492503001E-015D;
        double d3 = (0.0D / 0.0D);
        double d17 = chi(d, d1) + d2;
        double d11 = d17;
        double d6 = noncentralchi(d11, d1, d2) - d;
        if(d6 == 0.0D)
        {
            double d12 = d11;
            d3 = (d11 + d12) * 0.5D;
            return d3;
        }
        double d13;
        double d15;
        if(Math.abs(d17) >= 1.0D)
        {
            d13 = d17 * 1.05D;
            d15 = d13 - d11;
        } else
        {
            d13 = d17 + 0.050000000000000003D;
            d15 = 0.050000000000000003D;
        }
        double d7;
        if(d13 <= 0.0D)
            d7 = -d;
        else
            d7 = noncentralchi(d13, d1, d2) - d;
        double d10 = Math.max(0.01D, (d7 - d6) / d15);
        double d4 = -d6 / d10;
        int i = 0;
        d4 = 2D * d4;
        i++;
        d13 = d11 + d4;
        if(d13 <= 0.0D)
            d7 = -d;
        else
            d7 = noncentralchi(d13, d1, d2) - d;
        if(d6 * d7 >= 0.0D)
            d11 = d13;
        do
        {
            if(d6 * d7 < 0.0D)
                break;
            d4 = 2D * d4;
            i++;
            if(i > 100)
            {
                Object aobj[] = {
                    new Double(d), new Double(d1), new Double(d2)
                };
                System.out.println(new Cdf()+ "com.imsl.stat"+ "inverseNoncentralchi.UnableToBracket"+aobj);
                return (0.0D / 0.0D);
            }
            d13 = d11 + d4;
            if(d13 <= 0.0D)
                d7 = -d;
            else
                d7 = noncentralchi(d13, d1, d2) - d;
            if(d6 * d7 >= 0.0D)
                d11 = d13;
        } while(true);
        boolean flag = false;
        for(int j = 1; j <= 100; j++)
        {
            double d18 = (d11 + d13) * 0.5D;
            double d9 = d7 - d6;
            double d16 = d13 - d11;
            if(d18 != 0.0D)
            {
                if(Math.abs(d16) < Math.abs(d18 * d5))
                {
                    d3 = (d11 + d13) * 0.5D;
                    return d3;
                }
            } else
            if(Math.abs(d16) < d5)
            {
                d3 = (d11 + d13) * 0.5D;
                return d3;
            }
            double d14;
            if(flag)
                d14 = d18;
            else
                d14 = d13 - (d7 * d16) / d9;
            flag = false;
            double d8;
            if(d14 <= 0.0D)
                d8 = -d;
            else
                d8 = noncentralchi(d14, d1, d2) - d;
            if(d8 * d7 <= 0.0D)
            {
                d11 = d13;
                d6 = d7;
                d13 = d14;
                d7 = d8;
                continue;
            }
            d13 = d14;
            d7 = d8;
            d6 *= 0.5D;
            if(Math.abs(d7) > Math.abs(d6))
            {
                d6 = 2D * d6;
                flag = true;
            }
        }

        System.out.println(new Cdf()+ "com.imsl.stat"+ "inverseNoncentralchi.ConvergenceAssumed"+ null);
        d3 = (d11 + d13) * 0.5D;
        return d3;
    }

    /**
     * @deprecated Method inverseStudentsT is deprecated
     */

    public static double inverseStudentsT(double d, double d1)
    {
        if(Double.isNaN(d) || Double.isNaN(d1) || d < 0.0D || d > 1.0D || d1 < 1.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return (-1.0D / 0.0D);
        if(Math.abs(d - 0.5D) < 2.2204460492503E-016D)
            return 0.0D;
        double d10;
        if(d < 0.5D)
            d10 = 2D * d;
        else
            d10 = 2D * (1.0D - d);
        double d11;
        if(Math.abs(d1 - 1.0D) <= 2.2204460492503E-016D)
        {
            double d2 = d10 * 0.5D * 3.1415926535897931D;
            d11 = Math.cos(d2) / Math.sin(d2);
        } else
        if(Math.abs(d1 - 2D) < 2.2204460492503E-016D)
            d11 = Math.sqrt(2D / (d10 * (2D - d10)) - 2D);
        else
        if(d1 <= 2D)
        {
            double d8 = 1.0D;
            double d7 = d1;
            double d13 = 1.0D - 2D * d;
            if(d > 0.5D)
                d13 = -d13;
            d11 = inverseBeta(1.0D - d13, 0.5D * d7, 0.5D * d8);
            if(d11 == 0.0D)
                return (1.0D / 0.0D);
            double d9 = ((1.0D / d11 - 1.0D) * d7) / d8;
            d11 = Math.sqrt(d9);
        } else
        {
            double d3 = 1.0D / (d1 - 0.5D);
            double d4 = 48D / (d3 * d3);
            double d5 = (((20700D * d3) / d4 - 98D) * d3 - 16D) * d3 + 96.359999999999999D;
            double d6 = ((94.5D / (d4 + d5) - 3D) / d4 + 1.0D) * Math.sqrt(d3 * 0.5D * 3.1415926535897931D) * d1;
            double d14 = d6 * d10;
            double d12 = Math.pow(d14, 2D / d1);
            if(d12 <= d3 + 0.050000000000000003D)
            {
                d12 = (((1.0D / (((d1 + 6D) / (d1 * d12) - 0.088999999999999996D * d6 - 0.82199999999999995D) * (d1 + 2D) * 3D) + 0.5D / (d1 + 4D)) * d12 - 1.0D) * (d1 + 1.0D)) / (d1 + 2D) + 1.0D / d12;
            } else
            {
                d11 = 0.5D * d10;
                double d15 = inverseNormal(d11);
                d12 = d15 * d15;
                if(d1 < 5D)
                    d5 += 0.29999999999999999D * (d1 - 4.5D) * (d15 + 0.59999999999999998D);
                d5 += (((0.050000000000000003D * d6 * d15 - 5D) * d15 - 7D) * d15 - 2D) * d15 + d4;
                d12 = (((((0.40000000000000002D * d12 + 6.2999999999999998D) * d12 + 36D) * d12 + 94.5D) / d5 - d12 - 3D) / d4 + 1.0D) * d15;
                d12 *= d3 * d12;
                d6 = d12;
                if(d6 <= 0.002D)
                    d12 = 0.5D * d12 * d12 + d12;
                else
                    d12 = Math.exp(d12) - 1.0D;
            }
            d11 = Math.sqrt(d1 * d12);
        }
        if(d < 0.5D)
            d11 = -d11;
        return d11;
    }

    /**
     * @deprecated Method inverseNoncentralstudentsT is deprecated
     */

    public static double inverseNoncentralstudentsT(double d, int i, double d1)
    {
        if(Double.isNaN(d) || Double.isNaN(d1))
            return (0.0D / 0.0D);
        if(d <= 0.0D || d >= 1.0D)
            return (0.0D / 0.0D);
        if(i <= 0)
            return (0.0D / 0.0D);
        double d9 = (0.0D / 0.0D);
        double d3 = 2.2204460492503E-016D;
        double d16 = inverseNormal(d) + d1;
        double d10 = d16;
        double d4 = noncentralstudentsT(d10, i, d1) - d;
        if(d4 == 0.0D)
        {
            double d11 = d10;
            d9 = (d10 + d11) * 0.5D;
            return d9;
        }
        double d12;
        double d14;
        if(Math.abs(d16) >= 1.0D)
        {
            d12 = d16 * 1.05D;
            d14 = d12 - d10;
        } else
        {
            d12 = d16 + 0.050000000000000003D;
            d14 = 0.050000000000000003D;
        }
        double d5 = noncentralstudentsT(d12, i, d1) - d;
        double d8 = Math.max(0.01D, (d5 - d4) / d14);
        double d2 = -d4 / d8;
        int j = 0;
        d2 = 2D * d2;
        j++;
        d12 = d10 + d2;
        d5 = noncentralstudentsT(d12, i, d1) - d;
        if(d4 * d5 >= 0.0D)
            d10 = d12;
        do
        {
            if(d4 * d5 < 0.0D)
                break;
            d2 = 2D * d2;
            j++;
            if(j > 100)
            {
                Object aobj[] = {
                    new Double(d), new Integer(i), new Double(d1)
                };
                System.out.println(new Cdf()+ "com.imsl.stat"+"inverseNoncentralstudentsT.UnableToBracket"+aobj);
                return (0.0D / 0.0D);
            }
            d12 = d10 + d2;
            d5 = noncentralstudentsT(d12, i, d1) - d;
            if(d4 * d5 >= 0.0D)
                d10 = d12;
        } while(true);
        boolean flag = false;
        for(int k = 1; k <= 100; k++)
        {
            double d17 = (d10 + d12) * 0.5D;
            double d7 = d5 - d4;
            double d15 = d12 - d10;
            if(d17 != 0.0D)
            {
                if(Math.abs(d15) < Math.abs(d17 * d3))
                {
                    d9 = (d10 + d12) * 0.5D;
                    return d9;
                }
            } else
            if(Math.abs(d15) < d3)
            {
                d9 = (d10 + d12) * 0.5D;
                return d9;
            }
            double d13;
            if(flag)
                d13 = d17;
            else
                d13 = d12 - (d5 * d15) / d7;
            flag = false;
            double d6 = noncentralstudentsT(d13, i, d1) - d;
            if(d6 * d5 <= 0.0D)
            {
                d10 = d12;
                d4 = d5;
                d12 = d13;
                d5 = d6;
                continue;
            }
            d12 = d13;
            d5 = d6;
            d4 *= 0.5D;
            if(Math.abs(d5) > Math.abs(d4))
            {
                d4 = 2D * d4;
                flag = true;
            }
        }

        System.out.println(new Cdf()+ "com.imsl.stat"+ "inverseNoncentralstudentsT.ConvergenceAssumed"+ null);
        d9 = (d10 + d12) * 0.5D;
        return d9;
    }

    /**
     * @deprecated Method inverseWeibull is deprecated
     */

    public static double inverseWeibull(double d, double d1, double d2)
    {
        double d3 = 0.0D;
        if(d < 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return 0.0D;
        if(d == 1.0D)
            return 1.7976931348623157E+308D;
        else
            return Math.pow(-Sfun.dlnrel(-d), 1.0D / d1) * d2;
    }

    /**
     * @deprecated Method inverseLogNormal is deprecated
     */

    public static double inverseLogNormal(double d, double d1, double d2)
    {
        double d3 = 0.0D;
        if(d < 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return 0.0D;
        if(d == 1.0D)
            return 1.7976931348623157E+308D;
        else
            return Math.exp(inverseNormal(d) * d2 + d1);
    }

    /**
     * @deprecated Method inverseExponential is deprecated
     */

    public static double inverseExponential(double d, double d1)
    {
        double d2 = 1.0D;
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        else
            return inverseGamma(d, d2) * d1;
    }

    /**
     * @deprecated Method inverseExtremeValue is deprecated
     */

    public static double inverseExtremeValue(double d, double d1, double d2)
    {
        double d3 = 0.0D;
        if(d < 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d2 <= 0.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return 4.9406564584124654E-324D;
        if(d == 1.0D)
            return 1.7976931348623157E+308D;
        else
            return Math.log(-Sfun.dlnrel(-d)) * d2 + d1;
    }

    /**
     * @deprecated Method inverseRayleigh is deprecated
     */

    public static double inverseRayleigh(double d, double d1)
    {
        double d2 = 0.0D;
        if(d1 <= 0.0D)
            return (0.0D / 0.0D);
        if(d < 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return 0.0D;
        if(d == 1.0D)
        {
            return 1.7976931348623157E+308D;
        } else
        {
            double d3 = inverseWeibull(d, 2D, Math.sqrt(2D) * d1);
            return d3;
        }
    }

    /**
     * @deprecated Method inverseUniform is deprecated
     */

    public static double inverseUniform(double d, double d1, double d2)
    {
        if(d1 >= d2)
            return (0.0D / 0.0D);
        if(d < 0.0D || d > 1.0D)
            return (0.0D / 0.0D);
        if(d == 0.0D)
            return d1;
        if(d == 1.0D)
            return d2;
        else
            return (d2 - d1) * d + d1;
    }

    /**
     * @deprecated Method inverseDiscreteUniform is deprecated
     */

    public static int inverseDiscreteUniform(double d, int i)
    {
        if((double)i <= 0.0D)
            return -1;
        if(d <= 0.0D || d >= 1.0D)
            return -1;
        else
            return (int)(-Math.floor(-d * (double)i));
    }

    /**
     * @deprecated Method inverseGeometric is deprecated
     */

    public static double inverseGeometric(double d, double d1)
    {
        if(d1 <= 0.0D || d1 >= 1.0D)
            return (0.0D / 0.0D);
        if(d <= 0.0D || d >= 1.0D)
            return (0.0D / 0.0D);
        else
            return Math.max(Math.floor(Sfun.dlnrel(-d) / Sfun.dlnrel(-d1)), 0.0D);
    }

    public static double noncentralF(double d, double d1, double d2, double d3)
    {
        double d4 = d3;
        if(Double.isNaN(d) || Double.isNaN(d1) || Double.isNaN(d2) || Double.isNaN(d4))
            return (0.0D / 0.0D);
        if(d2 <= 0.0D || d1 <= 0.0D || d4 < 0.0D)
            return (0.0D / 0.0D);
        if(d < 0.0D)
            return 0.0D;
        if(d == 0.0D)
            return 0.0D;
        double d5 = 1E-300D;
        double d6 = Math.log(d5);
        double d7 = d1 / 2D;
        double d8 = d2 / 2D;
        double d9 = (1.1102230246251999E-016D * d2) / d1;
        double d12 = 0.0D;
        if(d > d9)
        {
            if(d4 == 0.0D)
            {
                double d13 = (d1 * d) / d2;
                double d10 = d13 / (1.0D + d13);
                d12 = beta(d10, d7, d8);
                return d12;
            }
            double d14 = 1.1102230246251999E-016D;
            double d18 = 0.5D * d4;
            double d19 = Math.log(d18);
            double d20 = (d1 * d) / d2;
            double d11 = d20 / (1.0D + d20);
            double d21 = Math.log(d11);
            double d22 = Math.log(1.0D - d11);
            double d23 = Math.floor(d18);
            int i = 0x186a0;
            if(d23 + (double)i > 4503599627370523D)
                return (0.0D / 0.0D);
            double d24 = d23;
            double d27 = d7;
            double d30 = 0.0D;
            double d32 = 0.0D;
            double d35 = 0.0D;
            double d38 = 0.0D;
            for(int j = 0; j <= i; j++)
            {
                d24 = d23 + (double)j;
                d27 = d7 + d24;
                double d31 = (-d18 + d24 * d19) - Sfun.logGamma(d24 + 1.0D);
                d32 = Math.exp(d31);
                d35 = beta(d11, d27, d8);
                d38 = d32 * d35;
                if(d38 > d5)
                {
                    d23 = d24;
                    break;
                }
                d24 = d23 - (double)j;
                if(d24 < 0.0D)
                    continue;
                d27 = d7 + d24;
                d31 = (-d18 + d24 * d19) - Sfun.logGamma(d24 + 1.0D);
                d32 = Math.exp(d31);
                d35 = beta(d11, d27, d8);
                d38 = d32 * d35;
                if(d38 <= d5)
                    continue;
                d23 = d24;
                break;
            }

            double d39 = d24 + 1.0D;
            double d40 = d24 - 1.0D;
            double d41 = Math.exp((d27 * d21 + d8 * d22 + Sfun.logGamma(d27 + d8)) - Sfun.logGamma(d27 + 1.0D) - Sfun.logGamma(d8));
            double d44 = d35 - d41;
            double d45 = (d32 * d18) / d39;
            double d46 = d45 * d44;
            double d47 = (d41 * d11 * (d27 + d8)) / (d27 + 1.0D);
            double d48 = 0.0D;
            double d49 = 0.0D;
            double d50 = 0.0D;
            double d51 = 0.0D;
            if(d40 >= 0.0D)
            {
                d48 = (d41 * d27) / (d11 * ((d27 + d8) - 1.0D));
                d49 = d35 + d48;
                d50 = (d32 * d24) / d18;
                d51 = d50 * d49;
            }
            for(int k = 1; k <= i; k++)
            {
                if(d38 < d5)
                    return d12;
                d12 += d38;
                double d16 = d38 / d12;
                if(d16 <= d14)
                    return d12;
                if(d40 < 0.0D || d46 > d51)
                {
                    double d25 = d39;
                    d39 = d25 + 1.0D;
                    d38 = d46;
                    double d33 = d45;
                    double d36 = d44;
                    double d42 = d47;
                    d45 = (d33 * d18) / d39;
                    d44 = d36 - d42;
                    d46 = d45 * d44;
                    double d28 = d7 + d25;
                    d47 = (d42 * d11 * (d28 + d8)) / (d28 + 1.0D);
                } else
                {
                    double d26 = d40;
                    d40 = d26 - 1.0D;
                    d38 = d51;
                    double d34 = d50;
                    double d37 = d49;
                    double d43 = d48;
                    double d29 = d7 + d26;
                    d48 = (d43 * d29) / (d11 * ((d29 + d8) - 1.0D));
                    d49 = d37 + d48;
                    d50 = (d26 * d34) / d18;
                    d51 = d50 * d49;
                }
            }

        } else
        {
            d12 = 0.0D;
            double d15 = (-d4 / 2D + Math.log(d1 / d2) * d7 + Sfun.logGamma(d7 + d8)) - Sfun.logGamma(d7) - Sfun.logGamma(d8);
            double d17 = (d15 - Math.log(d7)) + Math.log(d) * d7;
            if(d17 >= d6)
                d12 = Math.exp(d17);
        }
        return d12;
    }

    public static double beta(double d, double d1, double d2)
    {
        double d10 = d1;
        double d11 = d2;
        double d12 = 0.94999999999999996D * Math.log(2.2250738585072014E-308D);
        double d13 = -36.736800569677101D;
        double d14 = 2.2250738585072014E-306D;
        double d15 = -703.79124834627601D;
        double d3 = (0.0D / 0.0D);
        if(d10 <= 0.0D || d11 <= 0.0D)
            return d3;
        if(d <= 0.0D)
            return 0.0D;
        if(d >= 1.0D)
            return 1.0D;
        double d9 = d;
        double d16 = 0.001D;
        boolean flag = false;
        do
        {
            if(flag)
            {
                d9 = 1.0D - d9;
                double d17 = d10;
                d10 = d11;
                d11 = d17;
            }
            if(((d10 + d11) * d9) / (d10 + 1.0D) < 1.1102230246251999E-016D)
            {
                d3 = 0.0D;
                double d7 = d10 * Math.log(Math.max(d9, d14)) - Math.log(d10) - Sfun.logBeta(d10, d11);
                if(d7 > d15 && d9 != 0.0D)
                    d3 = Math.exp(d7);
                if(d9 != d || flag)
                    d3 = 1.0D - d3;
                return d3;
            }
            double d18 = d11 - (double)(int)d11;
            if(d18 == 0.0D)
                d18 = 1.0D;
            double d8 = d10 * Math.log(d9) - Sfun.logBeta(d18, d10) - Math.log(d10);
            d3 = 0.0D;
            if(d8 >= d15)
            {
                d3 = Math.exp(d8);
                double d5 = d3 * d10;
                if(d18 != 1.0D)
                {
                    double d19 = d13 / Math.log(d9);
                    if(d19 > 100000D)
                    {
                        flag = true;
                        continue;
                    }
                    int j = (int)Math.max(d19, 4D);
                    for(int l = 1; l <= j; l++)
                    {
                        d5 = (d5 * ((double)l - d18) * d9) / (double)l;
                        d3 += d5 / (d10 + (double)l);
                    }

                }
            }
            if(d11 <= 1.0D)
            {
                if(d9 != d || flag)
                    d3 = 1.0D - d3;
                else
                if(1.0D - d3 < d16)
                {
                    flag = true;
                    continue;
                }
                return Math.max(Math.min(d3, 1.0D), 0.0D);
            }
            d8 = (d10 * Math.log(d9) + d11 * Math.log(1.0D - d9)) - Sfun.logBeta(d10, d11) - Math.log(d11);
            int i = (int)Math.max(d8 / d15, 0.0D);
            double d6 = Math.exp(d8 - (double)i * d15);
            double d4 = 1.0D / (1.0D - d9);
            double d20 = (d11 * d4) / ((d10 + d11) - 1.0D);
            double d21 = 0.0D;
            int k = (int)d11;
            if(d11 == (double)k)
                k--;
            for(int i1 = 0; i1 < k && (d20 > 1.0D || d6 / 1.1102230246251999E-016D > d21); i1++)
            {
                d6 = ((d11 - (double)i1) * d4 * d6) / ((d10 + d11) - (double)(i1 + 1));
                if(d6 > 1.0D)
                {
                    i--;
                    d6 *= d14;
                }
                if(i == 0)
                    d21 += d6;
            }

            d3 += d21;
            if(d9 != d || flag)
            {
                d3 = 1.0D - d3;
                break;
            }
            if(1.0D - d3 >= d16)
                break;
            flag = true;
        } while(true);
        d3 = Math.max(Math.min(d3, 1.0D), 0.0D);
        return d3;
    }


}