package utility.forecasting;

import java.io.Serializable;

// Referenced classes of package
//            IEEE, Hyperbolic

public class Complex extends Number
    implements Serializable, Cloneable
{
 public static final Complex i = new Complex(0.0D, 1.0D);
    public double re;
    public double im;
    public static String suffix = "i";
    private static final long negZeroBits = 0x8000000000000000L;

    public Complex(Complex complex)
    {
        re = complex.re;
        im = complex.im;
    }

    public Complex(double d, double d1)
    {
        re = d;
        im = d1;
    }

    public Complex(double d)
    {
        re = d;
        im = 0.0D;
    }

    public Complex()
    {
        re = 0.0D;
        im = 0.0D;
    }

    private boolean isNaN()
    {
        return Double.isNaN(re) || Double.isNaN(im);
    }

    public boolean equals(Complex complex)
    {
        if(isNaN() && complex.isNaN())
            return true;
        else
            return re == complex.re && im == complex.im;
    }

    public boolean equals(Object obj)
    {
        if(obj == null)
            return false;
        if(obj instanceof Complex)
            return equals((Complex)obj);
        else
            return false;
    }

    public int hashCode()
    {
        long l = Double.doubleToLongBits(re);
        long l1 = Double.doubleToLongBits(im);
        return (int)(l ^ l1 ^ (l ^ l1) >> 32);
    }

    public int compareTo(Object obj)
    {
        return compareTo((Complex)obj);
    }

    public int compareTo(Complex complex)
    {
        int j = (new Double(re)).compareTo(new Double(complex.re));
        if(j == 0)
            j = (new Double(im)).compareTo(new Double(complex.im));
        return j;
    }

    public byte byteValue()
    {
        return (byte)(int)re;
    }

    public double doubleValue()
    {
        return re;
    }

    public float floatValue()
    {
        return (float)re;
    }

    public int intValue()
    {
        return (int)re;
    }

    public long longValue()
    {
        return (long)re;
    }

    public short shortValue()
    {
        return (short)(int)re;
    }

    public double real()
    {
        return re;
    }

    public double imag()
    {
        return im;
    }

    public static double real(Complex complex)
    {
        return complex.re;
    }

    public static double imag(Complex complex)
    {
        return complex.im;
    }

    public static Complex negate(Complex complex)
    {
        return new Complex(-complex.re, -complex.im);
    }

    public static Complex conjugate(Complex complex)
    {
        return new Complex(complex.re, -complex.im);
    }

    public static Complex add(Complex complex, Complex complex1)
    {
        return new Complex(complex.re + complex1.re, complex.im + complex1.im);
    }

    public static Complex add(Complex complex, double d)
    {
        return new Complex(complex.re + d, complex.im);
    }

    public static Complex add(double d, Complex complex)
    {
        return new Complex(d + complex.re, complex.im);
    }

    public static Complex subtract(Complex complex, Complex complex1)
    {
        return new Complex(complex.re - complex1.re, complex.im - complex1.im);
    }

    public static Complex subtract(Complex complex, double d)
    {
        return new Complex(complex.re - d, complex.im);
    }

    public static Complex subtract(double d, Complex complex)
    {
        return new Complex(d - complex.re, -complex.im);
    }

    public static Complex multiply(Complex complex, Complex complex1)
    {
        Complex complex2 = new Complex(complex.re * complex1.re - complex.im * complex1.im, complex.re * complex1.im + complex.im * complex1.re);
        if(Double.isNaN(complex2.re) && Double.isNaN(complex2.im))
            multiplyNaN(complex, complex1, complex2);
        return complex2;
    }

    private static void multiplyNaN(Complex complex, Complex complex1, Complex complex2)
    {
        boolean flag = false;
        double d = complex.re;
        double d1 = complex.im;
        double d2 = complex1.re;
        double d3 = complex1.im;
        if(Double.isInfinite(d) || Double.isInfinite(d1))
        {
            d = IEEE.copysign(Double.isInfinite(d) ? 1.0D : 0.0D, d);
            d1 = IEEE.copysign(Double.isInfinite(d1) ? 1.0D : 0.0D, d1);
            if(Double.isNaN(d2))
                d2 = IEEE.copysign(0.0D, d2);
            if(Double.isNaN(d3))
                d3 = IEEE.copysign(0.0D, d3);
            flag = true;
        }
        if(Double.isInfinite(d2) || Double.isInfinite(d3))
        {
            d = IEEE.copysign(Double.isInfinite(d2) ? 1.0D : 0.0D, d2);
            d1 = IEEE.copysign(Double.isInfinite(d3) ? 1.0D : 0.0D, d3);
            if(Double.isNaN(d))
                d = IEEE.copysign(0.0D, d);
            if(Double.isNaN(d1))
                d1 = IEEE.copysign(0.0D, d1);
            flag = true;
        }
        if(!flag && (Double.isInfinite(d * d2) || Double.isInfinite(d1 * d3) || Double.isInfinite(d * d3) || Double.isInfinite(d1 * d2)))
        {
            if(Double.isNaN(d))
                d = IEEE.copysign(0.0D, d);
            if(Double.isNaN(d1))
                d1 = IEEE.copysign(0.0D, d1);
            if(Double.isNaN(d2))
                d2 = IEEE.copysign(0.0D, d2);
            if(Double.isNaN(d3))
                d3 = IEEE.copysign(0.0D, d3);
            flag = true;
        }
        if(flag)
        {
            complex2.re = (1.0D / 0.0D) * (d * d2 - d1 * d3);
            complex2.im = (1.0D / 0.0D) * (d * d3 + d1 * d2);
        }
    }

    public static Complex multiply(Complex complex, double d)
    {
        return new Complex(complex.re * d, complex.im * d);
    }

    public static Complex multiply(double d, Complex complex)
    {
        return new Complex(d * complex.re, d * complex.im);
    }

    public static Complex multiplyImag(Complex complex, double d)
    {
        return new Complex(-complex.im * d, complex.re * d);
    }

    public static Complex multiplyImag(double d, Complex complex)
    {
        return new Complex(-d * complex.im, d * complex.re);
    }

    private static boolean isFinite(double d)
    {
        return !Double.isInfinite(d) && !Double.isNaN(d);
    }

    public static Complex divide(Complex complex, Complex complex1)
    {
        double d = complex.re;
        double d1 = complex.im;
        double d2 = complex1.re;
        double d3 = complex1.im;
        double d4 = Math.max(Math.abs(d2), Math.abs(d3));
        boolean flag = isFinite(d4);
        if(flag)
        {
            d2 /= d4;
            d3 /= d4;
        }
        double d5 = d2 * d2 + d3 * d3;
        Complex complex2 = new Complex((d * d2 + d1 * d3) / d5, (d1 * d2 - d * d3) / d5);
        if(flag)
        {
            complex2.re /= d4;
            complex2.im /= d4;
        }
        if(Double.isNaN(complex2.re) && Double.isNaN(complex2.im))
            if(d5 == 0.0D && (!Double.isNaN(d) || !Double.isNaN(d1)))
            {
                double d6 = IEEE.copysign((1.0D / 0.0D), d2);
                complex2.re = d6 * d;
                complex2.im = d6 * d1;
            } else
            if((Double.isInfinite(d) || Double.isInfinite(d1)) && isFinite(d2) && isFinite(d3))
            {
                d = IEEE.copysign(Double.isInfinite(d) ? 1.0D : 0.0D, d);
                d1 = IEEE.copysign(Double.isInfinite(d1) ? 1.0D : 0.0D, d1);
                complex2.re = (1.0D / 0.0D) * (d * d2 + d1 * d3);
                complex2.im = (1.0D / 0.0D) * (d1 * d2 - d * d3);
            } else
            if(Double.isInfinite(d4) && isFinite(d) && isFinite(d1))
            {
                d2 = IEEE.copysign(Double.isInfinite(d2) ? 1.0D : 0.0D, d2);
                d3 = IEEE.copysign(Double.isInfinite(d3) ? 1.0D : 0.0D, d3);
                complex2.re = 0.0D * (d * d2 + d1 * d3);
                complex2.im = 0.0D * (d1 * d2 - d * d3);
            }
        return complex2;
    }

    public static Complex divide(Complex complex, double d)
    {
        return new Complex(complex.re / d, complex.im / d);
    }

    public static Complex divide(double d, Complex complex)
    {
        Complex complex1;
        if(Math.abs(complex.re) > Math.abs(complex.im))
        {
            double d3 = complex.im / complex.re;
            double d1 = complex.re + complex.im * d3;
            complex1 = new Complex(d / d1, (-d * d3) / d1);
        } else
        {
            double d4 = complex.re / complex.im;
            double d2 = complex.im + complex.re * d4;
            complex1 = new Complex((d * d4) / d2, -d / d2);
        }
        return complex1;
    }

    public static double abs(Complex complex)
    {
        return IEEE.hypot(complex.re, complex.im);
    }

    public static double argument(Complex complex)
    {
        return StrictMath.atan2(complex.im, complex.re);
    }

    public static Complex sqrt(Complex complex)
    {
        Complex complex1 = new Complex();
        if(Double.isInfinite(complex.im))
        {
            complex1.re = (1.0D / 0.0D);
            complex1.im = complex.im;
        } else
        if(Double.isNaN(complex.re))
            complex1.re = complex1.im = (0.0D / 0.0D);
        else
        if(Double.isNaN(complex.im))
        {
            if(Double.isInfinite(complex.re))
            {
                if(complex.re > 0.0D)
                {
                    complex1.re = complex.re;
                    complex1.im = complex.im;
                } else
                {
                    complex1.re = complex.im;
                    complex1.im = (1.0D / 0.0D);
                }
            } else
            {
                complex1.re = complex1.im = (0.0D / 0.0D);
            }
        } else
        {
            double d = abs(complex);
            if(Math.abs(complex.re) <= Math.abs(complex.im))
            {
                complex1.re = Math.sqrt(0.5D * (d + complex.re));
                complex1.im = Math.sqrt(0.5D * (d - complex.re));
            } else
            if(complex.re > 0.0D)
            {
                complex1.re = d + complex.re;
                complex1.im = Math.abs(complex.im) * Math.sqrt(0.5D / complex1.re);
                complex1.re = Math.sqrt(0.5D * complex1.re);
            } else
            {
                complex1.im = d - complex.re;
                complex1.re = Math.abs(complex.im) * Math.sqrt(0.5D / complex1.im);
                complex1.im = Math.sqrt(0.5D * complex1.im);
            }
            if(complex.im < 0.0D)
                complex1.im = -complex1.im;
        }
        return complex1;
    }

    public static Complex exp(Complex complex)
    {
        Complex complex1 = new Complex();
        double d = StrictMath.exp(complex.re);
        double d1 = StrictMath.cos(complex.im);
        double d2 = StrictMath.sin(complex.im);
        if(Double.isInfinite(complex.re) || Double.isInfinite(d))
            if(complex.re < 0.0D)
            {
                d = 0.0D;
                if(Double.isInfinite(complex.im) || Double.isNaN(complex.im))
                {
                    d1 = d2 = 0.0D;
                } else
                {
                    d1 /= (1.0D / 0.0D);
                    d2 /= (1.0D / 0.0D);
                }
            } else
            {
                d = complex.re;
                if(Double.isNaN(complex.im))
                    d1 = 1.0D;
            }
        if(complex.im == 0.0D)
        {
            complex1.re = d;
            complex1.im = complex.im;
        } else
        {
            complex1.re = d * d1;
            complex1.im = d * d2;
        }
        return complex1;
    }

    public static Complex log(Complex complex)
    {
        Complex complex1 = new Complex();
        if(Double.isNaN(complex.re))
        {
            complex1.re = complex1.im = complex.re;
            if(Double.isInfinite(complex.im))
                complex1.re = (1.0D / 0.0D);
        } else
        if(Double.isNaN(complex.im))
        {
            complex1.re = complex1.im = complex.im;
            if(Double.isInfinite(complex.re))
                complex1.re = (1.0D / 0.0D);
        } else
        {
            complex1.re = Math.log(abs(complex));
            complex1.im = argument(complex);
        }
        return complex1;
    }

    public static Complex sin(Complex complex)
    {
        Complex complex1 = new Complex(-complex.im, complex.re);
        Complex complex2 = sinh(complex1);
        double d = complex2.im;
        complex2.im = -complex2.re;
        complex2.re = d;
        return complex2;
    }

    public static Complex cos(Complex complex)
    {
        return cosh(new Complex(-complex.im, complex.re));
    }

    public static Complex tan(Complex complex)
    {
        Complex complex1 = new Complex(-complex.im, complex.re);
        Complex complex2 = tanh(complex1);
        double d = complex2.im;
        complex2.im = -complex2.re;
        complex2.re = d;
        return complex2;
    }

    public static Complex asin(Complex complex)
    {
        Complex complex1 = new Complex();
        double d = abs(complex);
        if(Double.isInfinite(d))
        {
            boolean flag = Double.isInfinite(complex.re);
            boolean flag1 = Double.isInfinite(complex.im);
            if(flag)
            {
                double d1 = 1.5707963267948966D;
                complex1.re = complex.re <= 0.0D ? -d1 : d1;
                if(flag1)
                    complex1.re /= 2D;
            } else
            if(flag1)
                complex1.re = complex.re / (1.0D / 0.0D);
            if(Double.isNaN(complex.im))
            {
                complex1.im = -complex.re;
                complex1.re = complex.im;
            } else
            {
                complex1.im = complex.im * (1.0D / 0.0D);
            }
            return complex1;
        }
        if(Double.isNaN(d))
        {
            complex1.re = complex1.im = (0.0D / 0.0D);
            if(complex.re == 0.0D)
                complex1.re = complex.re;
        } else
        if(d < 2.58095E-008D)
        {
            complex1.re = complex.re;
            complex1.im = complex.im;
        } else
        if(complex.re == 0.0D)
        {
            complex1.re = 0.0D;
            complex1.im = Hyperbolic.asinh(complex.im);
        } else
        if(d <= 0.10000000000000001D)
        {
            Complex complex2 = multiply(complex, complex);
            for(int j = 1; j <= 8; j++)
            {
                double d2 = 2 * (8 - j) + 1;
                complex1 = multiply(multiply(complex1, complex2), d2 / (d2 + 1.0D));
                complex1.re += 1.0D / d2;
            }

            complex1 = multiply(complex1, complex);
        } else
        {
            Complex complex3 = complex.im >= 0.0D ? complex : negate(complex);
            Complex complex4 = sqrt(add(complex3, 1.0D));
            if(complex4.im < 0.0D)
                complex4 = negate(complex4);
            Complex complex5 = sqrt(subtract(complex3, 1.0D));
            complex1 = log(add(complex3, multiply(complex4, complex5)));
            double d3 = complex1.re;
            complex1.re = 1.5707963267948966D + complex1.im;
            complex1.im = -d3;
        }
        if(complex1.re > 1.5707963267948966D)
        {
            complex1.re = 3.1415926535897931D - complex1.re;
            complex1.im = -complex1.im;
        }
        if(complex1.re < -1.5707963267948966D)
        {
            complex1.re = -3.1415926535897931D - complex1.re;
            complex1.im = -complex1.im;
        }
        if(complex.im < 0.0D)
        {
            complex1.re = -complex1.re;
            complex1.im = -complex1.im;
        }
        return complex1;
    }

    public static Complex acos(Complex complex)
    {
        Complex complex1 = new Complex();
        double d = abs(complex);
        if(Double.isInfinite(complex.re) && Double.isNaN(complex.im))
        {
            complex1.re = (0.0D / 0.0D);
            complex1.im = (-1.0D / 0.0D);
        } else
        if(Double.isInfinite(d))
        {
            complex1.re = Math.atan2(Math.abs(complex.im), complex.re);
            complex1.im = complex.im * (-1.0D / 0.0D);
        } else
        if(d == 0.0D)
        {
            complex1.re = 1.5707963267948966D;
            complex1.im = -complex.im;
        } else
        {
            complex1 = subtract(1.5707963267948966D, asin(complex));
        }
        return complex1;
    }

    public static Complex atan(Complex complex)
    {
        Complex complex1 = new Complex();
        double d = abs(complex);
        if(Double.isInfinite(d))
        {
            double d1 = 1.5707963267948966D;
            double d3 = Double.isNaN(complex.im) ? 0.0D : complex.im;
            complex1.re = complex.re >= 0.0D ? d1 : -d1;
            complex1.im = (double)(d3 >= 0.0D ? 1 : -1) / (1.0D / 0.0D);
            if(Double.isNaN(complex.re))
                complex1.re = complex.re;
        } else
        if(Double.isNaN(d))
        {
            complex1.re = complex1.im = (0.0D / 0.0D);
            if(complex.im == 0.0D)
                complex1.im = complex.im;
        } else
        if(d < 1.82501E-008D)
        {
            complex1.re = complex.re;
            complex1.im = complex.im;
        } else
        if(d < 0.10000000000000001D)
        {
            Complex complex2 = multiply(complex, complex);
            for(int j = 0; j < 17; j++)
            {
                Complex complex3 = multiply(complex2, complex1);
                int k = 2 * (17 - j) - 1;
                complex1.re = 1.0D / (double)k - complex3.re;
                complex1.im = -complex3.im;
            }

            complex1 = multiply(complex1, complex);
        } else
        if(d < 9007200000000000D)
        {
            double d2 = d * d;
            complex1.re = 0.5D * Math.atan2(2D * complex.re, 1.0D - d2);
            complex1.im = 0.25D * Math.log((d2 + 2D * complex.im + 1.0D) / ((d2 - 2D * complex.im) + 1.0D));
        } else
        {
            complex1.re = complex.re >= 0.0D ? 1.5707963267948966D : -1.5707963267948966D;
        }
        return complex1;
    }

    public static Complex sinh(Complex complex)
    {
        double d = Hyperbolic.cosh(complex.re);
        double d1 = Hyperbolic.sinh(complex.re);
        double d2 = StrictMath.cos(complex.im);
        double d3 = StrictMath.sin(complex.im);
        boolean flag = Double.isInfinite(d);
        boolean flag1 = Double.isInfinite(complex.im);
        Complex complex1;
        if(complex.im == 0.0D)
        {
            complex1 = new Complex(Hyperbolic.sinh(complex.re));
        } else
        {
            complex1 = new Complex(d1 * d2, d * d3);
            if(flag1)
            {
                complex1.im = (0.0D / 0.0D);
                if(complex.re == 0.0D)
                    complex1.re = 0.0D;
            }
            if(flag)
            {
                complex1.re = complex.re * d2;
                complex1.im = complex.re * d3;
                if(complex.im == 0.0D)
                    complex1.im = 0.0D;
                if(flag1)
                    complex1.re = complex.im;
            }
        }
        return complex1;
    }

    public static Complex cosh(Complex complex)
    {
        if(complex.im == 0.0D)
            return new Complex(Hyperbolic.cosh(complex.re));
        double d = Hyperbolic.cosh(complex.re);
        double d1 = Hyperbolic.sinh(complex.re);
        double d2 = StrictMath.cos(complex.im);
        double d3 = StrictMath.sin(complex.im);
        boolean flag = Double.isInfinite(d);
        boolean flag1 = Double.isInfinite(complex.im);
        Complex complex1 = new Complex(d * d2, d1 * d3);
        if(flag1)
            complex1.re = (0.0D / 0.0D);
        if(complex.re == 0.0D)
            complex1.im = 0.0D;
        else
        if(flag)
        {
            complex1.re = complex.re * d2;
            complex1.im = complex.re * d3;
            if(complex.im == 0.0D)
                complex1.im = 0.0D;
            if(Double.isNaN(complex.im))
                complex1.re = complex.re;
            else
            if(flag1)
                complex1.re = complex.im;
        }
        return complex1;
    }

    public static Complex tanh(Complex complex)
    {
        double d = Hyperbolic.sinh(2D * complex.re);
        if(complex.im == 0.0D)
            return new Complex(Hyperbolic.tanh(complex.re));
        if(d == 0.0D)
            return new Complex(0.0D, Math.tan(complex.im));
        double d1 = Hyperbolic.cosh(2D * complex.re);
        double d2 = StrictMath.cos(2D * complex.im);
        double d3 = StrictMath.sin(2D * complex.im);
        boolean flag = Double.isInfinite(d1);
        if(flag)
        {
            return new Complex(complex.re <= 0.0D ? -1D : 1.0D);
        } else
        {
            double d4 = d1 + d2;
            return new Complex(d / d4, d3 / d4);
        }
    }

    public static Complex asinh(Complex complex)
    {
        Complex complex1 = new Complex(complex.im, -complex.re);
        Complex complex2 = asin(complex1);
        double d = complex2.im;
        complex2.im = complex2.re;
        complex2.re = -d;
        return complex2;
    }

    public static Complex acosh(Complex complex)
    {
        Complex complex1 = acos(complex);
        double d = -complex1.im;
        complex1.im = complex1.re;
        complex1.re = d;
        boolean flag = Double.doubleToLongBits(complex1.re) == 0x8000000000000000L;
        if(complex1.re < 0.0D || flag)
        {
            complex1.re = -complex1.re;
            complex1.im = -complex1.im;
        }
        return complex1;
    }

    public static Complex atanh(Complex complex)
    {
        Complex complex1 = new Complex(complex.im, -complex.re);
        Complex complex2 = atan(complex1);
        double d = complex2.im;
        complex2.im = complex2.re;
        complex2.re = -d;
        return complex2;
    }

    public static Complex pow(Complex complex, double d)
    {
        double d1 = abs(complex);
        Complex complex1 = new Complex();
        if(d1 == 0.0D)
        {
            complex1 = complex;
        } else
        {
            double d2 = argument(complex);
            double d3 = StrictMath.pow(d1, d);
            complex1.re = d3 * StrictMath.cos(d * d2);
            complex1.im = d3 * StrictMath.sin(d * d2);
        }
        return complex1;
    }

    public static Complex pow(Complex complex, Complex complex1)
    {
        return exp(multiply(complex1, log(complex)));
    }

    public String toString()
    {
        if(im == 0.0D)
            return String.valueOf(re);
        if(re == 0.0D)
        {
            return (new StringBuilder()).append(String.valueOf(im)).append(suffix).toString();
        } else
        {
            String s = im >= 0.0D ? "+" : "";
            return (new StringBuilder()).append(String.valueOf(re)).append(s).append(String.valueOf(im)).append(suffix).toString();
        }
    }

    public static Complex valueOf(String s)
        throws NumberFormatException
    {
        if(s == null || s.length() == 0)
            throw new NumberFormatException(s);
        String s1 = s.trim();
        int j = 0;
        Complex complex = new Complex();
        byte byte0 = 0;
        int k = 1;
        boolean flag = false;
        int l = s1.length();
        for(int i1 = 0; i1 < l; i1++)
        {
            char c = s1.charAt(i1);
            switch(c)
            {
            case 48: // '0'
            case 49: // '1'
            case 50: // '2'
            case 51: // '3'
            case 52: // '4'
            case 53: // '5'
            case 54: // '6'
            case 55: // '7'
            case 56: // '8'
            case 57: // '9'
                if(byte0 == 0 || byte0 == 1)
                {
                    byte0 = 2;
                    break;
                }
                if(byte0 == 4)
                    byte0 = 5;
                break;

            case 43: // '+'
            case 45: // '-'
                k = c != '+' ? -1 : 1;
                if(byte0 == 0)
                {
                    byte0 = 1;
                    break;
                }
                if(byte0 == 4)
                {
                    byte0 = 5;
                    break;
                }
                if(!flag)
                {
                    complex.re = Double.valueOf(s1.substring(j, i1)).doubleValue();
                    flag = true;
                    j = i1;
                    byte0 = 1;
                } else
                {
                    throw new NumberFormatException(s1);
                }
                break;

            case 46: // '.'
                if(byte0 == 0 || byte0 == 1 || byte0 == 2)
                    byte0 = 3;
                else
                    throw new NumberFormatException(s1);
                break;

            case 73: // 'I'
            case 74: // 'J'
            case 105: // 'i'
            case 106: // 'j'
                if(i1 + 1 != l)
                    throw new NumberFormatException(s1);
                if(byte0 == 0 || byte0 == 1)
                {
                    complex.im = k;
                    return complex;
                }
                if(byte0 == 2 || byte0 == 3 || byte0 == 5)
                {
                    complex.im = Double.valueOf(s1.substring(j, i1)).doubleValue();
                    return complex;
                } else
                {
                    throw new NumberFormatException(s1);
                }

            case 68: // 'D'
            case 69: // 'E'
            case 100: // 'd'
            case 101: // 'e'
                if(byte0 == 2 || byte0 == 3)
                    byte0 = 4;
                else
                    throw new NumberFormatException(s1);
                break;

            case 44: // ','
            case 47: // '/'
            case 58: // ':'
            case 59: // ';'
            case 60: // '<'
            case 61: // '='
            case 62: // '>'
            case 63: // '?'
            case 64: // '@'
            case 65: // 'A'
            case 66: // 'B'
            case 67: // 'C'
            case 70: // 'F'
            case 71: // 'G'
            case 72: // 'H'
            case 75: // 'K'
            case 76: // 'L'
            case 77: // 'M'
            case 78: // 'N'
            case 79: // 'O'
            case 80: // 'P'
            case 81: // 'Q'
            case 82: // 'R'
            case 83: // 'S'
            case 84: // 'T'
            case 85: // 'U'
            case 86: // 'V'
            case 87: // 'W'
            case 88: // 'X'
            case 89: // 'Y'
            case 90: // 'Z'
            case 91: // '['
            case 92: // '\\'
            case 93: // ']'
            case 94: // '^'
            case 95: // '_'
            case 96: // '`'
            case 97: // 'a'
            case 98: // 'b'
            case 99: // 'c'
            case 102: // 'f'
            case 103: // 'g'
            case 104: // 'h'
            default:
                throw new NumberFormatException(s1);
            }
        }

        if(!flag)
        {
            complex.re = Double.valueOf(s1).doubleValue();
            return complex;
        } else
        {
            throw new NumberFormatException(s1);
        }
    }



}