
package utility.forecasting;

import java.io.Serializable;
import java.util.Random;

// Referenced classes of package
//            Cdf, Sort, Random

public class Summary
    implements Serializable, Cloneable
{

	private static final long serialVersionUID = 0x7b634a600f90e0efL;
    private int nObs;
    private double sumx;
    private double sumx2;
    private double sumx3;
    private double sumx4;
    private double sumw;
    private double min;
    private double max;

    public Summary()
    {
        sumx = sumx2 = sumx3 = sumw = 0.0D;
        min = max = (0.0D / 0.0D);
    }

    public synchronized void update(double d)
    {
        if(!Double.isNaN(d))
        {
            double d1 = nObs != 0 ? sumx / sumw : 0.0D;
            sumx += d;
            double d2 = sumx / (double)(nObs + 1);
            double d3 = d - d2;
            double d4 = d3 * d3;
            double d5 = d2 - d1;
            double d6 = d5 * d5;
            sumx4 = sumx4 + ((d4 * d4 - 4D * d5 * sumx3) + 6D * d6 * sumx2 + (double)nObs * d6 * d6);
            sumx3 += d4 * d3 - 3D * d5 * sumx2 - (double)nObs * d6 * d5;
            sumx2 += d4 + (double)nObs * d6;
            sumw++;
            if(Double.isNaN(min))
            {
                min = max = d;
            } else
            {
                min = d >= min ? min : d;
                max = d <= max ? max : d;
            }
            nObs++;
        }
    }

    public synchronized void update(double d, double d1)
    {
        if(!Double.isNaN(d))
        {
            double d2 = nObs != 0 ? sumx / sumw : 0.0D;
            sumx += d1 * d;
            double d3 = sumx / (sumw + d1);
            double d4 = d - d3;
            double d5 = d4 * d4;
            double d6 = d3 - d2;
            double d7 = d6 * d6;
            sumx4 += (d1 * d5 * d5 - 4D * d6 * sumx3) + 6D * d7 * sumx2 + sumw * d7 * d7;
            sumx3 += d1 * d5 * d4 - 3D * d6 * sumx2 - sumw * d7 * d6;
            sumx2 += d1 * d5 + sumw * d7;
            sumw += d1;
            if(Double.isNaN(min))
            {
                min = max = d;
            } else
            {
                min = d >= min ? min : d;
                max = d <= max ? max : d;
            }
            nObs++;
        }
    }

    public synchronized void update(double ad[])
    {
        for(int i = 0; i < ad.length; i++)
            update(ad[i]);

    }

    public synchronized void update(double ad[], double ad1[])
    {
        for(int i = 0; i < ad.length; i++)
            update(ad[i], ad1[i]);

    }

    public double getMinimum()
    {
        return min;
    }

    public double getMaximum()
    {
        return max;
    }

    public double getMean()
    {
        return sumx / sumw;
    }

    public double getVariance()
    {
        return sumx2 / (double)nObs;
    }

    public double getSampleVariance()
    {
        return sumx2 / (double)(nObs - 1);
    }

    public double getStandardDeviation()
    {
        return Math.sqrt(getVariance());
    }

    public double getSampleStandardDeviation()
    {
        return Math.sqrt(getSampleVariance());
    }

    public double getSkewness()
    {
        double d = getVariance();
        return sumx3 / (double)nObs / (Math.sqrt(d) * d);
    }

    public double getKurtosis()
    {
        double d = getVariance();
        return sumx4 / (double)nObs / (d * d) - 3D;
    }

    public double[] confidenceMean(double d)
    {
        if(d < 0.0D || d > 1.0D)
        {
            Object aobj[] = {
                new String("p"), new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("OutOfRange"+ aobj);
        }
        double d1 = Cdf.inverseStudentsT(0.5D + 0.5D * d, nObs - 1);
        double d2 = getMean();
        double d3 = (d1 * getSampleStandardDeviation()) / Math.sqrt(nObs);
        double ad[] = {
            d2 - d3, d2 + d3
        };
        return ad;
    }

    public double[] confidenceVariance(double d)
    {
        if(d < 0.0D || d > 1.0D)
        {
            Object aobj[] = {
                new String("p"), new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("OutOfRange"+ aobj);
        }
        double d1 = nObs - 1;
        double d2 = Cdf.inverseChi(0.5D + 0.5D * d, d1);
        double d3 = Cdf.inverseChi(0.5D - 0.5D * d, d1);
        double d4 = getSampleVariance();
        double ad[] = {
            (d4 * d1) / d2, (d4 * d1) / d3
        };
        return ad;
    }

    public static double minimum(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = ad[0];
        for(int j = 1; j < i; j++)
            if(!Double.isNaN(ad[j]))
                d = Math.min(d, ad[j]);

        return d;
    }

    public static int minimum(int ai[])
    {
        int i = ai[0];
        for(int j = 1; j < ai.length; j++)
            i = Math.min(i, ai[j]);

        return i;
    }

    public static double maximum(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = ad[0];
        for(int j = 1; j < i; j++)
            if(!Double.isNaN(ad[j]))
                d = Math.max(d, ad[j]);

        return d;
    }

    public static int maximum(int ai[])
    {
        int i = ai[0];
        for(int j = 1; j < ai.length; j++)
            i = Math.max(i, ai[j]);

        return i;
    }

    public static double mean(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                d += ad[k];
                j++;
            }

        if(j == 0)
            return (0.0D / 0.0D);
        else
            return d / (double)j;
    }

    public static double mean(double ad[], double ad1[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = 0.0D;
        double d1 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                d += ad1[k] * ad[k];
                d1 += ad1[k];
                j++;
            }

        if(j == 0)
            return (0.0D / 0.0D);
        else
            return d / d1;
    }

    public static double variance(double ad[])
    {
        int i = ad.length;
        if(i < 2)
            return (0.0D / 0.0D);
        double d = mean(ad);
        double d1 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                d1 += (ad[k] - d) * (ad[k] - d);
                j++;
            }

        if(j < 2)
            return (0.0D / 0.0D);
        else
            return d1 / (double)j;
    }

    public static double variance(double ad[], double ad1[])
    {
        int i = ad.length;
        if(i < 2)
            return (0.0D / 0.0D);
        double d = mean(ad, ad1);
        double d1 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                d1 += ad1[k] * (ad[k] - d) * (ad[k] - d);
                j++;
            }

        if(j < 2)
            return (0.0D / 0.0D);
        else
            return d1 / (double)j;
    }

    public static double sampleVariance(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = mean(ad);
        double d1 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                d1 += (ad[k] - d) * (ad[k] - d);
                j++;
            }

        if(j == 0)
            return (0.0D / 0.0D);
        else
            return d1 / (double)(j - 1);
    }

    public static double sampleVariance(double ad[], double ad1[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = mean(ad, ad1);
        double d1 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                d1 += ad1[k] * (ad[k] - d) * (ad[k] - d);
                j++;
            }

        if(j == 0)
            return (0.0D / 0.0D);
        else
            return d1 / (double)(j - 1);
    }

    public static double standardDeviation(double ad[])
    {
        return Math.sqrt(variance(ad));
    }

    public static double standardDeviation(double ad[], double ad1[])
    {
        return Math.sqrt(variance(ad, ad1));
    }

    public static double sampleStandardDeviation(double ad[])
    {
        return Math.sqrt(sampleVariance(ad));
    }

    public static double sampleStandardDeviation(double ad[], double ad1[])
    {
        return Math.sqrt(sampleVariance(ad, ad1));
    }

    public static double skewness(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = mean(ad);
        double d1 = 0.0D;
        double d2 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                double d3 = ad[k] - d;
                double d4 = d3 * d3;
                d1 += d4;
                d2 += d3 * d4;
                j++;
            }

        if(j == 0)
        {
            return (0.0D / 0.0D);
        } else
        {
            d1 /= j;
            d2 /= j;
            return d2 / (Math.sqrt(d1) * d1);
        }
    }

    public static double skewness(double ad[], double ad1[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = mean(ad, ad1);
        double d1 = 0.0D;
        double d2 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                double d3 = ad[k] - d;
                double d4 = d3 * d3;
                d1 += ad1[k] * d4;
                d2 += ad1[k] * d3 * d4;
                j++;
            }

        if(j == 0)
        {
            return (0.0D / 0.0D);
        } else
        {
            d1 /= j;
            d2 /= j;
            return d2 / (Math.sqrt(d1) * d1);
        }
    }

    public static double kurtosis(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = mean(ad);
        double d1 = 0.0D;
        double d2 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                double d3 = ad[k] - d;
                double d4 = d3 * d3;
                d1 += d4;
                d2 += d4 * d4;
                j++;
            }

        if(j == 0)
        {
            return (0.0D / 0.0D);
        } else
        {
            d1 /= j;
            d2 /= j;
            return d2 / (d1 * d1) - 3D;
        }
    }

    public static double kurtosis(double ad[], double ad1[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double d = mean(ad, ad1);
        double d1 = 0.0D;
        double d2 = 0.0D;
        int j = 0;
        for(int k = 0; k < i; k++)
            if(!Double.isNaN(ad[k]))
            {
                double d3 = ad[k] - d;
                double d4 = d3 * d3;
                d1 += ad1[k] * d4;
                d2 += ad1[k] * d4 * d4;
                j++;
            }

        if(j == 0)
        {
            return (0.0D / 0.0D);
        } else
        {
            d1 /= j;
            d2 /= j;
            return d2 / (d1 * d1) - 3D;
        }
    }

    public static double median(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double ad1[] = new double[ad.length];
        System.arraycopy(ad, 0, ad1, 0, ad.length);
        Sort.ascending(ad1);
        if(i == 0)
            return (0.0D / 0.0D);
        if(i % 2 == 1)
            return ad1[i / 2];
        else
            return 0.5D * (ad1[i / 2 - 1] + ad1[i / 2]);
    }

    public static double mode(double ad[])
    {
        int i = ad.length;
        if(i == 0)
            return (0.0D / 0.0D);
        double ad1[] = new double[ad.length];
        System.arraycopy(ad, 0, ad1, 0, ad.length);
        Sort.ascending(ad1);
        Random random = new Random();
        double d = ad1[0];
        int j = 0;
        int k = 1;
        double d1 = ad1[0];
        for(int l = 1; l < i; l++)
            if(ad1[l] == d1)
            {
                if(++k == j)
                {
                    if(random.nextFloat() > 0.5F)
                        d = d1;
                    continue;
                }
                if(k > j)
                {
                    d = d1;
                    j = k;
                }
            } else
            {
                k = 1;
                d1 = ad1[l];
            }

        return d;
    }



}
