package learning.benchmarks;


import java.util.LinkedList;
import java.util.Random;
import utility.Utils;

public class BanditAlone extends OperatorSelection {

	protected LinkedList<Option> lst_option;
	private boolean normPchapeau;
	private double best_pchapeau;

	public BanditAlone(int N_operators, boolean User, double v_Scaling, int _normalize, int _W) {
		super(N_operators, User);
		normPchapeau = false;
		if (_normalize > 1) {
			normPchapeau = true; //0: no normalization; 1: normalize reward; 2: normalize pchapeau; 3: normalize reward and pchapeau.
		}
		lst_option = new LinkedList<Option>();
		if (_W==0) {//if MAB or DMAB
			for (int i = 0; i < n_operators; i++) {
				Option op = new Option(i, v_Scaling);
				lst_option.add(op);
			}
		} else {//if SlMAB or SlMAB+
			for (int i = 0; i < n_operators; i++) {
				DynOption op= new DynOption(i, v_Scaling, _W);
				lst_option.add(op) ;
			}
		}
		best_pchapeau = 0;
		//counter=0;
	}

	void reset() {
		// re-initialize the Bandit
		best_pchapeau = 0;
		times = 0;
		best_op = 0;
		for (int i = 0; i < n_operators; i++) {
			lst_option.get(i).initialize();
		}
	}

	@Override
	public double applyReward(int op) {
		//check if this operator was chosen based on \hat{p} or on sqrt{n}
		double reward = getReward(op);
		times++;
		if (credit.get(0).getType() < 3 || times > n_operators) {
			if (best_op == op) {
				exploit++;
			} else {
				explore++;
			}
			lst_option.get(op).applyReward(reward, times, true);
			updateBestOp(op);
			if (normPchapeau) {
				updateNormPchapeau();
			}
		} else {
			lst_option.get(op).setTried(true);
			explore++;
		}

		/*cout << times << " " << best_op << " " << best_pchapeau;
		for (int i=0; i<n_operators; i++)
		cout << " " << lst_option[i]->getExploitation();
		cout << endl;*/

		return reward;
	}

	public void updateBestOp(int op) {
		double exploit_op = lst_option.get(op).getExploitation();
		//if the one that was the best has a lower \hat p, check between all; or if the previous best changed
		if ((best_op == op && exploit_op < best_pchapeau) || (best_op >= 0 && lst_option.get(best_op).getExploitation() != best_pchapeau)) {
			best_pchapeau = -1;
			best_op = 0;
			double exploit_aux = -1;
			for (int i = 0; i < n_operators; i++) {
				exploit_aux = lst_option.get(i).getExploitation();
				if (exploit_aux > best_pchapeau) {
					best_pchapeau = exploit_aux;
					best_op = i;
				}
			}
		}
		else if (exploit_op >= best_pchapeau) { //else if it is best than the previous best
			best_op = op;
			best_pchapeau = exploit_op;
		}
	}

	public void updateNormPchapeau() {
		double divide = 1.0;
		if (best_pchapeau > 0) {
			divide = best_pchapeau;
		}
		if (divide != lst_option.get(0).getNormPchapeau()) {
			for (int i = 0; i < n_operators; i++) {
				lst_option.get(i).setNormPchapeau(divide);
			}
		}
	}

	int optionNotTried() {
		Random rand = Utils.random;
		int i = (int) (rand.nextDouble() * n_operators);
		for (int j = 0; j < n_operators; j++) {
			int k = (i + j) % n_operators;
			//if (lst_option[k]->getTimes() == 0)
			if (!lst_option.get(k).getTried()) {
				return k;
			}
		}
		return 0;
	}

	@Override
	public int selectOperator(double times_total)
	{
		if (times < n_operators) {
			//cout << 1 << endl;
			return optionNotTried();
		} else {
			/*if(credit[0]->getType()>=3 && !credit[0]->checkFullQueue()){
			counter++;
			return (int) (rng.uniform() * n_operators);
			}else{*/

			//delayed reward for the ranking-based after 1x for each operator
			if (credit.get(0).getType() >= 3 && times == n_operators) {
				//cout << 2 << endl;
				for (int i = 0; i < n_operators; i++) {
					lst_option.get(i).applyReward(credit.get(0).getReward(false), times, true);
					updateBestOp(i);
				}
				if (normPchapeau) {
					updateNormPchapeau();
				}
			}//else cout << 3 << endl;

			int op = 0;
			double score_max = 0, vscore = 0;

			for (int i = 0; i < n_operators; i++) {
				vscore = lst_option.get(i).score(times_total);
				if (vscore > score_max) {
					score_max = vscore;
					op = i;
				}
			}
			return op;
			//}
		}
	}

	int selectOperator() {
		return selectOperator((double) times);
	}

}
